@@ -3,7 +3,7 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-22
+ *  Updated   : 2003-08-10
  *  Notes     :
  *
  * Copyright (c) 1997-2003 Iain Lea <iain@bricbrac.de>
@@ -55,10 +55,10 @@
 	extern int _flsbuf(int, FILE *);
 #endif /* DECL__FLSBUF */
 #ifdef DECL_ATOI
-	extern int atoi(char *);
+	extern int atoi(const char *);
 #endif /* DECL_ATOI */
 #ifdef DECL_ATOL
-	extern long atol(char *);
+	extern long atol(const char *);
 #endif /* DECL_ATOL */
 #ifndef HAVE_MEMCPY
 #	ifdef DECL_BCOPY
@@ -121,7 +121,7 @@
 	extern struct hostent *gethostbyname(const char *);
 #endif /* DECL_GETHOSTBYNAME */
 #ifdef DECL_GETHOSTNAME
-	extern int gethostname(char *, int);
+	extern int gethostname(char *, size_t);
 #endif /* DECL_GETHOSTNAME */
 #ifdef DECL_GETLOGIN
 	extern char *getlogin(void);
@@ -185,7 +185,7 @@
 	extern int printf(const char *, ...);
 #endif /* DECL_PRINTF */
 #ifdef DECL_PUTENV
-	extern int putenv(const char *);
+	extern int putenv(char *);
 #endif /* DECL_PUTENV */
 #ifdef DECL_QSORT
 	extern void qsort(void *, size_t, size_t, int (*)(t_comptype*, t_comptype*));
@@ -233,7 +233,7 @@
 	extern long strtol(const char *, char **, int);
 #endif /* DECL_STRTOL */
 #ifdef DECL_SYSTEM
-	extern int system(char *);
+	extern int system(const char *);
 #endif /* DECL_SYSTEM */
 #ifdef DECL_TEMPNAM
 	extern char *tempnam(const char *, const char *);
@@ -290,9 +290,11 @@
 #ifdef DECL_VSNPRINTF
 	extern int vsnprintf(char *, size_t, const char *, va_list);
 #endif /* DECL_VSNPRINTF */
+#if 0 /* some (most?) systems have "int vsprintf(char *, const char *, va_list)" */
 #ifdef DECL_VSPRINTF
 	extern int vsprintf(char *, char *, va_list);
 #endif /* DECL_VSPRINTF */
+#endif /* 0 */
 
 
 extern int optind;
@@ -327,8 +329,8 @@ extern char **news_headers_to_not_display_array;
 extern char *OPT_CHAR_list[];
 extern char *OPT_STRING_list[];
 extern char *ch_post_process;
-extern char *glob_group;
 extern char *nntp_server;
+extern char *xover_cmd;
 extern char active_times_file[PATH_LEN];
 extern char bug_addr[LEN];
 extern char bug_nntpserver1[PATH_LEN];
@@ -339,6 +341,7 @@ extern char dead_article[PATH_LEN];
 extern char dead_articles[PATH_LEN];
 extern char default_organization[PATH_LEN];
 extern char default_signature[PATH_LEN];
+extern char global_attributes_file[PATH_LEN];
 extern char global_config_file[PATH_LEN];
 extern char homedir[PATH_LEN];
 extern char index_maildir[PATH_LEN];
@@ -372,9 +375,7 @@ extern char newsrc[PATH_LEN];
 #endif /* !NNTP_ONLY */
 extern char page_header[LEN];
 extern char posted_info_file[PATH_LEN];
-extern char posted_msgs_file[PATH_LEN];
 extern char postponed_articles_file[PATH_LEN];
-extern char proc_ch_default;
 extern char tin_progname[PATH_LEN];
 extern char rcdir[PATH_LEN];
 extern char save_active_file[PATH_LEN];
@@ -388,30 +389,29 @@ extern char host_name[];
 
 extern const char base64_alphabet[64];
 
-extern const char **info_help;
 extern constext *content_encodings[];
 extern constext *content_types[];
-extern constext *help_group[];
-extern constext *help_page[];
-extern constext *help_select[];
-extern constext *help_thread[];
+extern constext *txt_attrs[];
 extern constext *txt_colors[];
 extern constext *txt_confirm_choices[];
+extern constext *txt_hide_uue_type[];
 extern constext *txt_kill_level_type[];
 #ifdef CHARSET_CONVERSION
-	extern constext *txt_mime_charsets[];
+	extern constext *txt_mime_charsets[];	/* supported charsets */
 #endif /* CHARSET_CONVERSION */
+extern constext *txt_mime_7bit_charsets[]; /* 7bit charsets */
 extern constext *txt_mailbox_formats[];
 extern constext *txt_marks[];
-extern constext *txt_mime_encodings[NUM_MIME_ENCODINGS];
+extern constext *txt_mime_encodings[];
 extern constext *txt_onoff[];
 extern constext *txt_post_process_type[];
 extern constext *txt_quote_style_type[];
 extern constext *txt_show_from[];
+extern constext *txt_show_info_type[];
 extern constext *txt_sort_a_type[];	/* a=articles */
 extern constext *txt_sort_t_type[];	/* t=threads */
 extern constext *txt_strip_bogus_type[];
-extern constext *txt_thread[];
+extern constext *txt_threading[];
 extern constext *txt_thread_score_type[];
 extern constext *txt_wildcard_type[];
 extern constext txt_1_resp[];
@@ -438,11 +438,14 @@ extern constext txt_article_cancelled[];
 extern constext txt_article_plural[];
 extern constext txt_article_reposted[];
 extern constext txt_article_singular[];
+extern constext txt_article_upper[];
 extern constext txt_articles_mailed[];
 #ifndef DISABLE_PRINTING
 	extern constext txt_articles_printed[];
 #endif /* !DISABLE_PRINTING */
 extern constext txt_attach[];
+extern constext txt_attach_charset[];
+extern constext txt_attach_description[];
 extern constext txt_uue[];
 extern constext txt_at_s[];
 extern constext txt_auth_failed[];
@@ -461,24 +464,29 @@ extern constext txt_bad_article[];
 extern constext txt_bad_attrib[];
 extern constext txt_bad_command[];
 extern constext txt_base64[];
+extern constext txt_base_article[];
+extern constext txt_base_article_range[];
 extern constext txt_batch_update_unavail[];
 extern constext txt_begin_of_art[];
+extern constext txt_caching_off[];
+extern constext txt_caching_on[];
 extern constext txt_cancel_article[];
 extern constext txt_cancelling_art[];
 extern constext txt_cannot_create[];
 extern constext txt_cannot_create_uniq_name[];
 extern constext txt_cannot_find_base_art[];
 extern constext txt_cannot_get_nntp_server_name[];
-#ifndef USE_CURSES
+#if defined(M_UNIX) && !defined(USE_TERMINFO)
 	extern constext txt_cannot_get_term_entry[];
-#endif /* !USE_CURSES */
+#endif /* M_UNIX && !USE_TERMINFO */
 extern constext txt_cannot_open[];
 extern constext txt_cannot_open_for_saving[];
 extern constext txt_cannot_post[];
 extern constext txt_cannot_post_group[];
 extern constext txt_cannot_retrieve[];
-extern constext txt_cannot_write_index[];
 extern constext txt_cannot_write_to_directory[];
+extern constext txt_catchup[];
+extern constext txt_catchup_group[];
 extern constext txt_catchup_all_read_groups[];
 extern constext txt_catchup_despite_tags[];
 extern constext txt_catchup_update_info[];
@@ -494,8 +502,13 @@ extern constext txt_color_off[];
 extern constext txt_color_on[];
 extern constext txt_command_failed[];
 extern constext txt_confirm_select_on_exit[];
-extern constext txt_connecting[];
-extern constext txt_connecting_port[];
+#ifdef NNTP_ABLE
+	extern constext txt_connecting[];
+	extern constext txt_connecting_port[];
+#endif /* NNTP_ABLE */
+#if defined(NNTP_ABLE) && !defined(INET6)
+	extern constext txt_connection_to[];
+#endif /* NNTP_ABLE && !INET6 */
 extern constext txt_copyright_notice[];
 extern constext txt_cr[];
 extern constext txt_creating_active[];
@@ -503,7 +516,9 @@ extern constext txt_creating_newsrc[];
 extern constext txt_default[];
 extern constext txt_delete_processed_files[];
 extern constext txt_deleting[];
-extern constext txt_disconnecting[];
+#ifdef NNTP_ABLE
+	extern constext txt_disconnecting[];
+#endif /* NNTP_ABLE */
 extern constext txt_end_of_art[];
 extern constext txt_end_of_arts[];
 extern constext txt_end_of_groups[];
@@ -511,6 +526,7 @@ extern constext txt_end_of_thread[];
 extern constext txt_enter_getart_limit[];
 extern constext txt_enter_message_id[];
 extern constext txt_enter_next_thread[];
+extern constext txt_enter_next_unread_group[];
 extern constext txt_enter_option_num[];
 extern constext txt_enter_range[];
 extern constext txt_error_approved[];
@@ -519,8 +535,12 @@ extern constext txt_error_bad_approved[];
 extern constext txt_error_bad_from[];
 extern constext txt_error_bad_replyto[];
 extern constext txt_error_bad_msgidfqdn[];
+extern constext txt_error_cant_unlock[];
 extern constext txt_error_copy_fp[];
 extern constext txt_error_corrupted_file[];
+extern constext txt_error_couldnt_dotlock[];
+extern constext txt_error_couldnt_lock[];
+extern constext txt_error_fseek[];
 extern constext txt_error_gnksa_internal[];
 extern constext txt_error_gnksa_langle[];
 extern constext txt_error_gnksa_lparen[];
@@ -554,12 +574,13 @@ extern constext txt_error_header_line_bad_encoding[];
 extern constext txt_error_header_line_blank[];
 extern constext txt_error_header_line_colon[];
 extern constext txt_error_header_line_empty[];
-#ifndef FOLLOW_USEFOR_DRAFT
-	extern constext txt_error_header_line_comma[];
-	extern constext txt_error_header_line_groups_contd[];
-#endif /* !FOLLOW_USEFOR_DRAFT */
 extern constext txt_error_header_line_missing[];
 extern constext txt_error_header_line_space[];
+extern constext txt_error_insecure_permissions[];
+extern constext txt_error_invalid_response_to_group[];
+extern constext txt_error_locale[];
+extern constext txt_error_mime_end[];
+extern constext txt_error_mime_start[];
 extern constext txt_error_no_domain_name[];
 extern constext txt_error_no_enter_permission[];
 extern constext txt_error_no_from[];
@@ -569,16 +590,33 @@ extern constext txt_error_no_write_permission[];
 extern constext txt_error_passwd_missing[];
 extern constext txt_error_plural[];
 extern constext txt_error_sender_in_header_not_allowed[];
+extern constext txt_error_server_has_no_listed_groups[];
 extern constext txt_error_singular[];
+#if defined(NNTP_ABLE) && defined(INET6)
+	extern constext txt_error_socket_or_connect_problem[];
+#endif /* NNTP_ABLE && INET6 */
+#if defined(NNTP_ABLE) && defined(TLI) && !defined(INET6)
+	extern constext txt_error_server_unavailable[];
+	extern constext txt_error_topen[];
+#endif /* NNTP_ABLE && TLI && !INET6 */
 extern constext txt_error_unknown_dlevel[];
+#if defined(NNTP_ABLE) && defined(HAVE_GETSERVBYNAME)
+	extern constext txt_error_unknown_service[];
+#endif /* NNTP_ABLE && HAVE_GETSERVBYNAME */
+#ifdef NNTP_ABLE
+	extern constext txt_error_wrong_newsgroupname_in_group_response[];
+#endif /* NNTP_ABLE */
 extern constext txt_esc[];
 extern constext txt_exiting[];
 extern constext txt_external_mail_done[];
 extern constext txt_extracting_shar[];
-extern constext txt_failed_to_connect_to_server[];
+#ifdef NNTP_ABLE
+	extern constext txt_failed_to_connect_to_server[];
+#endif /* NNTP_ABLE */
 extern constext txt_feed_pattern[];
 extern constext txt_filesystem_full[];
 extern constext txt_filesystem_full_backup[];
+extern constext txt_filter_comment[];
 extern constext txt_filter_file[];
 extern constext txt_filter_global_rules[];
 extern constext txt_filter_rule_created[];
@@ -591,6 +629,9 @@ extern constext txt_from_line_only[];
 extern constext txt_from_line_only_case[];
 extern constext txt_full[];
 extern constext txt_gethostbyname[];
+#if defined(NNTP_ABLE) && !defined(INET6)
+	extern constext txt_giving_up[];
+#endif /* NNTP_ABLE && !INET6 */
 extern constext txt_global[];
 extern constext txt_group[];
 extern constext txt_group_aliased[];
@@ -674,7 +715,7 @@ extern constext txt_help_global_previous_menu[];
 extern constext txt_help_global_quit_tin[];
 extern constext txt_help_global_redraw_screen[];
 extern constext txt_help_global_save[];
-extern constext txt_help_global_save_tagged[];
+extern constext txt_help_global_auto_save[];
 extern constext txt_help_global_search_auth_backwards[];
 extern constext txt_help_global_search_auth_forwards[];
 extern constext txt_help_global_search_body[];
@@ -764,13 +805,19 @@ extern constext txt_help_title_disp[];
 extern constext txt_help_title_misc[];
 extern constext txt_help_title_navi[];
 extern constext txt_help_title_ops[];
+extern constext txt_incomplete[];
 extern constext txt_index_page_com[];
 extern constext txt_info_add_kill[];
 extern constext txt_info_add_select[];
+extern constext txt_info_all_parts_tagged[];
 extern constext txt_info_do_postpone[];
+extern constext txt_info_enter_valid_character[];
+extern constext txt_info_missing_part[];
 extern constext txt_info_nopostponed[];
+extern constext txt_info_not_multipart_message[];
 extern constext txt_info_not_subscribed[];
 extern constext txt_info_no_write[];
+extern constext txt_info_no_previous_expression[];
 extern constext txt_info_postponed[];
 extern constext txt_info_x_conversion_note[];
 extern constext txt_invalid_from[];
@@ -798,16 +845,21 @@ extern constext txt_last[];
 extern constext txt_last_resp[];
 extern constext txt_lines[];
 extern constext txt_mail[];
+extern constext txt_mailbox[];
 extern constext txt_mail_art_to[];
 extern constext txt_mail_log_to[];
 extern constext txt_mail_bug_report[];
 extern constext txt_mail_bug_report_confirm[];
 extern constext txt_mailed[];
 extern constext txt_mailing_to[];
+extern constext txt_mail_save_active_head[];
 extern constext txt_mark_arts_read[];
+extern constext txt_mark_art_read_tagged_current[];
 extern constext txt_mark_group_read[];
 extern constext txt_mark_thread_read[];
+extern constext txt_mark_thread_read_tagged_current[];
 extern constext txt_marked_as_unread[];
+extern constext txt_marked_tagged_arts_as_read[];
 extern constext txt_matching_cmd_line_groups[];
 extern constext txt_mini_group_1[];
 extern constext txt_mini_group_2[];
@@ -825,6 +877,7 @@ extern constext txt_mini_thread_2[];
 extern constext txt_more[];
 extern constext txt_moving[];
 extern constext txt_msgid_line_only[];
+extern constext txt_name[];
 extern constext txt_newsgroup[];
 extern constext txt_newsgroup_plural[];
 extern constext txt_newsgroup_position[];
@@ -844,6 +897,7 @@ extern constext txt_no_groups[];
 extern constext txt_no_groups_to_read[];
 extern constext txt_no_last_message[];
 extern constext txt_no_mail_address[];
+extern constext txt_no_marked_arts[];
 extern constext txt_no_match[];
 extern constext txt_no_more_groups[];
 extern constext txt_no_newsgroups[];
@@ -855,7 +909,6 @@ extern constext txt_no_responses[];
 extern constext txt_no_resps_in_thread[];
 extern constext txt_no_search_string[];
 extern constext txt_no_subject[];
-extern constext txt_no_tagged_arts_to_save[];
 extern constext txt_no_term_clear_eol[];
 extern constext txt_no_term_clear_eos[];
 extern constext txt_no_term_clearscreen[];
@@ -889,6 +942,9 @@ extern constext txt_post_subject[];
 extern constext txt_posted_info_file[];
 extern constext txt_posting[];
 extern constext txt_postpone_repost[];
+extern constext txt_prefix_hot[];
+extern constext txt_prefix_tagged[];
+extern constext txt_prefix_untagged[];
 extern constext txt_prompt_fup_ignore[];
 extern constext txt_prompt_unchanged_mail[];
 extern constext txt_prompt_see_postponed[];
@@ -921,7 +977,9 @@ extern constext txt_reading_news_active_file[];
 extern constext txt_reading_news_newsrc_file[];
 extern constext txt_reading_newsgroups_file[];
 extern constext txt_reading_newsrc[];
-extern constext txt_reconnect_to_news_server[];
+#ifdef NNTP_ABLE
+	extern constext txt_reconnect_to_news_server[];
+#endif /* NNTP_ABLE */
 extern constext txt_remove_bogus[];
 extern constext txt_rename_error[];
 extern constext txt_reply_to_author[];
@@ -942,6 +1000,8 @@ extern constext txt_saved_group[];
 extern constext txt_saved_groupname[];
 extern constext txt_saved_nothing[];
 extern constext txt_saved_summary[];
+extern constext txt_saved_to[];
+extern constext txt_saved_to_range[];
 extern constext txt_saving[];
 extern constext txt_screen_init_failed[];
 extern constext txt_screen_too_small[];
@@ -952,7 +1012,6 @@ extern constext txt_search_forwards[];
 extern constext txt_searching[];
 extern constext txt_searching_body[];
 extern constext txt_select_art[];
-extern constext txt_select_comment[];
 extern constext txt_select_config_file_option[];
 extern constext txt_select_from[];
 extern constext txt_select_group[];
@@ -965,11 +1024,17 @@ extern constext txt_select_subj[];
 extern constext txt_select_text[];
 extern constext txt_select_thread[];
 extern constext txt_select_time[];
-extern constext txt_server_name_in_file_env_var[];
+extern constext txt_send_bugreport[];
+#ifdef NNTP_ABLE
+	extern constext txt_server_name_in_file_env_var[];
+#endif /* NNTP_ABLE */
+extern constext txt_serverconfig_header[];
 extern constext txt_servers_active[];
 extern constext txt_skipping_newgroups[];
 extern constext txt_space[];
 extern constext txt_starting_command[];
+extern constext txt_stp_list_thread[];
+extern constext txt_stp_thread[];
 extern constext txt_subj_line_only[];
 extern constext txt_subj_line_only_case[];
 extern constext txt_subscribe_pattern[];
@@ -981,12 +1046,14 @@ extern constext txt_supersede_group[];
 extern constext txt_superseding_art[];
 extern constext txt_suspended_message[];
 extern constext txt_tab[];
-extern constext txt_tagged_art[];
-extern constext txt_tagged_thread[];
 extern constext txt_there_is_no_news[];
+extern constext txt_thread_upper[];
 extern constext txt_thread_com[];
 extern constext txt_thread_marked_as_deselected[];
 extern constext txt_thread_marked_as_selected[];
+extern constext txt_thread_plural[];
+extern constext txt_thread_range[];
+extern constext txt_thread_singular[];
 extern constext txt_thread_x_of_n[];
 extern constext txt_threading_arts[];
 extern constext txt_time_default_days[];
@@ -999,6 +1066,11 @@ extern constext txt_toggled_high[];
 extern constext txt_toggled_rot13[];
 extern constext txt_toggled_tex2iso[];
 extern constext txt_toggled_tabwidth[];
+#if defined(NNTP_ABLE) && defined(HAVE_INET_NTOA)
+	extern constext txt_trying[];
+#endif /* NNTP_ABLE && HAVE_INET_NTOA */
+extern constext txt_trying_dotlock[];
+extern constext txt_trying_lock[];
 extern constext txt_type_h_for_help[];
 extern constext txt_unlimited_time[];
 extern constext txt_unread[];
@@ -1006,30 +1078,78 @@ extern constext txt_unsubscribe_pattern[];
 extern constext txt_unsubscribed_num_groups[];
 extern constext txt_unsubscribed_to[];
 extern constext txt_unsubscribing[];
-extern constext txt_untagged_art[];
-extern constext txt_untagged_thread[];
 extern constext txt_unthreading_arts[];
 extern constext txt_updated[];
+extern constext txt_updating[];
 extern constext txt_url_open[];
 extern constext txt_url_done[];
+extern constext txt_usage_catchup[];
+extern constext txt_usage_check_for_unread_news[];
+#ifdef DEBUG
+	extern constext txt_usage_debug[];
+#endif /* DEBUG */
+extern constext txt_usage_dont_check_new_newsgroups[];
+extern constext txt_usage_dont_save_files_on_quit[];
+extern constext txt_usage_dont_show_descriptions[];
+#ifdef NNTP_ABLE
+	extern constext txt_usage_force_authentication[];
+	extern constext txt_usage_newsserver[];
+	extern constext txt_usage_port[];
+#endif /* NNTP_ABLE */
+extern constext txt_usage_getart_limit[];
+extern constext txt_usage_help_information[];
+extern constext txt_usage_help_message[];
+extern constext txt_usage_index_newsdir[];
+extern constext txt_usage_update_index_files[];
+extern constext txt_usage_maildir[];
+extern constext txt_usage_mail_bugreport[];
+extern constext txt_usage_mail_new_news[];
+extern constext txt_usage_mail_new_news_to_user[];
+extern constext txt_usage_newsrc_file[];
+extern constext txt_usage_no_posting[];
+extern constext txt_usage_post_article[];
+extern constext txt_usage_post_postponed_arts[];
+extern constext txt_usage_read_news_remotely[];
+extern constext txt_usage_read_saved_news[];
+extern constext txt_usage_savedir[];
+extern constext txt_usage_save_new_news[];
+extern constext txt_usage_start_if_unread_news[];
+extern constext txt_usage_tin[];
+#ifdef HAVE_COLOR
+	extern constext txt_usage_toggle_color[];
+#endif /* HAVE_COLOR */
+extern constext txt_usage_quickstart[];
+extern constext txt_usage_read_only_subscribed[];
+extern constext txt_usage_read_only_active[];
+extern constext txt_usage_verbose[];
+extern constext txt_usage_version[];
+extern constext txt_useful_without_batch_mode[];
+extern constext txt_useful_with_batch_mode[];
+extern constext txt_use_mime[];
 extern constext txt_value_out_of_range[];
 extern constext txt_view_attachment[];
 extern constext txt_warn_art_line_too_long[];
 extern constext txt_warn_article_unchanged[];
 extern constext txt_warn_blank_subject[];
 extern constext txt_warn_cancel[];
+#ifdef CHARSET_CONVERSION
+	extern constext txt_warn_charset_conversion[];
+#endif /* CHARSET_CONVERSION */
+extern constext txt_warn_downgrade[];
 extern constext txt_warn_encoding_and_external_inews[];
 #ifdef FOLLOW_USEFOR_DRAFT
 	extern constext txt_warn_header_line_comma[];
 	extern constext txt_warn_header_line_groups_contd[];
 #endif /* FOLLOW_USEFOR_DRAFT */
-extern constext txt_warn_newsrc[];
 extern constext txt_warn_multiple_sigs[];
+extern constext txt_warn_newsrc[];
+extern constext txt_warn_not_all_arts_saved[];
 extern constext txt_warn_re_but_no_references[];
 extern constext txt_warn_references_but_no_re[];
 extern constext txt_warn_sig_too_long[];
 extern constext txt_warn_suspicious_mail[];
 extern constext txt_warn_update[];
+extern constext txt_warn_unrecognized_version[];
 extern constext txt_warn_wrong_sig_format[];
 extern constext txt_warn_xref_not_supported[];
 extern constext txt_writing_attributes_file[];
@@ -1073,6 +1193,7 @@ extern constext txt_uu_success[];
 #endif /* NNTP_ABLE && !NNTP_ONLY */
 
 #ifndef NO_SHELL_ESCAPE
+	extern constext txt_shell_command[];
 	extern constext txt_shell_escape[];
 #endif /* !NO_SHELL_ESCAPE */
 
@@ -1091,17 +1212,13 @@ extern int iso2asc_supported;
 extern int last_resp;
 extern int max_active;
 extern int max_art;
-extern int max_from;
 extern int max_newnews;
 extern int max_save;
 extern int num_headers_to_display;
 extern int num_headers_to_not_display;
-extern int max_subj;
 extern int need_resize;
 extern int num_active;
 extern int num_newnews;
-extern int num_of_killed_arts;
-extern int num_of_selected_arts;
 extern int num_of_tagged_arts;
 extern int num_save;
 extern int signal_context;
@@ -1142,9 +1259,11 @@ extern struct regex_cache strokes_regex;
 #endif /* HAVE_COLOR */
 
 extern struct t_article *arts;
+extern struct t_attribute glob_attributes;
 extern struct t_config tinrc;
 extern struct t_filters glob_filter;
 extern struct t_group *active;
+extern struct t_group *curr_group;
 extern struct t_newnews *newnews;
 extern struct t_option option_table[];
 extern struct t_save *save;
@@ -1156,6 +1275,7 @@ extern t_bool created_rcdir;
 extern t_bool dangerous_signal_exit; /* TRUE if SIGHUP, SIGTERM, SIGUSR1 */
 extern t_bool disable_gnksa_domain_check;
 extern t_bool disable_sender;
+extern t_bool force_no_post;
 extern t_bool force_reread_active_file;
 extern t_bool got_sig_pipe;
 extern t_bool have_linescroll;
@@ -1165,6 +1285,7 @@ extern t_bool newsrc_active;
 extern t_bool no_write;
 extern t_bool post_article_and_exit;
 extern t_bool post_postponed_and_exit;
+extern t_bool quitting;
 extern t_bool read_local_newsgroups_file;
 extern t_bool read_news_via_nntp;
 extern t_bool read_saved_news;
@@ -1172,10 +1293,8 @@ extern t_bool reconnected_in_last_get_server;
 extern t_bool reread_active_for_posted_arts;
 extern t_bool show_description;
 extern t_bool show_subject;
-extern t_bool tex2iso_supported;
 extern t_bool batch_mode;
 extern t_bool verbose;
-extern t_bool xover_supported;
 extern t_bool xref_supported;
 
 extern t_menu selmenu;
@@ -1283,10 +1402,11 @@ extern t_bool word_highlight;
 	extern constext txt_error_not_valid_newsgroup[];
 #endif /* HAVE_FASCIST_NEWSADMIN */
 
-#ifdef HAVE_METAMAIL
-	extern constext txt_use_mime[];
-	extern constext txt_error_metamail_failed[];
-#endif /* HAVE_METAMAIL */
+
+#ifndef FOLLOW_USEFOR_DRAFT
+	extern constext txt_error_header_line_comma[];
+	extern constext txt_error_header_line_groups_contd[];
+#endif /* !FOLLOW_USEFOR_DRAFT */
 
 #ifdef HAVE_PGP_GPG
 	extern constext txt_pgp_add[];
@@ -1311,13 +1431,9 @@ extern constext txt_processing_saved_arts[];
 	extern constext txt_reading_mailgroups_file[];
 #endif /* HAVE_MH_MAIL_HANDLING */
 
-#ifdef LOCAL_CHARSET
-	extern constext txt_tinrc_local_charset[];
-	extern t_bool use_local_charset;
-#endif /* LOCAL_CHARSET */
-
 #ifdef M_AMIGA
 	extern constext txt_env_var_not_found[];
+	extern constext txt_usage_bbs_mode[];
 	extern t_bool tin_bbs_mode;
 #endif /* M_AMIGA */
 
@@ -1347,6 +1463,7 @@ extern struct opttxt txt_art_marked_recent;
 extern struct opttxt txt_art_marked_return;
 extern struct opttxt txt_art_marked_selected;
 extern struct opttxt txt_art_marked_unread;
+extern struct opttxt txt_ask_for_metamail;
 extern struct opttxt txt_auto_bcc;
 extern struct opttxt txt_auto_cc;
 extern struct opttxt txt_auto_list_thread;
@@ -1373,8 +1490,6 @@ extern struct opttxt txt_hide_uue;
 extern struct opttxt txt_inews_prog;
 extern struct opttxt txt_inverse_okay;
 extern struct opttxt txt_keep_dead_articles;
-extern struct opttxt txt_keep_posted_articles;
-extern struct opttxt txt_keep_posted_articles_file;
 extern struct opttxt txt_kill_level;
 extern struct opttxt txt_mail_8bit_header;
 extern struct opttxt txt_mail_address;
@@ -1383,8 +1498,14 @@ extern struct opttxt txt_mail_quote_format;
 extern struct opttxt txt_mailbox_format;
 extern struct opttxt txt_maildir;
 extern struct opttxt txt_mailer_format;
+extern struct opttxt txt_mark_ignore_tags;
 extern struct opttxt txt_mark_saved_read;
+extern struct opttxt txt_mono_markstar;
+extern struct opttxt txt_mono_markdash;
+extern struct opttxt txt_mono_markslash;
+extern struct opttxt txt_mono_markstroke;
 extern struct opttxt txt_mm_charset;
+extern struct opttxt txt_metamail_prog;
 extern struct opttxt txt_news_headers_to_display;
 extern struct opttxt txt_news_headers_to_not_display;
 extern struct opttxt txt_news_quote_format;
@@ -1394,6 +1515,7 @@ extern struct opttxt txt_post_8bit_header;
 extern struct opttxt txt_post_mime_encoding;
 extern struct opttxt txt_post_process;
 extern struct opttxt txt_post_process_view;
+extern struct opttxt txt_posted_articles_file;
 extern struct opttxt txt_posting_options;
 extern struct opttxt txt_print_header;
 extern struct opttxt txt_printer;
@@ -1412,10 +1534,9 @@ extern struct opttxt txt_score_select;
 extern struct opttxt txt_scroll_lines;
 extern struct opttxt txt_show_author;
 extern struct opttxt txt_show_description;
-extern struct opttxt txt_show_lines;
+extern struct opttxt txt_show_info;
 extern struct opttxt txt_show_only_unread_arts;
 extern struct opttxt txt_show_only_unread_groups;
-extern struct opttxt txt_show_score;
 extern struct opttxt txt_show_signatures;
 extern struct opttxt txt_sigdashes;
 extern struct opttxt txt_sigfile;
@@ -1445,7 +1566,9 @@ extern struct opttxt txt_use_mailreader_i;
 extern struct opttxt txt_use_mouse;
 extern struct opttxt txt_wildcard;
 extern struct opttxt txt_word_highlight;
+extern struct opttxt txt_word_h_display_marks;
 extern struct opttxt txt_wrap_column;
+extern struct opttxt txt_wrap_on_next_unread;
 extern struct opttxt txt_xpost_quote_format;
 #ifdef CHARSET_CONVERSION
 	extern struct opttxt txt_mm_network_charset;
@@ -1473,7 +1596,6 @@ extern struct opttxt txt_xpost_quote_format;
 	extern struct opttxt txt_col_from;
 	extern struct opttxt txt_col_title;
 	extern struct opttxt txt_col_signature;
-	extern struct opttxt txt_word_h_display_marks;
 	extern struct opttxt txt_col_markstar;
 	extern struct opttxt txt_col_markdash;
 	extern struct opttxt txt_col_markslash;
@@ -1485,9 +1607,7 @@ extern struct opttxt txt_xpost_quote_format;
 #ifdef HAVE_KEYPAD
 	extern struct opttxt txt_use_keypad;
 #endif /* HAVE_KEYPAD */
-#ifdef HAVE_METAMAIL
-	extern struct opttxt txt_use_metamail;
-	extern struct opttxt txt_ask_for_metamail;
-#endif /* HAVE_METAMAIL */
-
+#ifdef XFACE_ABLE
+	extern struct opttxt txt_use_slrnface;
+#endif /* XFACE_ABLE */
 #endif /* !EXTERN_H */
