@@ -3,7 +3,7 @@
  *  Module    : menukeys.h
  *  Author    : S.Robbins
  *  Created   : 1994-08-15
- *  Updated   : 2003-01-07
+ *  Updated   : 2003-05-14
  *  Notes     :
  *
  * Copyright (c) 1994-2003 Steve Robbins <stever@cs.mcgill.ca>
@@ -220,7 +220,7 @@
 #define iKeyGroupNextUnreadArt 'N'
 #define iKeyGroupPrevUnreadArt 'P'
 #define iKeyGroupBugReport 'R'
-#define iKeyGroupAutoSaveTagged 'S'
+#define iKeyGroupAutoSave 'S'
 #define iKeyGroupTagParts 'T'
 #define iKeyGroupUntag 'U'
 #define iKeyGroupMarkUnselArtRead 'X'
@@ -284,12 +284,12 @@
 #define iKeyPageCancel 'D'
 #define iKeyPageEditFilter 'E'
 #define iKeyPageFollowup 'F'
-#define iKeyPageLastPage2 'G'
+#define iKeyPageLastPage 'G'
 #define iKeyPageKillThd 'K'
 #define iKeyPageNextUnreadArt 'N'
 #define iKeyPagePrevUnreadArt 'P'
 #define iKeyPageReply 'R'
-#define iKeyPageAutoSaveTagged 'S'
+#define iKeyPageAutoSave 'S'
 #define iKeyPageGroupSel 'T'
 #define iKeyPageViewUrl 'U'
 #define iKeyPageViewAttach 'V'
@@ -300,7 +300,7 @@
 #define iKeyPageCatchup 'c'
 #define iKeyPageEditArticle 'e'
 #define iKeyPageFollowupQuote 'f'
-#define iKeyPageFirstPage2 'g'
+#define iKeyPageFirstPage 'g'
 #define iKeyPageListThd 'l'
 #define iKeyPageMail 'm'
 #define iKeyPageNextArt 'n'
@@ -351,9 +351,9 @@
 
 #define iKeySaveAppendFile 'a'
 #define iKeySaveOverwriteFile 'o'
+#define iKeyPProcNo 'n'
 #define iKeyPProcShar 's'
-#define iKeyPProcUUDecode 'u'
-#define iKeyPProcNone 'n'
+#define iKeyPProcYes 'y'
 
 /* select.c */
 
@@ -393,7 +393,7 @@
 #define iKeyThreadCatchupNextUnread 'C'
 #define iKeyThreadMarkArtRead 'K'
 #define iKeyThreadBugReport 'R'
-#define iKeyThreadAutoSaveTagged 'S'
+#define iKeyThreadAutoSave 'S'
 #define iKeyThreadUntag 'U'
 #define iKeyThreadMarkThdUnread 'Z'
 #define iKeyThreadCatchup 'c'
@@ -404,6 +404,9 @@
 #define iKeyThreadMarkArtUnread 'z'
 #define iKeyThreadUndoSel '~'
 
+#define iKeyMarkReadTag 't'
+#define iKeyMarkReadCur 'c'
+
 typedef struct kn {
 	const unsigned char defaultkey;	/* The default key definition */
 	unsigned char localkey;				/* The actual key definition */
@@ -428,6 +431,7 @@ typedef struct mkm {
 	t_menukeys filter_quit_edit_save;
 	t_menukeys group_nav;
 	t_menukeys info_nav;
+	t_menukeys mark_read_tagged_current;
 	t_menukeys nrctbl_create;
 	t_menukeys page_nav;
 	t_menukeys pgp_mail;
@@ -448,10 +452,10 @@ typedef struct mkm {
 	t_menukeys thread_nav;
 } t_menukeymap;
 
-extern int handle_keypad (int (*left_action) (void), int (*right_action) (void), const t_menukeys *menukeys);
-extern int prompt_slk_response (int ch_default, const t_menukeys *responses, const char *fmt, ...);
-extern int map_to_local (const char key, const t_menukeys *menukeys);
-extern int map_to_default (const char key, const t_menukeys *menukeys);
+extern int handle_keypad(int (*left_action) (void), int (*right_action) (void), const t_menukeys *menukeys);
+extern int prompt_slk_response(int ch_default, const t_menukeys *responses, const char *fmt, ...);
+extern int map_to_local(const char key, const t_menukeys *menukeys);
+extern int map_to_default(const char key, const t_menukeys *menukeys);
 extern t_menukeymap menukeymap;
 
 #endif /* !MENUKEYS_H */
