@@ -3,7 +3,7 @@
  *  Module    : policy.h
  *  Author    : Ralf Doeblitz <doeblitz@gmx.de>
  *  Created   : 1999-01-12
- *  Updated   : 2002-11-13
+ *  Updated   : 2003-03-11
  *  Notes     : #defines and static data for policy configuration
  *
  * Copyright (c) 1999-2003 Ralf Doeblitz <doeblitz@gmx.de>
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Ralf Doeblitz.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -213,7 +210,7 @@
  * .nt  Neutral Zone
  *
  * requested new TLDs:
- * .eu  EU
+ * .eu  European Union
  *
  * 'expireing' TLDs:
  *(.hk  Hong Kong, now: .cn)
@@ -264,16 +261,14 @@ static const char *gnksa_domain_list[] = {
 	"mil",	/* US Dept of Defense */
 	"net",	/* Networks */
 	"org",	/* Organizations */
+	"pro",	/* Accountants, lawyers, and physicians */
 	"aero",	/* Air-transport industry */
 	"arpa",	/* Address and Routing Parameter Area */
 	"coop",	/* Non-profit cooperatives */
 	"info",	/* Unrestricted use */
 	"name",	/* For registration by individuals */
 	"museum",	/* Museums */
-#	if 0		/* not yet valid */
-	"pro",	/* Accountants, lawyers, and physicians */
 	/* 3 more gTLDs to come in 2003 (.travel is likely to be one of them) */
-#	endif /* 0 */
 #	if 0		/* $DEAD */
 	"uucp",
 	"bitnet",
