@@ -3,7 +3,7 @@
  *  Module    : active.c
  *  Author    : I. Lea
  *  Created   : 1992-02-16
- *  Updated   : 2002-03-26
+ *  Updated   : 2003-06-29
  *  Notes     :
  *
  * Copyright (c) 1992-2003 Iain Lea <iain@bricbrac.de>
@@ -62,7 +62,6 @@ static time_t active_timestamp;	/* time active file read (local) */
 /*
  * Local prototypes
  */
-static int find_newnews_index(char *cur_newnews_host);
 static void active_add(struct t_group *ptr, long count, long max, long min, const char *moderated);
 static void append_group_line(char *active_file, char *group_path, long art_max, long art_min, char *base_dir);
 static void check_for_any_new_groups(void);
@@ -182,7 +181,7 @@ active_add(
 
 	if (moderated[0] == '/') {
 		ptr->type = GROUP_TYPE_SAVE;
-		ptr->spooldir = my_strdup(moderated);
+		ptr->spooldir = my_strdup(moderated); /* TODO: Unix'ism, other OSs need transformation */
 	} else {
 		ptr->type = GROUP_TYPE_NEWS;
 		ptr->spooldir = spooldir;		/* another global - sigh */
@@ -235,9 +234,9 @@ parse_active_line(
 		return FALSE;
 
 	(void) strtok(line, ACTIVE_SEP);		/* skip group name */
-	p = strtok((char *) 0, ACTIVE_SEP);	/* group max count */
-	q = strtok((char *) 0, ACTIVE_SEP);	/* group min count */
-	r = strtok((char *) 0, ACTIVE_SEP);	/* mod status or path to mailgroup */
+	p = strtok(NULL, ACTIVE_SEP);	/* group max count */
+	q = strtok(NULL, ACTIVE_SEP);	/* group min count */
+	r = strtok(NULL, ACTIVE_SEP);	/* mod status or path to mailgroup */
 
 	if (!p || !q || !r) {
 		error_message(_(txt_bad_active_file), line);
@@ -306,6 +305,7 @@ read_newsrc_active_file(
 			/*
 			 * 128 should be enough for a groupname, >256 and we overflow buffers
 			 * later on
+			 * TODO: check RFCs for possible max. size
 			 */
 			strncpy(ngname, ptr, 128);
 			ptr = ngname;
@@ -362,9 +362,9 @@ read_newsrc_active_file(
 							sprintf(fmt, "%%ld %%ld %%ld %%%ds", NNTP_STRLEN);
 
 							if (sscanf(line, fmt, &count, &min, &max, ngname) != 4)
-								error_message(_("Invalid response to GROUP command, %s"), line); /* FIXME: -> lang.c */
+								error_message(_(txt_error_invalid_response_to_group), line);
 							if (strcmp(ngname, ngnames[index_o]) != 0)
-								error_message(_("Wrong newsgroup name in response of GROUP command, %s for %s"), line, ngnames[index_o]); /* FIXME: -> lang.c */
+								error_message(_(txt_error_wrong_newsgroupname_in_group_response), line, ngnames[index_o]);
 							ptr = ngname;
 							free(ngnames[index_o]);
 							index_o = (index_o + 1) % NUM_SIMULTANEOUS_GROUP_COMMAND;
@@ -379,7 +379,7 @@ read_newsrc_active_file(
 						continue;
 
 					case ERR_ACCESS:
-						error_message(cCRLF "%s", line);
+						error_message("%s%s", cCRLF, line);
 						tin_done(NNTP_ERROR_EXIT);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
@@ -444,8 +444,8 @@ read_newsrc_active_file(
 	 * Exit if active file wasn't read correctly or is empty
 	 */
 	if (tin_errno || !num_active) {
-		if (newsrc_active && !num_active) /* TODO: -> lang.c */
-			error_message(_("Server has non of the groups listed in %s"), newsrc);
+		if (newsrc_active && !num_active)
+			error_message(_(txt_error_server_has_no_listed_groups), newsrc);
 		else
 			error_message(_(txt_active_file_is_empty), (read_news_via_nntp ? _(txt_servers_active) : news_active_file));
 		tin_done(EXIT_FAILURE);
@@ -558,12 +558,12 @@ read_news_active_file(
 		if ((fp = fopen(newsrc, "r")) == NULL) {
 			list_active = TRUE;
 			newsrc_active = FALSE;
-		} else
+		} else {
 			fclose(fp);
-
-		if (file_size(newsrc) <= 0L) {
-			list_active = TRUE;
-			newsrc_active = FALSE;
+			if (file_size(newsrc) <= 0L) {
+				list_active = TRUE;
+				newsrc_active = FALSE;
+			}
 		}
 	}
 
@@ -578,7 +578,16 @@ read_news_active_file(
 	(void) time(&active_timestamp);
 	force_reread_active_file = FALSE;
 
-	check_for_any_new_groups();
+	/*
+	 * check_for_any_new_groups() also does $AUTOSUBSCRIBE
+	 */
+	if (check_for_new_newsgroups)
+		check_for_any_new_groups();
+
+	/*
+	 * finally we have a list of all groups an can set the attributes
+	 */
+	read_attributes_files();
 }
 
 
@@ -599,30 +608,23 @@ check_for_any_new_groups(
 	char *autosubscribe, *autounsubscribe;
 	char *ptr, *line, buf[NNTP_STRLEN];
 	char old_newnews_host[PATH_LEN];
-	char new_newnews_host[PATH_LEN];
 	int newnews_index;
 	time_t old_newnews_time;
 	time_t new_newnews_time;
 
-	if (!check_for_new_newsgroups /* || !batch_mode */)
-		return;
-
-#if 0
-	if (!batch_mode /* || (batch_mode && verbose) */)
-#endif /* 0 */
+	if (!batch_mode /* || verbose */)
 		wait_message(0, _(txt_checking_new_groups));
 
 	(void) time(&new_newnews_time);
-	strcpy(new_newnews_host, (read_news_via_nntp ? nntp_server : "local")); /* What if nntp server called local? */
 
 	/*
 	 * find out if we have read news from here before otherwise -1
 	 */
-	if ((newnews_index = find_newnews_index(new_newnews_host)) >= 0) {
-		strcpy(old_newnews_host, newnews[newnews_index].host);
+	if ((newnews_index = find_newnews_index(nntp_server)) >= 0) {
+		STRCPY(old_newnews_host, newnews[newnews_index].host);
 		old_newnews_time = newnews[newnews_index].time;
 	} else {
-		strcpy(old_newnews_host, "UNKNOWN");
+		STRCPY(old_newnews_host, "UNKNOWN");
 		old_newnews_time = (time_t) 0;
 	}
 
@@ -656,10 +658,6 @@ check_for_any_new_groups(
 		}
 		TIN_FCLOSE(fp);
 
-		free_attributes_array();		/* TODO - wtf is this doing here? */
-		read_attributes_file(TRUE);
-		read_attributes_file(FALSE);
-
 		if (tin_errno)
 			return;				/* Don't update the time if we quit */
 	}
@@ -672,10 +670,9 @@ check_for_any_new_groups(
 	if (newnews_index >= 0)
 		newnews[newnews_index].time = new_newnews_time;
 	else {
-		sprintf(buf, "%s %lu", new_newnews_host, (unsigned long int) new_newnews_time);
+		sprintf(buf, "%s %lu", nntp_server, (unsigned long int) new_newnews_time);
 		load_newnews_info(buf);
 	}
-	my_fputc('\n', stdout);
 }
 
 
@@ -723,9 +720,15 @@ subscribe_new_group(
 	}
 
 	if (!no_write && (autosubscribe != NULL) && match_group_list(group, autosubscribe)) {
-		my_printf(_(txt_autosubscribed), group);
+		if (!batch_mode || verbose)
+			my_printf(_(txt_autosubscribed), group);
 
-		subscribe(&active[my_group[idx]], SUBSCRIBED);
+		/*
+		 * as subscribe_new_group() is called from check_for_any_new_groups()
+		 * which has pending data on the socket if reading via NNTP we are not
+		 * allowed to issue any NNTP comands yet
+		 */
+		subscribe(&active[my_group[idx]], SUBSCRIBED, bool_not(read_news_via_nntp));
 		/*
 		 * Bad kluge to stop group later appearing in New newsgroups. This
 		 * effectively loses the group, and it has now been subscribed to and
@@ -854,9 +857,9 @@ load_newnews_info(
 /*
  * Return the index of cur_newnews_host in newnews[] or -1 if not found
  */
-static int
+int
 find_newnews_index(
-	char *cur_newnews_host)
+	const char *cur_newnews_host)
 {
 	int i;
 
@@ -872,6 +875,9 @@ find_newnews_index(
 /*
  * Get a single status char from the moderated field. Used on selection screen
  * and in header of group screen
+ *
+ * TODO: what about 'j' groups? active(5) says:
+ *       "local postings to that group should not be generated"
  */
 char
 group_flag(
@@ -902,7 +908,7 @@ create_save_active_file(
 	char group_path[PATH_LEN];
 	char local_save_active_file[PATH_LEN];
 
-	snprintf(local_save_active_file, sizeof(local_save_active_file) - 1, "%s/%s/%s", homedir, RCDIR, ACTIVE_SAVE_FILE);
+	joinpath(local_save_active_file, rcdir, ACTIVE_SAVE_FILE);
 
 	if (no_write && file_size(local_save_active_file) != -1L)
 		return;
@@ -935,7 +941,7 @@ make_group_list(
 		is_dir = FALSE;
 		while ((direntry = readdir(dir)) != NULL) {
 			STRCPY(filename, direntry->d_name);
-			sprintf(path, "%s/%s", group_path, filename);
+			joinpath(path, group_path, filename);
 
 			if (!(filename[0] == '.' && filename[1] == '\0') &&
 				!(filename[0] == '.' && filename[1] == '.' && filename[2] == '\0')) {
@@ -952,7 +958,7 @@ make_group_list(
 				find_art_max_min(group_path, &art_max, &art_min);
 				append_group_line(active_file, group_path, art_max, art_min, base_dir);
 
-				ptr = strrchr(group_path, '/');
+				ptr = strrchr(group_path, '/'); /* TODO: Unix'ism */
 				if (ptr != NULL)
 					*ptr = '\0';
 			}
@@ -988,8 +994,7 @@ append_group_line(
 		make_group_name(base_dir, group_name, group_path);
 		my_printf("Appending=[%s %ld %ld %s]\n", group_name, art_max, art_min, base_dir);
 		print_group_line(fp, group_name, art_max, art_min, base_dir);
-		if (ferror(fp) || fclose(fp))
-			/* TODO: issue warning? */
+		if (ferror(fp) || fclose(fp)) /* TODO: issue warning? */
 			rename(file_tmp, active_file);
 	}
 	unlink(file_tmp);
