@@ -3,7 +3,7 @@
  *  Module    : config.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-08-10
+ *  Updated   : 2003-09-01
  *  Notes     : Configuration file routines
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
@@ -94,7 +94,7 @@ read_config_file(
 {
 	FILE *fp;
 	char newnews_info[PATH_LEN];
-	char buf[LEN];
+	char buf[LEN], tmp[LEN];
 	int upgrade = RC_CHECK;
 
 	if ((fp = fopen(file, "r")) == NULL)
@@ -128,48 +128,48 @@ read_config_file(
 			if (match_boolean(buf, "alternative_handling=", &tinrc.alternative_handling))
 				break;
 
-			if (match_string(buf, "art_marked_deleted=", buf, sizeof(buf))) {
-				tinrc.art_marked_deleted = !buf[0] ? ART_MARK_DELETED : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_deleted=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_deleted = !tmp[0] ? ART_MARK_DELETED : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_inrange=", buf, sizeof(buf))) {
-				tinrc.art_marked_inrange = !buf[0] ? MARK_INRANGE : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_inrange=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_inrange = !tmp[0] ? MARK_INRANGE : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_killed=", buf, sizeof(buf))) {
-				tinrc.art_marked_killed = !buf[0] ? ART_MARK_KILLED : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_killed=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_killed = !tmp[0] ? ART_MARK_KILLED : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_read=", buf, sizeof(buf))) {
-				tinrc.art_marked_read = !buf[0] ? ART_MARK_READ : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_read=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_read = !tmp[0] ? ART_MARK_READ : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_read_selected=", buf, sizeof(buf))) {
-				tinrc.art_marked_read_selected = !buf[0] ? ART_MARK_READ_SELECTED : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_read_selected=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_read_selected = !tmp[0] ? ART_MARK_READ_SELECTED : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_recent=", buf, sizeof(buf))) {
-				tinrc.art_marked_recent = !buf[0] ? ART_MARK_RECENT : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_recent=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_recent = !tmp[0] ? ART_MARK_RECENT : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_return=", buf, sizeof(buf))) {
-				tinrc.art_marked_return = !buf[0] ? ART_MARK_RETURN : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_return=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_return = !tmp[0] ? ART_MARK_RETURN : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_selected=", buf, sizeof(buf))) {
-				tinrc.art_marked_selected = !buf[0] ? ART_MARK_SELECTED : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_selected=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_selected = !tmp[0] ? ART_MARK_SELECTED : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
-			if (match_string(buf, "art_marked_unread=", buf, sizeof(buf))) {
-				tinrc.art_marked_unread = !buf[0] ? ART_MARK_UNREAD : DASH_TO_SPACE(buf[0]);
+			if (match_string(buf, "art_marked_unread=", tmp, sizeof(tmp))) {
+				tinrc.art_marked_unread = !tmp[0] ? ART_MARK_UNREAD : DASH_TO_SPACE(tmp[0]);
 				break;
 			}
 
@@ -340,8 +340,8 @@ read_config_file(
 			if (match_boolean(buf, "default_filter_select_expire=", &tinrc.default_filter_select_expire))
 				break;
 
-			if (match_string(buf, "default_save_mode=", buf, sizeof(buf))) {
-				tinrc.default_save_mode = buf[0];
+			if (match_string(buf, "default_save_mode=", tmp, sizeof(tmp))) {
+				tinrc.default_save_mode = tmp[0];
 				break;
 			}
 
@@ -2880,19 +2880,19 @@ read_server_config(
 			snprintf(tmp_info, tmp_len, "%s %s", nntp_server, newnews_info);
 			load_newnews_info(tmp_info);
 			free(tmp_info);
-			break;
+			continue;
 		}
 		if (match_string(line, "version=", version, sizeof(version))) {
 			if (RC_CHECK != upgrade)
-				/* ignore duplicate version lines; first match counts */
-				break;
+				/* ignore duplicate version lines; last match counts */
+				continue;
 			upgrade = check_upgrade(line, "version=", SERVERCONFIG_VERSION);
 			if (RC_IGNORE == upgrade)
 				/* Expected version number; nothing to do -> continue */
-				break;
+				continue;
 
 			/* Nothing to do yet for RC_UPGRADE and RC_DOWNGRADE */
-			break;
+			continue;
 		}
 	}
 	fclose(fp);
