@@ -3,7 +3,7 @@
  *  Module    : filter.c
  *  Author    : I. Lea
  *  Created   : 1992-12-28
- *  Updated   : 2003-01-18
+ *  Updated   : 2003-05-16
  *  Notes     : Filter articles. Kill & auto selection are supported.
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
@@ -38,6 +38,9 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
+#ifndef VERSION_H
+#	include "version.h"
+#endif /* !VERSION_H */
 #ifndef TCURSES_H
 #	include "tcurses.h"
 #endif /* !TCURSES_H */
@@ -45,6 +48,7 @@
 #	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
+
 #define IS_READ(i)	(arts[i].status == ART_READ)
 #define IS_KILLED(i)	(arts[i].killed)
 #define IS_SELECTED(i)	(arts[i].selected)
@@ -87,16 +91,16 @@ struct t_filters glob_filter = { 0, 0, (struct t_filter *) 0 };
  */
 static int get_choice(int x, const char *help, const char *prompt, const char *opt1, const char *opt2, const char *opt3, const char *opt4, const char *opt5);
 static int set_filter_scope(struct t_group *group);
-static t_bool add_filter_rule(struct t_group *group, struct t_article *art, struct t_filter_rule *rule);
-static t_bool test_regex(const char *string, char *regex, t_bool nocase, struct regex_cache *cache);
 static struct t_filter_comment *add_filter_comment(struct t_filter_comment *ptr, char *text);
 static struct t_filter_comment *free_filter_comment(struct t_filter_comment *ptr);
 static struct t_filter_comment *copy_filter_comment(struct t_filter_comment *from, struct t_filter_comment *to);
+static t_bool add_filter_rule(struct t_group *group, struct t_article *art, struct t_filter_rule *rule);
+static t_bool test_regex(const char *string, char *regex, t_bool nocase, struct regex_cache *cache);
 static void expand_filter_array(struct t_filters *ptr);
 static void free_filter_item(struct t_filter *ptr);
 static void print_filter_menu(void);
 static void set_filter(struct t_filter *ptr);
-static void write_filter_array(FILE *fp, struct t_filters *ptr, time_t theTime);
+static void write_filter_array(FILE *fp, struct t_filters *ptr);
 
 
 /*
@@ -113,9 +117,9 @@ add_filter_comment(
 		ptr->text = my_strdup(text);
 		ptr->next = (struct t_filter_comment *) 0;
 	} else
-		ptr->next = (struct t_filter_comment *) add_filter_comment((struct t_filter_comment *) ptr->next, (char *) text);
+		ptr->next = add_filter_comment(ptr->next, text);
 
-	return (struct t_filter_comment *) ptr;
+	return ptr;
 }
 
 
@@ -132,11 +136,12 @@ free_filter_comment(
 	tmp = ptr;
 	while (tmp != NULL) {
 		next = tmp->next;
+		free(tmp->text);
 		free(tmp);
 		tmp = next;
 	}
 
-	return (struct t_filter_comment *) tmp;
+	return tmp;
 }
 
 
@@ -151,12 +156,10 @@ copy_filter_comment(
 	if (from != NULL) {
 		to = my_malloc(sizeof(struct t_filter_comment));
 		to->text = my_strdup(from->text);
-		/* don't know if the next line is necessary, but it doesn't harm */
-		to->next = (struct t_filter_comment *) 0;
-		to->next = (struct t_filter_comment *) copy_filter_comment(from->next, to->next);
+		to->next = copy_filter_comment(from->next, NULL);
 	}
 
-	return (struct t_filter_comment *) to;
+	return to;
 }
 
 
@@ -232,8 +235,6 @@ set_filter(
 		ptr->gnksa_num = 0;
 		ptr->score = 0;
 		ptr->xref = (char *) 0;
-		ptr->xref_max = 0;
-		ptr->xref_score_cnt = 0;
 		ptr->time = (time_t) 0;
 		ptr->next = (struct t_filter *) 0;
 	}
@@ -263,7 +264,7 @@ void
 free_filter_array(
 	struct t_filters *ptr)
 {
-	register int i;
+	int i;
 
 	if (ptr != NULL) {
 		for (i = 0; i < ptr->num; i++)
@@ -284,7 +285,6 @@ read_filter_file(
 	const char *file)
 {
 	FILE *fp;
-	char *s;
 	char buf[HEADER_LEN];
 	char scope[HEADER_LEN];
 	char comment_line[LEN];  /* one line of comment */
@@ -294,26 +294,23 @@ read_filter_file(
 	char buffer[HEADER_LEN];
 	char gnksa[HEADER_LEN];
 	char xref[HEADER_LEN];
-	char xref_score[HEADER_LEN];
 	char scbuf[PATH_LEN];
 	int i = 0;
 	int icase = 0;
 	int score = 0;
-	int xref_max = 0;
-	int xref_score_cnt = 0;
-	int xref_score_value = 0;
 	long secs = 0L;
 	struct t_filter_comment *comment;
 	struct t_filter *ptr;
-	struct t_group *group;
 	t_bool expired = FALSE;
 	t_bool expired_time = FALSE;
 	time_t current_secs = (time_t) 0;
+	static t_bool first_read = TRUE;
+	int upgrade = RC_CHECK;
 
 	if ((fp = fopen(file, "r")) == NULL)
 		return FALSE;
 
-	if (!batch_mode || (batch_mode && verbose))
+	if (!batch_mode || verbose)
 		wait_message(0, _(txt_reading_filter_file));
 
 	(void) time(&current_secs);
@@ -321,15 +318,23 @@ read_filter_file(
 	/*
 	 * Reset all filter arrays if doing a reread of the active file
 	 */
-	free_filter_array(&glob_filter);
+	if (!first_read)
+		free_filter_array(&glob_filter);
 
-	group = (struct t_group *) 0;
 	ptr = (struct t_filter *) 0;
 	comment = (struct t_filter_comment *) 0;
 
 	while (fgets(buf, (int) sizeof(buf), fp) != NULL) {
-		if (*buf == '#' || *buf == '\n')
+		if (*buf == '#' || *buf == '\n') {
+			if (upgrade == RC_CHECK && first_read) {
+				first_read = FALSE;
+				upgrade = check_upgrade(buf, "# Filter file V", FILTER_VERSION);
+				if (upgrade != RC_IGNORE)
+					upgrade_prompt_quit(upgrade, FILTER_FILE);
+				/* TODO: do something (more) useful here */
+			}
 			continue;
+		}
 
 		switch (tolower((unsigned char) buf[0])) {
 		case 'c':
@@ -340,16 +345,24 @@ read_filter_file(
 				break;
 			}
 			if (match_string(buf + 1, "omment=", comment_line, sizeof(comment_line))) {
-				comment = (struct t_filter_comment *) add_filter_comment(comment, comment_line);
+				comment = add_filter_comment(comment, comment_line);
 				break;
 			}
 			break;
 
 		case 'f':
 			if (match_string(buf + 1, "rom=", from, sizeof(from))) {
-				if (ptr && !expired_time)
-					ptr[i].from = my_strdup(from);
-
+				if (ptr && !expired_time) {
+					if (tinrc.wildcard && ptr[i].from != NULL) {
+						/* merge with already read value */
+						ptr[i].from = my_realloc(ptr[i].from, strlen(ptr[i].from) + strlen(from) + 2);
+						strcat(ptr[i].from, "|");
+						strcat(ptr[i].from, from);
+					} else {
+						FreeIfNeeded(ptr[i].from);
+						ptr[i].from = my_strdup(from);
+					}
+				}
 				break;
 			}
 			break;
@@ -371,8 +384,8 @@ read_filter_file(
 				expired_time = FALSE;
 				ptr[i].scope = my_strdup(scope);
 				if (comment != NULL) {
-					ptr[i].comment = (struct t_filter_comment *) copy_filter_comment(comment, ptr[i].comment);
-					comment = (struct t_filter_comment *) free_filter_comment(comment);
+					ptr[i].comment = copy_filter_comment(comment, ptr[i].comment);
+					comment = free_filter_comment(comment);
 				}
 				subj[0] = '\0';
 				from[0] = '\0';
@@ -381,7 +394,6 @@ read_filter_file(
 				xref[0] = '\0';
 				icase = 0;
 				secs = 0L;
-				group = (struct t_group *) 0;		/* fudge for out of order rules */
 				break;
 			}
 			if (match_string(buf + 1, "nksa=", gnksa, sizeof(gnksa))) {
@@ -421,23 +433,47 @@ read_filter_file(
 
 		case 'm':
 			if (match_string(buf + 1, "sgid=", msgid, sizeof(msgid))) {
-				if (ptr) {
-					ptr[i].msgid = my_strdup(msgid);
-					ptr[i].fullref = FILTER_MSGID;
+				if (ptr && !expired_time) {
+					if (tinrc.wildcard && ptr[i].msgid != NULL && ptr[i].fullref == FILTER_MSGID) {
+						/* merge with already read value */
+						ptr[i].msgid = my_realloc(ptr[i].msgid, strlen(ptr[i].msgid) + strlen(msgid) + 2);
+						strcat(ptr[i].msgid, "|");
+						strcat(ptr[i].msgid, msgid);
+					} else {
+						FreeIfNeeded(ptr[i].msgid);
+						ptr[i].msgid = my_strdup(msgid);
+						ptr[i].fullref = FILTER_MSGID;
+					}
 				}
 				break;
 			}
 			if (match_string(buf + 1, "sgid_last=", msgid, sizeof(msgid))) {
-				if (ptr) {
-					ptr[i].msgid = my_strdup(msgid);
-					ptr[i].fullref = FILTER_MSGID_LAST;
+				if (ptr && !expired_time) {
+					if (tinrc.wildcard && ptr[i].msgid != NULL && ptr[i].fullref == FILTER_MSGID_LAST) {
+						/* merge with already read value */
+						ptr[i].msgid = my_realloc(ptr[i].msgid, strlen(ptr[i].msgid) + strlen(msgid) + 2);
+						strcat(ptr[i].msgid, "|");
+						strcat(ptr[i].msgid, msgid);
+					} else {
+						FreeIfNeeded(ptr[i].msgid);
+						ptr[i].msgid = my_strdup(msgid);
+						ptr[i].fullref = FILTER_MSGID_LAST;
+					}
 				}
 				break;
 			}
 			if (match_string(buf + 1, "sgid_only=", msgid, sizeof(msgid))) {
-				if (ptr) {
-					ptr[i].msgid = my_strdup(msgid);
-					ptr[i].fullref = FILTER_MSGID_ONLY;
+				if (ptr && !expired_time) {
+					if (tinrc.wildcard && ptr[i].msgid != NULL && ptr[i].fullref == FILTER_MSGID_ONLY) {
+						/* merge with already read value */
+						ptr[i].msgid = my_realloc(ptr[i].msgid, strlen(ptr[i].msgid) + strlen(msgid) + 2);
+						strcat(ptr[i].msgid, "|");
+						strcat(ptr[i].msgid, msgid);
+					} else {
+						FreeIfNeeded(ptr[i].msgid);
+						ptr[i].msgid = my_strdup(msgid);
+						ptr[i].fullref = FILTER_MSGID_ONLY;
+					}
 				}
 				break;
 			}
@@ -445,9 +481,17 @@ read_filter_file(
 
 		case 'r':
 			if (match_string(buf + 1, "efs_only=", msgid, sizeof(msgid))) {
-				if (ptr) {
-					ptr[i].msgid = my_strdup(msgid);
-					ptr[i].fullref = FILTER_REFS_ONLY;
+				if (ptr && !expired_time) {
+					if (tinrc.wildcard && ptr[i].msgid != NULL && ptr[i].fullref == FILTER_REFS_ONLY) {
+						/* merge with already read value */
+						ptr[i].msgid = my_realloc(ptr[i].msgid, strlen(ptr[i].msgid) + strlen(msgid) + 2);
+						strcat(ptr[i].msgid, "|");
+						strcat(ptr[i].msgid, msgid);
+					} else {
+						FreeIfNeeded(ptr[i].msgid);
+						ptr[i].msgid = my_strdup(msgid);
+						ptr[i].fullref = FILTER_REFS_ONLY;
+					}
 				}
 				break;
 			}
@@ -455,8 +499,17 @@ read_filter_file(
 
 		case 's':
 			if (match_string(buf + 1, "ubj=", subj, sizeof(subj))) {
-				if (ptr && !expired_time)
-					ptr[i].subj = my_strdup(subj);
+				if (ptr && !expired_time) {
+					if (tinrc.wildcard && ptr[i].subj != NULL) {
+						/* merge with already read value */
+						ptr[i].subj = my_realloc(ptr[i].subj, strlen(ptr[i].subj) + strlen(subj) + 2);
+						strcat(ptr[i].subj, "|");
+						strcat(ptr[i].subj, subj);
+					} else {
+						FreeIfNeeded(ptr[i].subj);
+						ptr[i].subj = my_strdup(subj);
+					}
+				}
 
 #ifdef DEBUG
 				if (debug) {
@@ -523,34 +576,44 @@ read_filter_file(
 			break;
 
 		case 'x':
+			/*
+			 * TODO: fromat has changed in FILTER_VERSION 1.0.0,
+			 *       should we comment out older xref rules like below?
+			 */
 			if (match_string(buf + 1, "ref=", xref, sizeof(xref))) {
-				if (ptr && !expired_time)
-					ptr[i].xref = my_strdup(xref);
-
-				break;
-			}
-			if (match_integer(buf + 1, "ref_max=", &xref_max, 1000)) {
-				if (ptr && !expired_time)
-					ptr[i].xref_max = xref_max;
-
-				break;
-			}
-			if (match_string(buf + 1, "ref_score=", xref_score, sizeof(xref_score))) {
 				if (ptr && !expired_time) {
-					if (xref_score_cnt < 10) {
-						if (isdigit((int) xref_score[0])) {
-							xref_score_value = atoi(xref_score);
-							if ((s = strchr(xref_score, ',')))
-								s++;
-							ptr[i].xref_scores[xref_score_cnt] = xref_score_value;
-							ptr[i].xref_score_strings[xref_score_cnt] = (s != 0 ? my_strdup(s) : '\0');
-							ptr[i].xref_score_cnt++;
-							xref_score_cnt++;
-						}
+					if (tinrc.wildcard && ptr[i].xref != NULL) {
+						/* merge with already read value */
+						ptr[i].xref = my_realloc(ptr[i].xref, strlen(ptr[i].xref) + strlen(xref) + 2);
+						strcat(ptr[i].xref, "|");
+						strcat(ptr[i].xref, xref);
+					} else {
+						FreeIfNeeded(ptr[i].xref);
+						ptr[i].xref = my_strdup(xref);
 					}
 				}
 				break;
 			}
+			if (upgrade == RC_UPGRADE) {
+				char foo[HEADER_LEN];
+
+				if (match_string(buf + 1, "ref_max=", foo, LEN - 1)) {
+					/*
+					 * TODO: add to the right rule, give better explanation, -> lang.c
+					 */
+					snprintf(foo, HEADER_LEN, "%s%s", _("Removed from the previous rule: "), str_trim(buf));
+					comment = (struct t_filter_comment *) add_filter_comment(comment, foo);
+					break;
+				}
+				if (match_string(buf + 1, "ref_score=", foo, LEN - 1)) {
+					/*
+					 * TODO: add to the right rule, give better explanation, -> lang.c
+					 */
+					snprintf(foo, HEADER_LEN, "%s%s", _("Removed from the previous rule: "), str_trim(buf));
+					comment = (struct t_filter_comment *) add_filter_comment(comment, foo);
+					break;
+				}
+			}
 			break;
 
 		default:
@@ -559,10 +622,11 @@ read_filter_file(
 	}
 	fclose(fp);
 
-	if (expired)
+	if (expired || upgrade == RC_UPGRADE) {
 		write_filter_file(file);
+	}
 
-	if (cmd_line)
+	if (cmd_line && !batch_mode)
 		printf("\r\n");
 
 	if (!batch_mode)
@@ -597,13 +661,15 @@ write_filter_file(
 	if ((fp = fopen(filename, "w")) == NULL)
 		return;
 
+	/* TODO: -> lang.c */
+	fprintf(fp, "# Filter file V%s for the TIN newsreader\n#\n", FILTER_VERSION);
 	fprintf(fp, _(txt_filter_file));
 	fflush(fp);
 
 	/*
 	 * Save global filters
 	 */
-	write_filter_array(fp, &glob_filter, time(NULL));
+	write_filter_array(fp, &glob_filter);
 
 	if (ferror(fp) || fclose(fp)) {
 		error_message(_(txt_filesystem_full), filename);
@@ -618,12 +684,11 @@ write_filter_file(
 static void
 write_filter_array(
 	FILE *fp,
-	struct t_filters *ptr,
-	time_t theTime)
+	struct t_filters *ptr)
 {
 	int i;
-	int j;
 	struct t_filter_comment *comment = (struct t_filter_comment *) 0;
+	time_t theTime = time(NULL);
 
 	if (ptr == NULL)
 		return;
@@ -631,10 +696,8 @@ write_filter_array(
 	for (i = 0; i < ptr->num; i++) {
 /* my_printf("WRITE i=[%d] subj=[%s] from=[%s]\n", i, BlankIfNull(ptr->filter[i].subj), BlankIfNull(ptr->filter[i].from)); */
 
-		if (theTime && ptr->filter[i].time) {
-			if (theTime > ptr->filter[i].time)
+		if (ptr->filter[i].time && theTime > ptr->filter[i].time)
 				continue;
-		}
 /*
 my_printf("Scope=[%s]" cCRLF, (ptr->filter[i].scope != NULL ? ptr->filter[i].scope : "*"));
 my_flush();
@@ -649,7 +712,7 @@ my_flush();
 			 */
 			comment = ptr->filter[i].comment;
 			while (ptr->filter[i].comment != NULL) {
-				fprintf (fp, "comment=%s\n", ptr->filter[i].comment->text);
+				fprintf(fp, "comment=%s\n", ptr->filter[i].comment->text);
 				ptr->filter[i].comment = ptr->filter[i].comment->next;
 			}
 			ptr->filter[i].comment = comment;
@@ -736,12 +799,6 @@ my_flush();
 		if (ptr->filter[i].xref != NULL)
 			fprintf(fp, "xref=%s\n", ptr->filter[i].xref);
 
-		if (ptr->filter[i].xref_max > 0) {
-			fprintf(fp, "xref_max=%d\n", ptr->filter[i].xref_max);
-
-			for (j = 0; j < ptr->filter[i].xref_score_cnt; j++)
-				fprintf(fp, "xref_score=%d%s%s\n", ptr->filter[i].xref_scores[j], ptr->filter[i].xref_score_strings[j] ? "," : "", ptr->filter[i].xref_score_strings[j]);
-		}
 		if (ptr->filter[i].time) {
 			char timestring[25];
 			if (my_strftime(timestring, sizeof(timestring) - 1, "%Y-%m-%d %H:%M:%S UTC", gmtime(&(ptr->filter[i].time))))
@@ -793,9 +850,9 @@ get_choice(
 			continue;
 		if (++i == n)
 			i = 0;
-	} while (ch != '\n' && ch != '\r' && ch != ESC);
+	} while (ch != '\n' && ch != '\r' && ch != iKeyAbort); /* TODO: replace hardcoded keynames */
 
-	if (ch == ESC)
+	if (ch == iKeyAbort)
 		return -1;
 
 	return i;
@@ -824,15 +881,15 @@ print_filter_menu(
 	center_line(0, TRUE, ptr_filter_menu);
 
 	MoveCursor(INDEX_TOP, 0);
-	my_printf("%s" cCRLF cCRLF, ptr_filter_comment);
-	my_printf("%s" cCRLF, ptr_filter_text);
-	my_printf("%s" cCRLF cCRLF, _(txt_filter_text_type));
-	my_printf("%s" cCRLF, text_subj);
-	my_printf("%s" cCRLF, text_from);
-	my_printf("%s" cCRLF cCRLF, text_msgid);
-	my_printf("%s" cCRLF, ptr_filter_lines);
-	my_printf("%s" cCRLF, text_score);
-	my_printf("%s" cCRLF cCRLF, ptr_filter_time);
+	my_printf("%s%s%s", ptr_filter_comment, cCRLF, cCRLF);
+	my_printf("%s%s", ptr_filter_text, cCRLF);
+	my_printf("%s%s%s",  _(txt_filter_text_type), cCRLF, cCRLF);
+	my_printf("%s%s", text_subj, cCRLF);
+	my_printf("%s%s", text_from, cCRLF);
+	my_printf("%s%s%s", text_msgid, cCRLF, cCRLF);
+	my_printf("%s%s", ptr_filter_lines, cCRLF);
+	my_printf("%s%s", text_score, cCRLF);
+	my_printf("%s%s%s", ptr_filter_time, cCRLF, cCRLF);
 	my_printf("%s%s", ptr_filter_scope, ptr_filter_groupname);
 	my_flush();
 }
@@ -884,6 +941,7 @@ filter_menu(
 	int ch, i, len;
 	struct t_filter_rule rule;
 	t_bool proceed;
+	t_bool ret;
 
 	signal_context = cFilter;
 
@@ -936,26 +994,17 @@ filter_menu(
 		ptr_filter_quit_edit_save = _(txt_quit_edit_save_select);
 	}
 
-	ptr_filter_comment = _(txt_select_comment);
+	ptr_filter_comment = _(txt_filter_comment);
 	ptr_filter_groupname = group->name;
 
 	len = cCOLS - 30;
 
 	snprintf(text_time, sizeof(text_time), _(txt_time_default_days), tinrc.filter_days);
-	text_time[sizeof(text_time) - 1] = '\0';
-
 	snprintf(text_subj, sizeof(text_subj), ptr_filter_subj, len, len, art->subject);
-	text_subj[sizeof(text_subj) - 1] = '\0';
-
 	snprintf(text_score, sizeof(text_score), _(txt_filter_score), (type == FILTER_KILL ? -tinrc.score_kill : tinrc.score_select));
-	text_score[sizeof(text_score) - 1] = '\0';
-
 	STRCPY(buf, art->from);
 	snprintf(text_from, sizeof(text_from), ptr_filter_from, len, len, buf);
-	text_from[sizeof(text_from) - 1] = '\0';
-
 	snprintf(text_msgid, sizeof(text_msgid), ptr_filter_msgid, len - 4, len - 4, MSGID(art));
-	text_msgid[sizeof(text_msgid) - 1] = '\0';
 
 	print_filter_menu();
 
@@ -966,18 +1015,22 @@ filter_menu(
 	 */
 	show_menu_help(_(txt_help_filter_comment));
 	while ((proceed = prompt_menu_string(INDEX_TOP, ptr_filter_comment, comment_line)) && comment_line[0] != '\0') {
-		rule.comment = (struct t_filter_comment *) add_filter_comment(rule.comment, comment_line);
+		rule.comment = add_filter_comment(rule.comment, comment_line);
 		comment_line[0] = '\0';
 	}
-	if (!proceed)
+	if (!proceed) {
+		rule.comment = free_filter_comment(rule.comment);
 		return FALSE;
+	}
 
 	/*
 	 * Text which might be used to filter on subj, from or msgid
 	 */
 	show_menu_help(_(txt_help_filter_text));
-	if (!prompt_menu_string(INDEX_TOP + 2, ptr_filter_text, rule.text))
+	if (!prompt_menu_string(INDEX_TOP + 2, ptr_filter_text, rule.text)) {
+		rule.comment = free_filter_comment(rule.comment);
 		return FALSE;
+	}
 
 	if (*rule.text) {
 		i = get_choice(INDEX_TOP + 3, _(txt_help_filter_text_type),
@@ -987,8 +1040,10 @@ filter_menu(
 			       _(txt_from_line_only_case),
 			       _(txt_from_line_only),
 			       _(txt_msgid_line_only));
-		if (i == -1)
+		if (i == -1) {
+			rule.comment = free_filter_comment(rule.comment);
 			return FALSE;
+		}
 
 		rule.counter = i;
 		switch (i) {
@@ -1018,18 +1073,20 @@ filter_menu(
 		 * Subject:
 		 */
 		i = get_choice(INDEX_TOP + 5, _(txt_help_filter_subj), text_subj, _(txt_yes), _(txt_no), (char *) 0, (char *) 0, (char *) 0);
-		if (i == -1)
+		if (i == -1) {
+			rule.comment = free_filter_comment(rule.comment);
 			return FALSE;
-		else
+		} else
 			rule.subj_ok = (i == 0);
 
 		/*
 		 * From:
 		 */
 		i = get_choice(INDEX_TOP + 6, _(txt_help_filter_from), text_from, (rule.subj_ok ? _(txt_no) : _(txt_yes)), (rule.subj_ok ? _(txt_yes) : _(txt_no)), (char *) 0, (char *) 0, (char *) 0);
-		if (i == -1)
+		if (i == -1) {
+			rule.comment = free_filter_comment(rule.comment);
 			return FALSE;
-		else
+		} else
 			rule.from_ok = rule.subj_ok ? (i != 0) : (i == 0);
 
 		/*
@@ -1040,9 +1097,10 @@ filter_menu(
 		else
 			i = get_choice(INDEX_TOP + 7, _(txt_help_filter_msgid), text_msgid, _(txt_full), _(txt_last), _(txt_only), _(txt_no), (char *) 0);
 
-		if (i == -1)
+		if (i == -1) {
+			rule.comment = free_filter_comment(rule.comment);
 			return FALSE;
-		else {
+		} else {
 			switch ((rule.subj_ok || rule.from_ok) ? i : i + 1) {
 				case 0:
 				case 4:
@@ -1081,8 +1139,10 @@ filter_menu(
 
 	buf[0] = '\0';
 
-	if (!prompt_menu_string(INDEX_TOP + 9, ptr_filter_lines, buf))
+	if (!prompt_menu_string(INDEX_TOP + 9, ptr_filter_lines, buf)) {
+		rule.comment = free_filter_comment(rule.comment);
 		return FALSE;
+	}
 
 	/*
 	 * Get the < > sign if any for the lines rule
@@ -1109,11 +1169,14 @@ filter_menu(
 	/*
 	 * Scoring value
 	 */
-	buf[0] = '\0';
-	show_menu_help(_(txt_filter_score_help)); /* FIXME: a sprintf() is necessary here */
+	snprintf(buf, sizeof(buf), _(txt_filter_score_help), SCORE_MAX);
+	show_menu_help(buf);
 
-	if (!prompt_menu_string(INDEX_TOP + 10, text_score, buf))
+	buf[0] = '\0';
+	if (!prompt_menu_string(INDEX_TOP + 10, text_score, buf)) {
+		rule.comment = free_filter_comment(rule.comment);
 		return FALSE;
+	}
 
 	/* check if a score has been entered */
 	if (buf[0] != '\0')
@@ -1127,8 +1190,10 @@ filter_menu(
 			rule.score = tinrc.score_select;
 	}
 
-	if (!rule.score) /* ignore 0 scores */
+	if (!rule.score) { /* ignore 0 scores */
+		rule.comment = free_filter_comment(rule.comment);
 		return FALSE;
+	}
 
 	/*
 	 * assure we are in range
@@ -1150,8 +1215,10 @@ filter_menu(
 	i = get_choice(INDEX_TOP + 11, _(txt_help_filter_time), ptr_filter_time,
 			_(txt_unlimited_time), text_time, double_time, quat_time, (char *) 0);
 
-	if (i == -1)
+	if (i == -1) {
+		rule.comment = free_filter_comment(rule.comment);
 		return FALSE;
+	}
 
 	rule.expire_time = i;
 
@@ -1188,12 +1255,16 @@ filter_menu(
 			       (argv[3][0] ? argv[3] : (char *) 0),
 			       (char *) 0);
 
-		if (i == -1)
+		if (i == -1) {
+			rule.comment = free_filter_comment(rule.comment);
 			return FALSE;
+		}
 
 		strcpy(rule.scope, ((i == 1) ? "*" : argv[i]));
-	} else
+	} else {
+		rule.comment = free_filter_comment(rule.comment);
 		return FALSE;
+	}
 
 	forever {
 		ch = prompt_slk_response(ch_default,
@@ -1204,6 +1275,7 @@ filter_menu(
 
 		case iKeyFilterEdit:
 			add_filter_rule(group, art, &rule); /* save the rule */
+			rule.comment = free_filter_comment(rule.comment);
 			if (!invoke_editor(filter_file, FILTER_FILE_OFFSET))
 				return FALSE;
 			unfilter_articles();
@@ -1214,6 +1286,7 @@ filter_menu(
 
 		case iKeyQuit:
 		case iKeyAbort:
+			rule.comment = free_filter_comment(rule.comment);
 			return FALSE;
 			/* keep lint quiet: */
 			/* FALLTHROUGH */
@@ -1222,7 +1295,9 @@ filter_menu(
 			/*
 			 * Add the filter rule and save it to the filter file
 			 */
-			return (add_filter_rule(group, art, &rule));
+			ret = add_filter_rule(group, art, &rule);
+			rule.comment = free_filter_comment(rule.comment);
+			return ret;
 			/* keep lint quiet: */
 			/* FALLTHROUGH */
 
@@ -1248,6 +1323,7 @@ quick_filter(
 	char txt[LEN];
 	int header, expire, icase;
 	struct t_filter_rule rule;
+	t_bool ret;
 
 	if (type == FILTER_KILL) {
 		header = group->attribute->quick_kill_header;
@@ -1282,12 +1358,11 @@ quick_filter(
 	rule.subj_ok = (header == FILTER_SUBJ_CASE_SENSITIVE || header == FILTER_SUBJ_CASE_IGNORE);
 
 	/* create an auto-comment. */
-	rule.comment = (struct t_filter_comment *) 0;	/* needs to be NULL, or add_filter_comment() will fail to create the first entry. */
 	if (type == FILTER_KILL)
-		snprintf(txt, sizeof(txt) - 1, "%s%s%c%s%s%s", _(txt_filter_rule_created), "'", iKeyGroupQuickKill, "' (", _(txt_help_article_quick_kill), ").");
+		snprintf(txt, sizeof(txt), "%s%s%c%s%s%s", _(txt_filter_rule_created), "'", iKeyGroupQuickKill, "' (", _(txt_help_article_quick_kill), ").");
 	else
-		snprintf (txt, sizeof(txt) - 1, "%s%s%c%s%s%s", _(txt_filter_rule_created), "'", iKeyGroupQuickAutoSel, "' (", _(txt_help_article_quick_select), ").");
-	rule.comment = (struct t_filter_comment *) add_filter_comment (rule.comment, (char *) txt);
+		snprintf(txt, sizeof(txt), "%s%s%c%s%s%s", _(txt_filter_rule_created), "'", iKeyGroupQuickAutoSel, "' (", _(txt_help_article_quick_select), ").");
+	rule.comment = add_filter_comment(NULL, txt);
 
 	rule.text[0] = '\0';
 	rule.icase = icase;
@@ -1295,7 +1370,9 @@ quick_filter(
 	rule.check_string = TRUE;
 	rule.score = (type == FILTER_KILL) ? tinrc.score_kill : tinrc.score_select;
 
-	return (add_filter_rule(group, art, &rule));
+	ret = add_filter_rule(group, art, &rule);
+	rule.comment = free_filter_comment(rule.comment);
+	return ret;
 }
 
 
@@ -1344,9 +1421,8 @@ quick_filter_select_posted_art(
 		strcpy(rule.scope, group->name);
 
 		/* create an auto-comment. */
-		rule.comment = (struct t_filter_comment *) 0;	/* needs to be NULL, or add_filter_comment() will fail to create the first entry. */
-		snprintf (txt, sizeof(txt) - 1, "%s%s", _(txt_filter_rule_created), "add_posted_to_filter=ON.");
-		rule.comment = (struct t_filter_comment *) add_filter_comment(rule.comment, txt);
+		snprintf(txt, sizeof(txt), "%s%s", _(txt_filter_rule_created), "add_posted_to_filter=ON.");
+		rule.comment = add_filter_comment(NULL, txt);
 
 		/*
 		 * Setup dummy article with posted articles subject
@@ -1381,6 +1457,7 @@ quick_filter_select_posted_art(
 			filtered = add_filter_rule(group, &art, &rule);
 			FreeIfNeeded(art.subject);
 		}
+		rule.comment = free_filter_comment(rule.comment);
 	}
 	return filtered;
 }
@@ -1421,11 +1498,9 @@ add_filter_rule(
 	ptr[i].gnksa_num = 0;
 	ptr[i].score = rule->score;
 	ptr[i].xref = (char *) 0;
-	ptr[i].xref_max = 0;
-	ptr[i].xref_score_cnt = 0;
 
 	if (rule->comment != NULL)
-		ptr[i].comment = (struct t_filter_comment *) copy_filter_comment(rule->comment, ptr[i].comment);
+		ptr[i].comment = copy_filter_comment(rule->comment, ptr[i].comment);
 
 	if (rule->scope[0] == '\0') /* replace empty scope with current group name */
 		ptr[i].scope = my_strdup(group->name);
@@ -1502,7 +1577,51 @@ add_filter_rule(
 		 * message-ids should be quoted
 		 */
 		if (rule->msgid_ok) {
-			STRCPY(sbuf, MSGID(art));
+			/*
+			 * If threading by references is set, and a quick kill is applied
+			 * (in group level), it is applied with the data of the root
+			 * article of the thread built by tin.
+			 * In case of threading by references, if tin's root is not the
+			 * *real* root of thread (which is the first entry in references
+			 * field) any applying of filtering for MSGID (or MSGID_LAST)
+			 * doesn't work, because the filter is applied with the data of
+			 * tin's root article which doesn't cover the other articles in
+			 * the thread.
+			 * So the thread remains open (in group level). To overcome this,
+			 * the first msgid from references field is taken in this case.
+			 */
+#if 1	/* This is what I think is more correct, but I did NOT test it! (urs) */
+			if (group->attribute->thread_arts == THREAD_REFS &&
+				(group->attribute->quick_kill_header == FILTER_MSGID ||
+				 group->attribute->quick_kill_header == FILTER_REFS_ONLY) &&
+				 art->refptr->parent != NULL)
+			{
+				/* not real parent, take first references entry as MID */
+				struct t_msgid *xptr;
+
+				for (xptr = art->refptr->parent; xptr->parent != NULL; xptr = xptr->parent)
+					;
+				STRCPY(sbuf, xptr->txt);
+			} else {
+				STRCPY(sbuf, MSGID(art));
+			}
+#else /* original logic by guido */
+			if (group->attribute->thread_arts != THREAD_REFS ||
+				origin != GROUP_LEVEL ||
+				(group->attribute->quick_kill_header != FILTER_MSGID &&
+				group->attribute->quick_kill_header != FILTER_MSGID_LAST)
+				(art->refptr->parent == NULL))
+			{
+				STRCPY(sbuf, MSGID(art));
+			} else {
+				/* not real parent, take first references entry as MID */
+				struct t_msgid *xptr;
+
+				for (xptr = art->refptr->parent; xptr->parent != NULL; xptr = xptr->parent)
+					;
+				STRCPY(sbuf, xptr->txt);
+			}
+#endif /* 1 */
 			sprintf(acBuf, REGEX_FMT, quote_wild(sbuf));
 			ptr[i].msgid = my_strdup(acBuf);
 			ptr[i].fullref = rule->fullref;
@@ -1538,28 +1657,21 @@ add_filter_rule(
  * tagged as being read BECAUSE they were killed. So, we retag
  * them as being unread. Selected articles will be un"select"ed.
  */
-int
+void
 unfilter_articles(
-	void) /* return value is always ignored */
+	void)
 {
-	int unkilled = 0;
 	int i;
 
 	for_each_art(i) {
 		arts[i].score = 0;
 		if (IS_KILLED(i)) {
-			arts[i].killed = FALSE;
+			arts[i].killed = ART_NOTKILLED;
 			arts[i].status = ART_UNREAD;
-			unkilled++;
 		}
-		if (IS_SELECTED(i)) {
+		if (IS_SELECTED(i))
 			arts[i].selected = FALSE;
-		}
 	}
-	num_of_killed_arts = 0;
-	num_of_selected_arts = 0;
-
-	return unkilled;
 }
 
 
@@ -1575,18 +1687,14 @@ filter_articles(
 {
 	char buf[LEN];
 	int num, inscope;
-/*	int score; */
-	int i, j, k;
-	struct t_filter *ptr; /*, *curr; */
+	int i, j;
+	struct t_filter *ptr;
 	struct regex_cache *regex_cache_subj = NULL;
 	struct regex_cache *regex_cache_from = NULL;
 	struct regex_cache *regex_cache_msgid = NULL;
 	struct regex_cache *regex_cache_xref = NULL;
 	t_bool filtered = FALSE;
 
-	num_of_killed_arts = 0;
-	num_of_selected_arts = 0;
-
 	/*
 	 * check if there are any global filter rules
 	 */
@@ -1642,7 +1750,7 @@ filter_articles(
 		 */
 		mesg[0] = '\0';				/* Clear system message field */
 
-		if (tinrc.kill_level == KILL_READ && IS_READ(i)) /* skip only when the article is read */
+		if (tinrc.kill_level == KILL_UNREAD && IS_READ(i)) /* skip only when the article is read */
 			continue;
 
 		for (j = 0; j < num; j++) {
@@ -1680,12 +1788,12 @@ filter_articles(
 					char *refs = NULL;
 					const char *myrefs = NULL;
 					const char *mymsgid = NULL;
-
-/*
- * TODO nice idea del'd; better apply one rule on all fitting
- * TODO articles, so we can switch to an appropriate algorithm
- * TODO for each kind of rule, including the deleted one.
- */
+					/*
+					 * TODO: nice idea del'd; better apply one rule on all
+					 *       fitting articles, so we can switch to an appropriate
+					 *       algorithm for each kind of rule, including the
+					 *       deleted one.
+					 */
 
 					/* myrefs does not need to be freed */
 
@@ -1791,11 +1899,15 @@ wait_message(1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lin
 
 				/*
 				 * Filter on Xref: lines
+				 *
+				 * 	Xref: news.foo.bar foo.bar:666 bar.bar:999
+				 * is turned into
+				 * 	foo.bar,bar.bar
 				 */
-				if (arts[i].xref && *arts[i].xref != '\0') {
-					if (ptr[j].xref_max > 0 || ptr[j].xref != NULL) {
-						char *s, *e;
-						int group_count = 0;
+				if (arts[i].xref && *arts[i].xref) {
+					if (ptr[j].score && ptr[j].xref != NULL) {
+						char *s, *e, *k;
+						t_bool skip = FALSE;
 
 						s = arts[i].xref;
 						while (*s && !isspace((int) *s))
@@ -1803,42 +1915,27 @@ wait_message(1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lin
 						while (*s && isspace((int) *s))
 							s++;
 
+						/* reformat */
+						k = e = my_malloc(strlen(s));
 						while (*s) {
-							e = s;
-							while (*e && *e != ':' && !isspace((int) *e))
-								e++;
+							if (*s == ':') {
+								*e++ = ',';
+								skip = TRUE;
+							}
+							if (*s != ':' && !isspace(*s) && !skip)
+								*e++ = *s;
+							if (isspace(*s))
+								skip = FALSE;
+							s++;
+						}
+						*--e = '\0';
 
-							if (ptr[j].xref_max > 0) {
-								strncpy(buf, s, e - s);
-								buf[e - s] = '\0';
-								for (k = 0; k < ptr[j].xref_score_cnt; k++) {
-									if (GROUP_MATCH(buf, ptr[j].xref_score_strings[k], TRUE)) {
-										group_count += ptr[j].xref_scores[k];
-										break;
-									}
-								}
-								if (k == ptr[j].xref_score_cnt)
-									group_count++;
+						if (ptr[j].xref != NULL) {
+							if (test_regex(k, ptr[j].xref, ptr[j].icase, &regex_cache_xref[j])) {
+								SET_FILTER(group, i, j);
 							}
-							if (ptr[j].xref != NULL) {
-								strncpy(buf, s, e - s);
-								buf[e - s] = '\0';
-								/* don't filter when we are actually in that group */
-								/* Group names shouldn't be case sensitive in any case. Whatever */
-								if (ptr[j].score > 0 || strcmp(group->name, buf) != 0) {
-									if (test_regex(buf, ptr[j].xref, ptr[j].icase, &regex_cache_xref[j]))
-										group_count = -1;
-								}
-							}
-							s = e;
-							while (*s && !isspace((int) *s))
-								s++;
-							while (*s && isspace((int) *s))
-								s++;
-						}
-						if (group_count == -1 || group_count>ptr[j].xref_max) {
-							SET_FILTER(group, i, j);
 						}
+						free(k);
 					}
 				}
 			}
@@ -1875,13 +1972,14 @@ wait_message(1, "FILTERED Lines arts[%d] > [%d]", arts[i].line_count, ptr[j].lin
 	if (mesg[0] == '\0') {
 		for_each_art(i) {
 			if (arts[i].score <= tinrc.score_limit_kill) {
-				arts[i].killed = TRUE;
-				num_of_killed_arts++;
+				if (arts[i].status == ART_UNREAD)
+					arts[i].killed = ART_KILLED_UNREAD;
+				else
+					arts[i].killed = ART_KILLED;
 				filtered = TRUE;
-				art_mark_read(group, &arts[i]);
+				art_mark(group, &arts[i], ART_READ);
 			} else if (arts[i].score >= tinrc.score_limit_select) {
 				arts[i].selected = TRUE;
-				num_of_selected_arts++;
 			}
 		}
 	}
