@@ -1,9 +1,9 @@
 /*
  *  Project   : tin - a Usenet reader
- *  Module    : tin_getline.c
+ *  Module    : getline.c
  *  Author    : Chris Thewalt & Iain Lea
  *  Created   : 1991-11-09
- *  Updated   : 2003-01-21
+ *  Updated   : 2003-08-03
  *  Notes     : emacs style line editing input package.
  *  Copyright : (c) Copyright 1991-99 by Chris Thewalt & Iain Lea
  *              Permission to use, copy, modify, and distribute this
@@ -136,7 +136,9 @@ tin_getline(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	if (str != NULL) {
 		wchar_t wbuf[LEN];
-		if (mbstowcs(wbuf, str, LEN - 1) != (size_t) -1) {
+
+		if (mbstowcs(wbuf, str, ARRAY_SIZE(wbuf) - 1) != (size_t) -1) {
+			wbuf[ARRAY_SIZE(wbuf) - 1] = (wchar_t) '\0';
 			for (i = 0; wbuf[i]; i++)
 				gl_addwchar(wbuf[i]);
 		}
@@ -453,11 +455,11 @@ gl_kill_back_word(
 		;
 
 	/* delete not alnum characters but graph characters */
-	for(; i >= 0 && iswgraph((wint_t) gl_buf[i]) && !iswalnum((wint_t) gl_buf[i]); --i)
+	for (; i >= 0 && iswgraph((wint_t) gl_buf[i]) && !iswalnum((wint_t) gl_buf[i]); --i)
 		;
 
 	/* delete all graph characters except '/' */
-	for(; i >= 0 && gl_buf[i] != (wchar_t) '/' && iswgraph((wint_t) gl_buf[i]); --i)
+	for (; i >= 0 && gl_buf[i] != (wchar_t) '/' && iswgraph((wint_t) gl_buf[i]); --i)
 		;
 #else
 	/* delete spaces */
