@@ -674,10 +674,11 @@ init_selfinfo(
 
 	/*
 	 * the site_config-file was the last chance to set the domainname
-	 * if it's still unset exit tin.
+	 * if it's still unset fall into no posting mode.
 	 */
 	if (domain_name[0] == '\0') {
 		error_message(_(txt_error_no_domain_name));
+		sleep(2);
 		force_no_post = TRUE;
 	}
 
