@@ -3,10 +3,10 @@
  *  Module    : keymap.c
  *  Author    : D. Nimmich, J. Faultless
  *  Created   : 2000-05-25
- *  Updated   : 2003-01-07
+ *  Updated   : 2003-05-14
  *  Notes     : This file contains key mapping routines and variables.
  *
- * Copyright (c) 2000-2003 Dirk Nimmich <nimmich@uni-muenster.de>
+ * Copyright (c) 2000-2003 Dirk Nimmich <nimmich@muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -162,7 +162,7 @@ static struct keymap Key = {
 		{ iKeyGroupNextUnreadArt, iKeyGroupNextUnreadArt, "NextUnreadArt" },
 		{ iKeyGroupPrevUnreadArt, iKeyGroupPrevUnreadArt, "PrevUnreadArt" },
 		{ iKeyGroupBugReport, iKeyGroupBugReport, "BugReport" },
-		{ iKeyGroupAutoSaveTagged, iKeyGroupAutoSaveTagged, "AutoSaveTagged" },
+		{ iKeyGroupAutoSave, iKeyGroupAutoSave, "AutoSave" },
 		{ iKeyGroupTagParts, iKeyGroupTagParts, "TagParts" },
 		{ iKeyGroupUntag, iKeyGroupUntag, "Untag" },
 		{ iKeyGroupMarkUnselArtRead, iKeyGroupMarkUnselArtRead, "MarkUnselArtRead" },
@@ -191,6 +191,12 @@ static struct keymap Key = {
 		{ iKeyHelpLastPage2, iKeyHelpLastPage2, "LastPage2" },
 		{ 0, 0, NULL }
 	},
+	{
+	{ 0, 0, "MarkRead" },
+		{ iKeyMarkReadTag, iKeyMarkReadTag, "Tag" },
+		{ iKeyMarkReadCur, iKeyMarkReadCur, "Cur" },
+		{ 0, 0, NULL }
+	},
 	{
 		{ 0, 0, "Nrctbl" },
 		{ iKeyNrctblAlternative, iKeyNrctblAlternative, "Alternative" },
@@ -224,12 +230,12 @@ static struct keymap Key = {
 		{ iKeyPageCancel, iKeyPageCancel, "Cancel" },
 		{ iKeyPageEditFilter, iKeyPageEditFilter, "EditFilter" },
 		{ iKeyPageFollowup, iKeyPageFollowup, "Followup" },
-		{ iKeyPageLastPage2, iKeyPageLastPage2, "LastPage2" },
+		{ iKeyPageLastPage, iKeyPageLastPage, "LastPage" },
 		{ iKeyPageKillThd, iKeyPageKillThd, "KillThd" },
 		{ iKeyPageNextUnreadArt, iKeyPageNextUnreadArt, "NextUnreadArt" },
 		{ iKeyPagePrevUnreadArt, iKeyPagePrevUnreadArt, "PrevUnreadArt" },
 		{ iKeyPageReply, iKeyPageReply, "Reply" },
-		{ iKeyPageAutoSaveTagged, iKeyPageAutoSaveTagged, "AutoSaveTagged" },
+		{ iKeyPageAutoSave, iKeyPageAutoSave, "AutoSave" },
 		{ iKeyPageGroupSel, iKeyPageGroupSel, "GroupSel" },
 		{ iKeyPageViewUrl, iKeyPageViewUrl, "ViewUrl" },
 		{ iKeyPageViewAttach, iKeyPageViewAttach, "ViewAttach" },
@@ -240,7 +246,7 @@ static struct keymap Key = {
 		{ iKeyPageCatchup, iKeyPageCatchup, "Catchup" },
 		{ iKeyPageEditArticle, iKeyPageEditArticle, "EditArticle" },
 		{ iKeyPageFollowupQuote, iKeyPageFollowupQuote, "FollowupQuote" },
-		{ iKeyPageFirstPage2, iKeyPageFirstPage2, "FirstPage2" },
+		{ iKeyPageFirstPage, iKeyPageFirstPage, "FirstPage" },
 		{ iKeyPageListThd, iKeyPageListThd, "ListThd" },
 		{ iKeyPageMail, iKeyPageMail, "Mail" },
 		{ iKeyPageNextArt, iKeyPageNextArt, "NextArt" },
@@ -291,9 +297,9 @@ static struct keymap Key = {
 	},
 	{
 		{ 0, 0, "PProc" },
-		{ iKeyPProcNone, iKeyPProcNone, "None" },
+		{ iKeyPProcNo, iKeyPProcNo, "No" },
 		{ iKeyPProcShar, iKeyPProcShar, "Shar" },
-		{ iKeyPProcUUDecode, iKeyPProcUUDecode, "UUDecode" },
+		{ iKeyPProcYes, iKeyPProcYes, "Yes" },
 		{ 0, 0, NULL }
 	},
 	{
@@ -346,7 +352,7 @@ static struct keymap Key = {
 		{ iKeyThreadCatchupNextUnread, iKeyThreadCatchupNextUnread, "CatchupNextUnread" },
 		{ iKeyThreadMarkArtRead, iKeyThreadMarkArtRead, "MarkArtRead" },
 		{ iKeyThreadBugReport, iKeyThreadBugReport, "BugReport" },
-		{ iKeyThreadAutoSaveTagged, iKeyThreadAutoSaveTagged, "AutoSaveTagged" },
+		{ iKeyThreadAutoSave, iKeyThreadAutoSave, "AutoSave" },
 		{ iKeyThreadUntag, iKeyThreadUntag, "Untag" },
 		{ iKeyThreadMarkThdUnread, iKeyThreadMarkThdUnread, "MarkThdUnread" },
 		{ iKeyThreadCatchup, iKeyThreadCatchup, "Catchup" },
@@ -422,8 +428,8 @@ static t_keynode *keys_feed_art_thread_regex_tag[] = {
  * older tin because the new action replaces the old one.
  */
 static t_keynode *keys_feed_post_process_type[] = {
-	&Key.Global.Abort, &Key.Global.Quit, &Key.PProc.None, &Key.PProc.Shar,
-	&Key.PProc.UUDecode,
+	&Key.Global.Abort, &Key.Global.Quit, &Key.PProc.No, &Key.PProc.Shar,
+	&Key.PProc.Yes,
 	NULL };
 
 static t_keynode *keys_feed_supersede_article[] = {
@@ -446,7 +452,7 @@ static t_keynode *keys_group_nav[] = {
 #ifndef DISABLE_PRINTING
 	&Key.Global.Print,
 #endif /* !DISABLE_PRINTING */
-	&Key.Group.Repost, &Key.Group.Save, &Key.Group.AutoSaveTagged,
+	&Key.Group.Repost, &Key.Group.Save, &Key.Group.AutoSave,
 	&Key.Global.SetRange, &Key.Global.SearchAuthF, &Key.Global.SearchAuthB,
 	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Global.SearchRepeat, &Key.Global.SearchBody,
 	&Key.Group.ReadBasenote, &Key.Group.ReadBasenote2,
@@ -485,9 +491,13 @@ static t_keynode *keys_info_nav[] = {
 	&Key.Global.ToggleHelpDisplay, &Key.Global.SearchSubjF,
 	&Key.Global.SearchSubjB, &Key.Global.SearchRepeat, &Key.Global.Quit, NULL };
 
+static t_keynode *keys_mark_read_tagged_current[] = {
+	&Key.Global.Abort, &Key.Global.Quit, &Key.MarkRead.Tag,
+	&Key.MarkRead.Cur, NULL };
+
 static t_keynode *keys_nrctbl_create[] = {
-   &Key.Global.Abort, &Key.Nrctbl.Quit, &Key.Nrctbl.Alternative,
-   &Key.Nrctbl.Create, &Key.Nrctbl.Default, NULL };
+	&Key.Global.Abort, &Key.Nrctbl.Quit, &Key.Nrctbl.Alternative,
+	&Key.Nrctbl.Create, &Key.Nrctbl.Default, NULL };
 
 static t_keynode *keys_page_nav[] = {
 	&Key.Global.Abort, &Key.Global.Zero, &Key.Global.One, &Key.Global.Two,
@@ -506,7 +516,7 @@ static t_keynode *keys_page_nav[] = {
 #ifndef DISABLE_PRINTING
 	&Key.Global.Print,
 #endif /* !DISABLE_PRINTING */
-	&Key.Page.Repost, &Key.Page.Save, &Key.Page.AutoSaveTagged,
+	&Key.Page.Repost, &Key.Page.Save, &Key.Page.AutoSave,
 	&Key.Global.SearchAuthF, &Key.Global.SearchAuthB, &Key.Global.SearchSubjF,
 	&Key.Global.SearchSubjB, &Key.Global.SearchRepeat, &Key.Global.SearchBody,
 	&Key.Page.TopThd, &Key.Page.BotThd, &Key.Page.NextThd, &Key.Page.NextThd2,
@@ -580,6 +590,7 @@ static t_keynode *keys_post_post[] = {
 	&Key.Post.Ispell,
 #endif /* HAVE_ISPELL */
 	&Key.Post.Edit, &Key.Global.Post, &Key.Post.Post2, &Key.Post.Post3,
+	&Key.Global.OptionMenu,
 	&Key.Post.Postpone, NULL };
 
 static t_keynode *keys_post_postpone[] = {
@@ -643,7 +654,7 @@ static t_keynode *keys_thread_nav[] = {
 #endif /* !NO_SHELL_ESCAPE */
 	&Key.Global.FirstPage, &Key.Global.LastPage, &Key.Global.LastViewed,
 	&Key.Global.SetRange, &Key.Global.Pipe, &Key.Thread.Mail,
-	&Key.Thread.Save, &Key.Thread.AutoSaveTagged, &Key.Thread.ReadArt,
+	&Key.Thread.Save, &Key.Thread.AutoSave, &Key.Thread.ReadArt,
 	&Key.Thread.ReadArt2, &Key.Thread.ReadNextArtOrThread, &Key.Global.Post,
 	&Key.Global.RedrawScr, &Key.Global.Down, &Key.Global.Down2,
 	&Key.Global.Up, &Key.Global.Up2, &Key.Global.PageUp, &Key.Global.PageUp2,
@@ -674,6 +685,7 @@ t_menukeymap menukeymap = {
 	{ keys_filter_quit_edit_save, NULL, NULL },
 	{ keys_group_nav, NULL, NULL },
 	{ keys_info_nav, NULL, NULL },
+	{ keys_mark_read_tagged_current, NULL, NULL },
 	{ keys_nrctbl_create, NULL, NULL },
 	{ keys_page_nav, NULL, NULL },
 	{ keys_pgp_mail, NULL, NULL },
@@ -854,8 +866,7 @@ check_duplicates(
 	for (ptr1 = keyptr1 + 1; ptr1->localkey != '\0'; ++ptr1) {
 		for (ptr2 = (keyptr1 == keyptr2) ? ptr1 + 1 : keyptr2 + 1; ptr2->localkey != '\0'; ++ptr2) {
 			if (ptr1->localkey == ptr2->localkey) {
-				fprintf(stderr, _(txt_keymap_conflict), printascii(buf, ptr1->localkey),
-								keyptr1->t, ptr1->t, keyptr2->t, ptr2->t);
+				wait_message(0, _(txt_keymap_conflict), printascii(buf, ptr1->localkey), keyptr1->t, ptr1->t, keyptr2->t, ptr2->t);
 				return FALSE;
 			}
 		}
@@ -900,7 +911,7 @@ processkey(
 
 
 #define KEYSEPS		" \t\n"
-/* TODO -> tin.h */
+/* TODO: -> tin.h */
 #define KEYMAP_FILE	"keymap"
 t_bool
 read_keymap_file(
@@ -909,7 +920,7 @@ read_keymap_file(
 	FILE *fp = (FILE *) 0;
 	char *line, *keydef, *kname;
 	char *map, *ptr;
-	char buf[LEN], buff[LEN];
+	char buf[LEN], buff[NAME_LEN + 1];
 	char key;
 	int i;
 	t_bool ret = TRUE;
@@ -927,7 +938,7 @@ read_keymap_file(
 	if (strlen(map)) {
 		if ((ptr = strchr(map, '.')))
 			*ptr = '\0';
-		snprintf(buff, sizeof(buff) - 1, "%s.%s", KEYMAP_FILE, map);
+		snprintf(buff, sizeof(buff), "%s.%s", KEYMAP_FILE, map);
 		joinpath(buf, rcdir, buff);
 		fp = fopen(buf, "r");
 	}
@@ -949,7 +960,7 @@ read_keymap_file(
 	FreeIfNeeded(map);
 
 	if (!fp)
-		return FALSE;
+		return TRUE; /* no keymap file is not an error */
 
 	while ((line = fgets(buf, sizeof(buf), fp)) != NULL) {
 		/*
@@ -965,7 +976,7 @@ read_keymap_file(
 		 * Warn about basic syntax errors
 		 */
 		if (keydef == NULL) {
-			fprintf(stderr, _(txt_keymap_missing_key), kname);
+			wait_message(0, _(txt_keymap_missing_key), kname);
 			ret = FALSE;
 			continue;
 		}
@@ -981,20 +992,23 @@ read_keymap_file(
 			switch (keydef[0]) {
 				case '^':
 					if (!(isupper((int)(unsigned char) keydef[1]))) {
-						fprintf(stderr, _(txt_keymap_invalid_key), keydef);
+						wait_message(0, _(txt_keymap_invalid_key), keydef);
 						ret = FALSE;
 						continue;
 					}
 					key = ctrl(keydef[1]);
 					break;
+
 				case 'S':		/* Only test 1st char - crude but effective */
 					key = ' ';
 					break;
+
 				case 'T':
 					key = ctrl('I');
 					break;
+
 				default:
-					fprintf(stderr, _(txt_keymap_invalid_key), keydef);
+					wait_message(0, _(txt_keymap_invalid_key), keydef);
 					ret = FALSE;
 					continue;
 			}
@@ -1011,10 +1025,10 @@ read_keymap_file(
 
 		/*
 		 * TODO: useful? shared keymaps (NFS-Home) may differ
-		 * depending on the OS (i.e. on tin has colr the other has not)
+		 * depending on the OS (i.e. on tin has color the other has not)
 		 */
 		if (keygroups[i] == NULL) {
-			fprintf(stderr, _(txt_keymap_invalid_name), kname);
+			wait_message(0, _(txt_keymap_invalid_name), kname);
 			ret = FALSE;
 			continue;
 		}
