@@ -3,7 +3,7 @@
  *  Module    : nntplib.c
  *  Author    : S. Barber & I. Lea
  *  Created   : 1991-01-12
- *  Updated   : 1997-12-22
+ *  Updated   : 2003-06-29
  *  Notes     : NNTP client routines taken from clientlib.c 1.5.11 (1991-02-10)
  *  Copyright : (c) Copyright 1991-99 by Stan Barber & Iain Lea
  *              Permission is hereby granted to copy, reproduce, redistribute
@@ -105,29 +105,38 @@ char *
 getserverbyfile(
 	const char *file)
 {
+	static char buf[256];
 #ifdef NNTP_ABLE
 	FILE *fp;
 	char *cp;
-	static char buf[256];
-#	ifdef HAVE_PUTENV
+#	if !defined(HAVE_SETENV) && defined(HAVE_PUTENV)
 	char tmpbuf[256];
 	char *new_env;
 	static char *old_env = NULL;
-#	endif /* HAVE_PUTENV */
-
+#	endif /* !HAVE_SETENV && HAVE_PUTENV */
+#endif
+	if (!read_news_via_nntp) {
+		STRCPY(buf, "local");	/* what if a server is named "local"? */
+		return buf;
+	}
+	if (read_saved_news) {
+		STRCPY(buf, "reading saved news");
+		return buf;
+	}
+#ifdef NNTP_ABLE
 	if (cmdline_nntpserver[0] != '\0') {
 		get_nntpserver(buf, cmdline_nntpserver);
-#	ifdef HAVE_PUTENV
+#	ifdef HAVE_SETENV
+		setenv("NNTPSERVER", buf, 1);
+#	else
+#		ifdef HAVE_PUTENV
 		sprintf(tmpbuf, "NNTPSERVER=%s", buf);
 		new_env = my_strdup(tmpbuf);
 		putenv(new_env);
 		FreeIfNeeded(old_env);
-		old_env = new_env;
-#	else
-#		ifdef HAVE_SETENV
-		setenv("NNTPSERVER", buf, 1);
-#		endif /* HAVE_SETENV */
-#	endif /* HAVE_PUTENV */
+		old_env = new_env; /* we are 'leaking' the last malloced mem at exit here */
+#		endif /* HAVE_PUTENV */
+#	endif /* HAVE_SETENV */
 		return buf;
 	}
 
@@ -323,7 +332,7 @@ get_tcp_socket(
 		strcpy(device, "/dev/tcp");
 
 	if ((s = t_open(device, O_RDWR, (struct t_info *) 0)) < 0){
-		t_error("t_open: can't t_open /dev/tcp"); /* FIXME: -> lang.c */
+		t_error(txt_error_topen);
 		return -EPROTO;
 	}
 	if (t_bind(s, (struct t_bind *) 0, (struct t_bind *) 0) < 0) {
@@ -345,7 +354,7 @@ get_tcp_socket(
 #		endif /* HAVE_INET_ATON */
 	{
 		if ((hp = gethostbyname(machine)) == NULL) {
-			my_fprintf(stderr, txt_gethostbyname, "gethostbyname() ", machine);
+			my_fprintf(stderr, _(txt_gethostbyname), "gethostbyname() ", machine);
 			t_close(s);
 			return -EHOSTUNREACH;
 		}
@@ -374,7 +383,7 @@ get_tcp_socket(
 	if (t_connect(s, callptr, (struct t_call *) 0) < 0) {
 		save_errno = t_errno;
 		if (save_errno == TLOOK)
-			fprintf(stderr, _("Server unavailable\n")); /* FIXME: -> lang.c */
+			fprintf(stderr, _(txt_error_server_unavailable));
 		else
 			t_error("t_connect");
 		t_free((char *) callptr, T_CALL);
@@ -408,7 +417,7 @@ get_tcp_socket(
 	struct hostent *hp;
 #			ifdef h_addr
 	int x = 0;
-	register char **cp;
+	char **cp;
 	static char *alist[2] = {0, 0};
 #			endif /* h_addr */
 	static struct hostent def;
@@ -417,7 +426,7 @@ get_tcp_socket(
 
 #			ifdef HAVE_GETSERVBYNAME
 	if ((sp = (struct servent *) getservbyname(service, "tcp")) == NULL) {
-		my_fprintf(stderr, _("%s/tcp: Unknown service.\n"), service); /* FIXME: -> lang.c */
+		my_fprintf(stderr, _(txt_error_unknown_service), service);
 		return -EHOSTUNREACH;
 	}
 #			else
@@ -493,7 +502,7 @@ get_tcp_socket(
 
 #			ifdef HAVE_INET_NTOA
 		if (x < 0)
-			my_fprintf(stderr, _("Trying %s"), (char *) inet_ntoa(sock_in.sin_addr)); /* FIXME: -> lang.c */
+			my_fprintf(stderr, _(txt_trying), (char *) inet_ntoa(sock_in.sin_addr));
 #			endif /* HAVE_INET_NTOA */
 
 #			if defined(__hpux) && defined(SVR4)	/* recommended by raj@cup.hp.com */
@@ -517,14 +526,14 @@ get_tcp_socket(
 
 		save_errno = errno;									/* Keep for later */
 #			ifdef HAVE_INET_NTOA
-		my_fprintf(stderr, _("\nConnection to %s: "), (char *) inet_ntoa(sock_in.sin_addr)); /* FIXME: -> lang.c */
+		my_fprintf(stderr, _(txt_connection_to), (char *) inet_ntoa(sock_in.sin_addr));
 		perror("");
 #			endif /* HAVE_INET_NTOA */
 		(void) s_close(s);
 	}
 
 	if (x < 0) {
-		my_fprintf(stderr, _("Giving up...\n")); /* FIXME: -> lang.c */
+		my_fprintf(stderr, _(txt_giving_up));
 		return -save_errno;					/* Return the last errno we got */
 	}
 #		else
@@ -602,10 +611,8 @@ get_tcp6_socket(
 	int s = -1, err = -1;
 	struct addrinfo hints, *res, *res0;
 
-	snprintf(mymachine, sizeof(mymachine) - 1, "%s", machine);
-	snprintf(myport, sizeof(myport) - 1, "%d", port);
-	mymachine[sizeof(mymachine) - 1] = '\0';
-	myport[sizeof(myport) - 1] = '\0';
+	snprintf(mymachine, sizeof(mymachine), "%s", machine);
+	snprintf(myport, sizeof(myport), "%d", port);
 
 /* just in case */
 #	ifdef AF_UNSPEC
@@ -642,7 +649,7 @@ get_tcp6_socket(
 	if (res0 != NULL)
 		freeaddrinfo(res0);
 	if (err < 0) {
-		my_fprintf(stderr, _("\nsocket or connect problem\n")); /* FIXME: -> lang.c */
+		my_fprintf(stderr, _(txt_error_socket_or_connect_problem));
 		return -1;
 	}
 	return s;
@@ -735,35 +742,44 @@ u_put_server(
 	const char *string)
 {
 	s_puts(string, nntp_wr_fp);
+#		ifdef DEBUG
+	debug_nntp(">>>", string);
+#		endif /* DEBUG */
 }
 
 
 /*
- * Send 'string' to the NNTP server, terminating it with CR and LF, as per ARPA
- * standard.
+ * Send 'string' to the NNTP server, terminating it with CR and LF, as per
+ * ARPA standard.
  *
  * Returns: Nothing.
  *
- *	Side effects:	Talks to the server.
+ *	Side effects: Talks to the server.
  *			Closes connection if things are not right.
  *
- * Note:	This routine flushes the buffer each time it is called. For large
- *			transmissions (i.e., posting news) don't use it. Instead, do the
- *			fprintf's yourself, and then a final fflush.
+ * Note: This routine flushes the buffer each time it is called. For large
+ *	      transmissions (i.e., posting news) don't use it. Instead, do the
+ *	      fprintf's yourself, and then a final fflush.
+ *       Only cache commands, don't cache data transmissions.
+ *
  */
 void
 put_server(
 	const char *string)
 {
-	/*
-	 * We remember the last thing we wrote, we will need to resend it if
-	 * we have to reconnect. Reconnection is handled by get_server()
-	 */
-	DEBUG_IO((stderr, "put_server(%s)\n", string));
-	strcpy(last_put, string);
-
-	s_puts(string, nntp_wr_fp);
-	s_puts("\r\n", nntp_wr_fp);
+	if (*string && strlen(string)) {
+		DEBUG_IO((stderr, "put_server(%s)\n", string));
+		s_puts(string, nntp_wr_fp);
+		s_puts("\r\n", nntp_wr_fp);
+#		ifdef DEBUG
+		debug_nntp(">>>", string);
+#		endif /* DEBUG */
+		/*
+		 * remember the last command we wrote to be able to resend it after a
+		 * reconnect. reconnection is handled by get_server()
+		 */
+		strcpy(last_put, string);
+	}
 	(void) s_flush(nntp_wr_fp);
 }
 
@@ -799,15 +815,17 @@ reconnect(
 	strcpy(buf, last_put);			/* Keep copy here, it will be clobbered a lot otherwise */
 
 	if (!nntp_open()) {
-
 		/*
 		 * Re-establish our current group and resend last command
 		 */
-		if (glob_group != NULL) {
+		if (curr_group != NULL) {
 			DEBUG_IO((stderr, _("Rejoin current group\n")));
-			sprintf(last_put, "GROUP %s", glob_group);
+			sprintf(last_put, "GROUP %s", curr_group->name);
 			put_server(last_put);
 			s_gets(last_put, NNTP_STRLEN, nntp_rd_fp);
+#		ifdef DEBUG
+			debug_nntp("<<<", last_put);
+#		endif /* DEBUG */
 			DEBUG_IO((stderr, _("Read (%s)\n"), last_put));
 		}
 		DEBUG_IO((stderr, _("Resend last command (%s)\n"), buf));
@@ -884,9 +902,13 @@ get_server(
 /*
  * Send "QUIT" command and close the connection to the server
  *
- * Side effects:	Closes the connection to the server.
- *					You can't use "put_server" or "get_server" after this
- *					routine is called.
+ * Side effects: Closes the connection to the server.
+ *	              You can't use "put_server" or "get_server" after this
+ *	              routine is called.
+ *
+ * TODO: remember servers response string and if it contains anything else
+ *       than just "." (i.e. transfer statistics) present it to the user?
+ *
  */
 void
 close_server(
@@ -895,7 +917,8 @@ close_server(
 	if (nntp_wr_fp == NULL || nntp_rd_fp == NULL)
 		return;
 
-	my_fputs(_(txt_disconnecting), stdout);
+	if (!batch_mode || verbose)
+		my_fputs(_(txt_disconnecting), stdout);
 	nntp_command("QUIT", OK_GOODBYE, NULL, 0);
 	quitting = TRUE;										/* Don't reconnect just for this */
 
