@@ -3,13 +3,13 @@
  *  Module    : nrctbl.c
  *  Author    : Sven Paulus <sven@tin.org>
  *  Created   : 1996-10-06
- *  Updated   : 1997-12-26
+ *  Updated   : 2003-01-18
  *  Notes     : This module does the NNTP server name lookup in
  *              ~/.tin/newsrctable and returns the real hostname
  *              and the name of the newsrc file for a given
  *              alias of the server.
  *
- * Copyright (c) 1996-2002 Sven Paulus <sven@tin.org>
+ * Copyright (c) 1996-2003 Sven Paulus <sven@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -20,10 +20,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Sven Paulus.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -57,7 +54,7 @@
 /*
  * local prototypes
  */
-static void write_newsrctable_file (void);
+static void write_newsrctable_file(void);
 
 
 /*
@@ -65,28 +62,29 @@ static void write_newsrctable_file (void);
  * create newsrctable file in local rc directory
  */
 static void
-write_newsrctable_file (
+write_newsrctable_file(
 	void)
 {
 	FILE *fp;
 
-	if ((fp = fopen(local_newsrctable_file, "w")) == (FILE *) 0)
+	if ((fp = fopen(local_newsrctable_file, "w")) == NULL)
 		return;
 
 	fprintf(fp, _(txt_nrctbl_info), PRODUCT, VERSION);
+	fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
 	fclose(fp);
 }
 
 
 /*
- * get_nntpserver ()
+ * get_nntpserver()
  * returns the FQDN of NNTP server by looking up a given
  * nickname or alias in the newsrctable
  * ---> extend to allow nameserver-lookups, if search in table
  *      failed
  */
 void
-get_nntpserver (
+get_nntpserver(
 	char *nntpserver_name,
 	char *nick_name)
 {
@@ -97,12 +95,12 @@ get_nntpserver (
 	int line_entry_counter;
 	t_bool found = FALSE;
 
-	if ((fp = fopen(local_newsrctable_file, "r")) != (FILE *) 0) {
-		while ((fgets(line, sizeof(line), fp) != NULL) && (!found)) {
+	if ((fp = fopen(local_newsrctable_file, "r")) != NULL) {
+		while ((fgets(line, sizeof(line), fp) != NULL) && !found) {
 			line_entry_counter = 0;
 
 			if (!strchr("# ;", line[0])) {
-				while ((line_entry = strtok(line_entry_counter ? 0 : line, " \t\n")) != 0) {
+				while ((line_entry = strtok(line_entry_counter ? NULL : line, " \t\n")) != NULL) {
 					line_entry_counter++;
 
 					if (line_entry_counter == 1)
@@ -128,7 +126,7 @@ get_nntpserver (
  * returns TRUE if name was found, FALSE if the search failed
  */
 int
-get_newsrcname (
+get_newsrcname(
 	char *newsrc_name,
 	const char *nntpserver_name) /* return value is always ignored */
 {
@@ -140,12 +138,12 @@ get_newsrcname (
 	int found = 0;
 	t_bool do_cpy = FALSE;
 
-	if ((fp = fopen(local_newsrctable_file, "r")) != (FILE *) 0) {
+	if ((fp = fopen(local_newsrctable_file, "r")) != NULL) {
 		while ((fgets(line, (int) sizeof(line), fp) != NULL) && (found != 1)) {
 			line_entry_counter = 0;
 
 			if (!strchr("# ;", line[0])) {
-				while ((line_entry = strtok(line_entry_counter ? 0 : line, " \t\n")) != 0) {
+				while ((line_entry = strtok(line_entry_counter ? NULL : line, " \t\n")) != NULL) {
 					line_entry_counter++;
 
 					if ((line_entry_counter == 1) && (!strcasecmp(line_entry, nntpserver_name))) {
@@ -170,73 +168,77 @@ get_newsrcname (
 			char tmp_newsrc[PATH_LEN];
 			int error = 0;
 
-			if (!strfpath (name_found, tmp_newsrc, sizeof (tmp_newsrc), &CURR_GROUP)) {
-					my_fprintf (stderr, _("couldn't expand %s\n"), name_found);
+			if (!strfpath(name_found, tmp_newsrc, sizeof(tmp_newsrc), NULL)) {
+					my_fprintf(stderr, _("couldn't expand %s\n"), name_found);
 					error = 1;
 			} else {
 				if (tmp_newsrc[0] == '/')
-					(void) strcpy (newsrc_name, tmp_newsrc);
+					(void) strcpy(newsrc_name, tmp_newsrc);
 				else
-					joinpath (newsrc_name, homedir, tmp_newsrc);
+					joinpath(newsrc_name, homedir, tmp_newsrc);
 			}
-			(void) strcpy (dir, newsrc_name);
-			if (strchr (dir, '/'))
-				*strrchr (dir, '/') = (char) 0;
+			(void) strcpy(dir, newsrc_name);
+			if (strchr(dir, '/'))
+				*strrchr(dir, '/') = (char) 0;
 
 			if (!error) {
 			/* FIXME - write a global permssion check routine */
-				if (access (dir, X_OK)) {
-					my_fprintf (stderr, _(txt_error_no_enter_permission), dir);
+				if (access(dir, X_OK)) {
+					my_fprintf(stderr, _(txt_error_no_enter_permission), dir);
 					error = 1;
-				} else if (access (newsrc_name, F_OK)) {
-					my_fprintf (stderr, _(txt_error_no_such_file), newsrc_name);
+				} else if (access(newsrc_name, F_OK)) {
+					my_fprintf(stderr, _(txt_error_no_such_file), newsrc_name);
 					error = 2;
-				} else if (access (dir, R_OK)) {
-					my_fprintf (stderr, _(txt_error_no_read_permission), dir);
+				} else if (access(dir, R_OK)) {
+					my_fprintf(stderr, _(txt_error_no_read_permission), dir);
 					error = 1;
-				} else if (access (newsrc_name, R_OK)) {
-					my_fprintf (stderr, _(txt_error_no_read_permission), newsrc_name);
+				} else if (access(newsrc_name, R_OK)) {
+					my_fprintf(stderr, _(txt_error_no_read_permission), newsrc_name);
 					error = 1;
-				} else if (access (dir, W_OK)) {
-					my_fprintf (stderr, _(txt_error_no_write_permission), dir);
+				} else if (access(dir, W_OK)) {
+					my_fprintf(stderr, _(txt_error_no_write_permission), dir);
 					error = 1;
-				} else if (access (newsrc_name, W_OK)) {
-					my_fprintf (stderr, _(txt_error_no_write_permission), newsrc_name);
+				} else if (access(newsrc_name, W_OK)) {
+					my_fprintf(stderr, _(txt_error_no_write_permission), newsrc_name);
 					error = 1;
 				}
 			}
 			if (error) {
 				char ch;
-				char default_ch = map_to_local (iKeyNrctblAlternative, &menukeymap.nrctbl_create);
+				char default_ch = map_to_local(iKeyNrctblAlternative, &menukeymap.nrctbl_create);
 
 				do {
 					/* very ugly code, but curses is not initialized yet */
 					if (error >= 2) {
-						default_ch = map_to_local (iKeyNrctblCreate, &menukeymap.nrctbl_create);
+						default_ch = map_to_local(iKeyNrctblCreate, &menukeymap.nrctbl_create);
 						printf("%s%c\b", _(txt_nrctbl_create), default_ch);
 					} else
 						printf("%s%c\b", _(txt_nrctbl_default), default_ch);
 
-					if ((ch = (char) ReadCh ()) == '\r' || ch == '\n')
+					if ((ch = (char) ReadCh()) == '\r' || ch == '\n')
 						ch = default_ch;
-				} while (!strchr (menukeymap.nrctbl_create.localkeys, ch));
+				} while (!strchr(menukeymap.nrctbl_create.localkeys, ch));
 				printf("%c\n", ch);
 
-				switch (map_to_default (ch, &menukeymap.nrctbl_create)) {
+				switch (map_to_default(ch, &menukeymap.nrctbl_create)) {
 					case iKeyNrctblCreate:
 						/* FIXME this doesn't check if we could create the file */
 						return TRUE;
+
 					case iKeyNrctblDefault:
 						joinpath(newsrc_name, homedir, ".newsrc");
 						return TRUE;
+
 					case iKeyNrctblAlternative:
 						sprintf(name_found, ".newsrc-%s", nntpserver_name);
 						joinpath(newsrc_name, homedir, name_found);
 						return TRUE;
+
 					case iKeyNrctblQuit:
-						exit (EXIT_SUCCESS);
+						exit(EXIT_SUCCESS);
 						/* keep lint quiet: */
 						/* FALLTHROUGH */
+
 					case ESC:
 					default:
 						return TRUE;
