@@ -1,14 +1,14 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : refs.c
- *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
+ *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1996-05-09
- *  Updated   : 1998-01-04
+ *  Updated   : 2003-07-22
  *  Notes     : Cacheing of message ids / References based threading
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2003 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1996-2003 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -19,10 +19,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Jason Faultless.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -45,7 +42,7 @@
 #endif /* !TIN_H */
 
 #define MAX_REFS	100			/* Limit recursion depth */
-#define REF_SEP	" "			/* Separator chars in ref headers */
+#define REF_SEP	" \t"			/* Separator chars in ref headers */
 
 /* Produce disgusting amounts of output to help me tame this thing */
 #undef DEBUG_REFS
@@ -87,7 +84,7 @@ static t_bool sort_ascend;
 static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
 
 /*
- * This part of the code deals with the cacheing and retrieval
+ * This part of the code deals with the caching and retrieval
  * of Message-ID and References headers
  *
  * Rationale:
@@ -96,7 +93,7 @@ static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
  *    period increases, so does the redundancy of data.
  *    At the time of writing, comp.os.ms-windows.advocacy held ~850
  *    articles. The references fields contained 192k of text, of which
- *    169k was saved using the new cacheing.
+ *    169k was saved using the new caching.
  *
  *    When threading on Refs, a much better view of the original thread
  *    can be built up using this data, and threading is much faster
@@ -361,8 +358,7 @@ parse_references(
 	DEBUG_PRINT((dbgfd, "parse_references: %s\n", r));
 
 	/*
-	 * Break the refs down, using a space as delimiters
-	 * A msgid can't contain a space, right ?
+	 * Break the refs down, using REF_SEP as delimiters
 	 */
 	if ((ptr = strtok(r, REF_SEP)) == NULL)
 		return NULL;
@@ -384,9 +380,9 @@ parse_references(
 
 /*
  * Reconstruct the References: field from the parent pointers
- * NB: The original Refs: can be no longer than HEADER_LEN (see open.c)
- *     Broken headers sometimes have malformed or circular reference
- *	    lists, which we strive to work around.
+ * NB: In deep threads this can lead to a very long line. If you want to use
+ *     this function to build a Reference: line for posting be aware that the
+ *     server might refuse long lines -- short it accordingly!
  */
 static char *
 _get_references(
@@ -394,34 +390,33 @@ _get_references(
 	int depth)
 {
 	char *refs;
-	static size_t len;							/* Accumulated size */
+	static size_t len;		/* Accumulated size */
+	static size_t pos;		/* current insertion position */
 
+	if (depth == 1)
+		len = 0;
+
+	len += strlen(refptr->txt) + 1;	/* msgid + space */
 	if (refptr->parent == NULL || depth > MAX_REFS) {
 
 #ifdef DEBUG_REFS
 		if (depth > MAX_REFS)
 			error_message("Warning: Too many refs near to %s. Truncated\n", refptr->txt);
 #endif /* DEBUG_REFS */
-		refs = my_malloc(HEADER_LEN);
-		len = 0;
+		refs = my_malloc(len + 1);	/* total length + nullbyte */
+		pos = 0;
 	} else
 		refs = _get_references(refptr->parent, depth + 1);
 
-	/*
-	 * Attempt at damage limitation in case of broken Refs fields
-	 */
-	if (len + strlen(refptr->txt) < (998 - strlen("References: "))) {
-		sprintf(refs + len, "%s ", refptr->txt);
-		len = strlen(refs);
-	}
+	sprintf(refs + pos, "%s ", refptr->txt);
+	pos = strlen(refs);
 
 	return refs;
 }
 
 
 /*
- * A wrapper to the above, null terminate the string and shrink it
- * to correct size
+ * A wrapper to the above, null terminate the string
  */
 char *
 get_references(
@@ -436,7 +431,6 @@ get_references(
 	refs = _get_references(refptr, 1);
 	len = strlen(refs);
 	refs[len - 1] = '\0';
-	refs = my_realloc(refs, len);
 
 	return refs;
 }
@@ -527,7 +521,7 @@ dump_msgids(
  * These pointers are automatically set up when we read in the
  * headers for a group.
  *
- * It remains for us to fill in the .thread and .inthread ptrs in
+ * It remains for us to fill in the .thread and .prev ptrs in
  * each article that exists in the spool, using the intelligence of
  * the reference tree to locate the 'next' article in the thread.
  *
@@ -721,7 +715,7 @@ find_next(
 
 
 /*
- * Run the .thread and .inthread pointers through the members of this
+ * Run the .thread and .prev pointers through the members of this
  * thread.
  */
 static void
@@ -741,7 +735,7 @@ build_thread(
 	 */
 	while ((newptr = find_next(ptr)) != NULL) {
 
-		arts[newptr->article].inthread = TRUE;
+		arts[newptr->article].prev = ptr->article;
 		arts[ptr->article].thread = newptr->article;
 
 		ptr = newptr;
@@ -787,7 +781,7 @@ thread_by_reference(
 			(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
 			(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
 			(arts[i].refptr->child) ? arts[i].refptr->child->article : -2,
-			arts[i].inthread, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
+			arts[i].prev, arts[i].thread, arts[i].refptr->txt, arts[i].subject);
 	}
 
 	fclose(dbgfd);
@@ -822,7 +816,7 @@ collate_subjects(
 		/*
 		 * Ignore already threaded and expired arts
 		 */
-		if (arts[i].inthread || IGNORE_ART(i))
+		if (arts[i].prev >= 0 || IGNORE_ART(i))
 			continue;
 
 		/*
@@ -837,16 +831,17 @@ collate_subjects(
 			 * is that we have to add later threads onto the end of the
 			 * previous thread
 			 */
-			if (((arts[i].subject == arts[j].subject) ||
-						   ((arts[i].part || arts[i].patch) &&
-							 arts[i].archive == arts[j].archive))) {
-/*DEBUG_PRINT((dbgfd, "RES: %d is now inthread, at end of %d\n", i, j));*/
+			if (
+					(arts[i].subject == arts[j].subject) /* ||
+					(arts[i].archive && arts[j].archive && (arts[i].archive->name == arts[j].archive->name)) */
+			) {
+/*DEBUG_PRINT((dbgfd, "RES: %d is now previous, at end of %d\n", i, j));*/
 
 				for (art = j; arts[art].thread >= 0; art = arts[art].thread)
 					;
 
 				arts[art].thread = i;
-				arts[i].inthread = TRUE;
+				arts[i].prev = art;
 			}
 		}
 
@@ -879,7 +874,7 @@ void
 build_references(
 	struct t_group *group)
 {
-	char *s;
+	char *s, *t;
 	int i;
 	struct t_article *art;
 	struct t_msgid *refs;
@@ -887,15 +882,14 @@ build_references(
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
-	if (group->attribute && group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
+	if (group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
 		sort_arts(group->attribute->sort_art_type);
 
-	if (group->attribute)
-		sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
-		               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
-		               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
-		               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
-			       group->attribute->sort_art_type == SORT_ARTICLES_BY_LINES_ASCEND);
+	sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
+	               group->attribute->sort_art_type == SORT_ARTICLES_BY_LINES_ASCEND);
 
 #ifdef DEBUG_REFS
 	dbgfd = fopen("Refs.dump", "w");
@@ -911,13 +905,14 @@ build_references(
 		art = &arts[i];
 
 		if (art->refs) {
-
 			strip_line(art->refs);
 
 			/*
 			 * Add the last ref, and then trim it to save wasting time adding
 			 * it again later
 			 * Check for circular references to current article
+			 *
+			 * TODO: do this in a single pass
 			 */
 			while (((s = strrchr(art->refs, ' ')) != NULL) && (!strcmp(art->msgid, s + 1))) {
 				/*
@@ -926,19 +921,27 @@ build_references(
 				DEBUG_PRINT((dbgfd, "removing circular reference to%s\n", s));
 				*s = '\0';
 			}
+			while (((t = strrchr(art->refs, '\t')) != NULL) && (!strcmp(art->msgid, t + 1))) {
+				/*
+				 * Remove circular reference to current article
+				 */
+				DEBUG_PRINT((dbgfd, "removing circular reference to%s\n", t));
+				*t = '\0';
+			}
+			if (t > s)
+				s = t;
+
 			if (s != NULL) {
 				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, s + 1, NULL));
 				*s = '\0';
 			} else {
 				art->refptr = add_msgid(MSGID_REF, art->msgid, add_msgid(REF_REF, art->refs, NULL));
-				free(art->refs);
-				art->refs = (char *) 0;
+				FreeAndNull(art->refs);
 			}
-
 		} else
 			art->refptr = add_msgid(MSGID_REF, art->msgid, NULL);
 
-		free(art->msgid);					/* Now cached - discard this */
+		FreeAndNull(art->msgid);	/* Now cached - discard this */
 	}
 
 	DEBUG_PRINT((dbgfd, "REFS phase\n"));
@@ -961,7 +964,7 @@ build_references(
 		if (art->refptr->parent)
 			add_msgid(REF_REF, art->refptr->parent->txt, refs);
 
-		free(art->refs);
+		FreeAndNull(art->refs);
 	}
 
 #ifdef DEBUG_REFS
