@@ -3,7 +3,7 @@
  *  Module    : screen.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2002-11-08
+ *  Updated   : 2003-08-03
  *  Notes     :
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -77,7 +77,7 @@ info_message(
 	fcol(tinrc.col_message);
 #endif /* HAVE_COLOR */
 
-	vsprintf(mesg, fmt, ap);
+	vsnprintf(mesg, sizeof(mesg), fmt, ap);
 
 	center_line(cLINES, FALSE, mesg);	/* center the message at screen bottom */
 
@@ -108,7 +108,7 @@ wait_message(
 	fcol(tinrc.col_message);
 #endif /* HAVE_COLOR */
 
-	vsprintf(mesg, fmt, ap);
+	vsnprintf(mesg, sizeof(mesg), fmt, ap);
 	my_fputs(mesg, stdout);
 
 #ifdef HAVE_COLOR
@@ -138,7 +138,7 @@ error_message(
 
 	errno = 0;
 	clear_message();
-	vsprintf(mesg, fmt, ap);
+	vsnprintf(mesg, sizeof(mesg), fmt, ap);
 
 	my_fputs(mesg, stderr);	/* don't use my_fprintf() here due to %format chars */
 	my_fflush(stderr);
@@ -174,7 +174,7 @@ perror_message(
 
 	clear_message();
 
-	vsnprintf(buf, sizeof(LEN) - 1, fmt, ap);
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 
 	va_end(ap);
 
@@ -224,6 +224,8 @@ center_line(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	if ((len = mbstowcs(wbuffer, buffer, ARRAY_SIZE(wbuffer) - 1)) <= 0)
 		len = strlen(buffer);
+	else
+		wbuffer[ARRAY_SIZE(wbuffer) - 1] = (wchar_t) '\0';
 	if ((width = wcswidth(wbuffer, ARRAY_SIZE(wbuffer) - 1)) <= 0)
 		width = len;
 #else
@@ -250,7 +252,7 @@ center_line(
 
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	if (width >= cCOLS) {
-		wcspart(wbuffer2, wbuffer, cCOLS - 6, ARRAY_SIZE(wbuffer2) - 5);
+		wcspart(wbuffer2, wbuffer, cCOLS - 6, ARRAY_SIZE(wbuffer2) - 5, TRUE);
 		mbstowcs(suffix_buf, TRUNC_TAIL, ARRAY_SIZE(suffix_buf) - 1);
 		wcsncat(wbuffer2, suffix_buf, 4);
 		wcstombs(buffer, wbuffer2, sizeof(buffer) - 1);
@@ -260,7 +262,7 @@ center_line(
 		char *buf;
 
 		buf = my_strdup(buffer);
-		snprintf(buffer, sizeof(buffer) - 1, "%-.*s%s", cCOLS - 6, buf, TRUNC_TAIL);
+		snprintf(buffer, sizeof(buffer), "%-.*s%s", cCOLS - 6, buf, TRUNC_TAIL);
 		free(buf);
 	}
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
@@ -342,7 +344,7 @@ erase_arrow(
 
 void
 show_title(
-	char *title)
+	const char *title)
 {
 	int col;
 
@@ -359,7 +361,7 @@ show_title(
 		fcol(tinrc.col_normal);
 #endif /* HAVE_COLOR */
 	}
-	center_line(0, TRUE, title);
+	center_line(0, TRUE, title); /* wastes some space on the left */
 }
 
 
@@ -384,7 +386,7 @@ void
 spin_cursor(
 	void)
 {
-	static const char buf[] = "|/-\\|/-\\ "; /* don't remove the taling sapce! */
+	static const char buf[] = "|/-\\|/-\\ "; /* don't remove the tailing space! */
 	static unsigned short int i = 0;
 
 	if (batch_mode)
@@ -413,41 +415,54 @@ show_progress(
 	long count,
 	long total)
 {
-#ifdef HAVE_GETTIMEOFDAY
+	char display[LEN];
+	int ratio;
+	time_t curr_time;
+	static char last_display[LEN];
 	static const char *last_txt;
 	static int last_length;
-	static int last_total;
-	static int last_count;
+	static int last_ratio;
+	static long last_total;
+	static time_t last_update;
+#ifdef HAVE_GETTIMEOFDAY
+	static long last_count;
 	static int average;
 	static int samples;
 	static int sum;
 	static struct timeval last_time;
 	static struct timeval this_time;
 	int time_diff;
-	int count_diff;
 	int secs_left;
+	long count_diff;
+#endif /* HAVE_GETTIMEOFDAY */
 
 	if (batch_mode || count <= 0 || total == 0)
 		return;
 
 	/* If this is a new progress meter, start recalculating */
-	if ((last_txt != txt) || (last_total != total))
+	if ((last_txt != txt) || (last_total != total)) {
 		last_length = 0;
-
-	MoveCursor(cLINES, 0);
-
-	/* Erase the previous text */
-	if (last_length) {
-		my_printf("%*s", last_length, " ");
-		MoveCursor(cLINES, 0);
+		last_ratio = -1;
+		last_display[0] = '\0';
+		last_update = time(NULL) - 2;
 	}
 
-	if (!last_length) {
-#	ifdef HAVE_COLOR
-		fcol(tinrc.col_message);
-#	endif /* HAVE_COLOR */
-		/* Don't print a time left this time */
-		/* last_length = */ my_printf("%s %3d%%", txt, (int) (count * 100 / total));
+	curr_time = time(NULL);
+	ratio = (int) ((count * 100) / total);
+	if ((ratio == last_ratio) && (curr_time - last_update < 2))
+		/*
+		 * return if ratio did not change and less than 1-2 seconds since last
+		 * update to reduce output
+		 */
+		return;
+
+	last_update = curr_time;
+
+#ifdef HAVE_GETTIMEOFDAY
+	if (last_length == 0) {
+		/* Don't print a "time remaining" this time */
+		snprintf(display, sizeof(display), "%s %3d%%", txt, ratio);
+		display[sizeof(display) - 1] = '\0';
 		last_length = strlen(txt) + 5;
 
 		/* Reset the variables */
@@ -463,6 +478,16 @@ show_progress(
 		if (!count_diff) /* avoid div by zero */
 			count_diff++;
 
+		/*
+		 * Calculate a running average based on the last 20 samples. For the
+		 * first 19 samples just add all and divide by the number of samples.
+		 * From the 20th sample on use only the last 20 samples to calculate
+		 * the running averave. To make things easier we don't want to store
+		 * and keep track of all of them, so we assume that the first sample
+		 * was close to the current average and substract it from sum. Then,
+		 * the new sample is added to the sum and the sum is divided by 20 to
+		 * get the new average.
+		 */
 		if (samples == 20) {
 			sum -= average;
 			sum += (time_diff / count_diff);
@@ -480,39 +505,38 @@ show_progress(
 		if (secs_left < 0)
 			secs_left = 0;
 
-#	ifdef HAVE_COLOR
-		fcol(tinrc.col_message);
-#	endif /* HAVE_COLOR */
 		/* TODO: -> lang.c, difficult with hardcoded last_length */
-		/* last_length = */ my_printf("%s %3d%% (%d:%02d remaining)", txt, (int) (count * 100 / total), secs_left / 60, secs_left % 60);
+		snprintf(display, sizeof(display), "%s %3d%% (%d:%02d remaining)", txt, ratio, secs_left / 60, secs_left % 60);
 		last_length = strlen(txt) + 21 + secs_left / 600;
 	}
 
-	my_flush();
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_normal);
-#	endif /* HAVE_COLOR */
-
-	last_txt = txt;
-	last_total = total;
 	last_count = count;
 	gettimeofday(&last_time, NULL);
 
-#else
-	if (batch_mode || count <= 0 || total == 0)
-		return;
-
-	MoveCursor(cLINES, 0);
-
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_message);
-#	endif /* HAVE_COLOR */
-
-	my_printf("%s %3d%%", txt, (int) (count * 100 / total));
-	my_flush();
-
-#	ifdef HAVE_COLOR
-	fcol(tinrc.col_normal);
-#	endif /* HAVE_COLOR */
+#else /* HAVE_GETTIMEOFDAY */
+	snprintf(display, sizeof(display), "%s %3d%%", txt, ratio);
 #endif /* HAVE_GETTIMEOFDAY */
+
+	/* Only display text if it changed from last time */
+	if (strcmp(display, last_display)) {
+		clear_message();
+		MoveCursor(cLINES, 0);
+
+#	ifdef HAVE_COLOR
+		fcol(tinrc.col_message);
+#	endif /* HAVE_COLOR */
+
+		my_printf("%s", display);
+
+#	ifdef HAVE_COLOR
+		fcol(tinrc.col_normal);
+#	endif /* HAVE_COLOR */
+
+		my_flush();
+		STRCPY(last_display, display);
+	}
+
+	last_txt = txt;
+	last_total = total;
+	last_ratio = ratio;
 }
