@@ -3,7 +3,7 @@
  *  Module    : search.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-07
+ *  Updated   : 2003-05-15
  *  Notes     :
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -57,8 +57,6 @@ static int search_group(t_bool forward, int current_art, char *searchbuff, int (
  */
 #define MATCH_MSG	(mesg[0] ? mesg : _(txt_no_match))
 
-/* char i_key_search_last; */				/* for repeated search */
-
 /*
  * Kludge to maintain some internal state for body search
  */
@@ -123,7 +121,7 @@ get_search_pattern(
 	/*
 	 * A gross hack to simulate substrings with wildmat()
 	 */
-/* TODO somehow use REGEX_FMT here? */
+/* TODO: somehow use REGEX_FMT here? */
 	sprintf(tmpbuf, "*%s*", def);
 	return tmpbuf;
 }
@@ -158,7 +156,7 @@ search_config(
 			if (n > last)
 				n = 0;
 		}
-		if (REGEX_MATCH(option_table[n].txt->opt, buf, TRUE)) {
+		if (REGEX_MATCH(_(option_table[n].txt->opt), buf, TRUE)) {
 			result = n;
 			break;
 		}
@@ -207,7 +205,7 @@ search_active(
 		 * Get the group name & description into buf2
 		 */
 		if (show_description && active[my_group[i]].description) {
-			snprintf(buf2, sizeof(buf2) - 1, "%s %s", active[my_group[i]].name, active[my_group[i]].description);
+			snprintf(buf2, sizeof(buf2), "%s %s", active[my_group[i]].name, active[my_group[i]].description);
 			ptr = buf2;
 		} else
 			ptr = active[my_group[i]].name;
@@ -235,14 +233,9 @@ body_search(
 	char *searchbuf)
 {
 	char *line;
-	char group_path[PATH_LEN];
 	t_openartinfo artinfo;
 
-	if (!read_news_via_nntp || CURR_GROUP.type != GROUP_TYPE_NEWS)
-		make_group_path(CURR_GROUP.name, group_path);
-
-	memset(&artinfo, 0, sizeof(t_openartinfo));
-	switch (art_open(TRUE, &arts[i], group_path, &artinfo, FALSE)) {
+	switch (art_open(TRUE, &arts[i], curr_group, &artinfo, FALSE)) {
 		case ART_ABORT:					/* User 'q'uit */
 			art_close(&artinfo);
 			return -1;
@@ -315,7 +308,7 @@ author_search(
 	if (arts[i].name == NULL)
 		ptr = arts[i].from;
 	else
-		snprintf(buf, sizeof(buf) - 1, "%s <%s>", arts[i].name, arts[i].from);
+		snprintf(buf, sizeof(buf), "%s <%s>", arts[i].name, arts[i].from);
 
 	return (REGEX_MATCH(ptr, searchbuf, TRUE)) ? 1 : 0;
 }
@@ -344,7 +337,6 @@ search_group(
 	char *searchbuff,
 	int (*search_func) (int i, char *searchbuff))
 {
-	char group_path[PATH_LEN];
 	int i;
 
 	if (grpmenu.curr < 0) {
@@ -352,9 +344,6 @@ search_group(
 		return -1;
 	}
 
-	if (!read_news_via_nntp || CURR_GROUP.type != GROUP_TYPE_NEWS)
-		make_group_path(CURR_GROUP.name, group_path);
-
 	i = current_art;
 
 	do {
@@ -439,7 +428,7 @@ search_article(
 	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_art, HIST_ART_SEARCH)))
 		return FALSE;
 
-	if (tinrc.wildcard && !(compile_regex(pattern, &srch, PCRE_EXTENDED | PCRE_CASELESS)))
+	if (tinrc.wildcard && !(compile_regex(pattern, &srch, PCRE_CASELESS)))
 		return -1;
 
 	srch_lineno = -1;
@@ -495,6 +484,7 @@ search_article(
  */
 int
 search_body(
+	struct t_group *group,
 	int current_art,
 	t_bool repeat)
 {
@@ -517,7 +507,7 @@ search_body(
 	/*
 	 * Count up the articles to be processed for the progress meter
 	 */
-	if (CURR_GROUP.attribute->show_only_unread) {
+	if (group->attribute->show_only_unread) {
 		for (i = 0; i < grpmenu.max; i++)
 			total_cnt += new_responses(i);
 	} else {
@@ -530,7 +520,7 @@ search_body(
 	/*
 	 * Pre-compile if we're using full regex
 	 */
-	if (tinrc.wildcard && !(compile_regex(buf, &srch_regex, PCRE_EXTENDED | PCRE_CASELESS)))
+	if (tinrc.wildcard && !(compile_regex(buf, &srch_regex, PCRE_CASELESS)))
 		return -1;
 
 	srch_lineno = -1;
