@@ -3,7 +3,7 @@
  *  Module    : select.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-07
+ *  Updated   : 2003-07-20
  *  Notes     :
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
@@ -74,7 +74,6 @@ static void yank_active_file(void);
  */
 t_menu selmenu = { 1, 0, 0, 0, show_selection_page, draw_group_arrow };
 
-/* char i_key_search_last;	*/			/* for repeated search */
 
 static int
 select_left(
@@ -129,10 +128,8 @@ selection_page(
 		}
 
 		set_xclick_on();
-		ch = handle_keypad(select_left, select_right, &menukeymap.select_nav);
-
-		switch (ch) {
 
+		switch ((ch = handle_keypad(select_left, select_right, &menukeymap.select_nav))) {
 			case iKeyAbort:		/* Abort */
 				break;
 
@@ -217,7 +214,7 @@ selection_page(
 				break;							/* Nothing more to do at the moment */
 
 			case iKeyRedrawScr:		/* redraw */
-				my_retouch();					/* TODO not done elsewhere, maybe should be in show_selection_page */
+				my_retouch();					/* TODO: not done elsewhere, maybe should be in show_selection_page */
 				set_xclick_off();
 				show_selection_page();
 				break;
@@ -289,7 +286,7 @@ selection_page(
 				break;
 
 			case iKeySelectMoveGrp:	/* reposition group within group list */
-				/* TODO move all this to reposition_group() */
+				/* TODO: move all this to reposition_group() */
 				if (!selmenu.max) {
 					info_message(_(txt_no_groups));
 					break;
@@ -322,9 +319,7 @@ selection_page(
 
 			case iKeyOptionMenu:	/* option menu */
 				(void) change_config_file(NULL);
-				free_attributes_array();
-				read_attributes_file(TRUE);
-				read_attributes_file(FALSE);
+				read_attributes_files();
 				show_selection_page();
 				break;
 
@@ -376,7 +371,7 @@ selection_page(
 					break;
 				}
 				if (!CURR_GROUP.subscribed && !CURR_GROUP.bogus) {
-					subscribe(&CURR_GROUP, SUBSCRIBED);
+					subscribe(&CURR_GROUP, SUBSCRIBED, TRUE);
 					show_selection_page();
 					info_message(_(txt_subscribed_to), CURR_GROUP.name);
 					move_down();
@@ -402,7 +397,7 @@ selection_page(
 				}
 				if (CURR_GROUP.subscribed) {
 					mark_screen(SELECT_LEVEL, selmenu.curr - selmenu.first, 2, CURR_GROUP.newgroup ? "N" : "u");
-					subscribe(&CURR_GROUP, UNSUBSCRIBED);
+					subscribe(&CURR_GROUP, UNSUBSCRIBED, TRUE);
 					info_message(_(txt_unsubscribed_to), CURR_GROUP.name);
 					move_down();
 				} else if (CURR_GROUP.bogus && tinrc.strip_bogus == BOGUS_SHOW) {
@@ -432,21 +427,36 @@ selection_page(
 				break;
 
 			case iKeyPost:	/* post a basenote */
-				if (!can_post) {
-					info_message(_(txt_cannot_post));
-					break;
-				}
 				if (!selmenu.max) {
 					sprintf(buf, _(txt_post_newsgroups), tinrc.default_post_newsgroups);
-					if (!(prompt_string_default(buf, tinrc.default_post_newsgroups, _(txt_no_newsgroups), HIST_POST_NEWSGROUPS)))
+					if (!prompt_string_default(buf, tinrc.default_post_newsgroups, _(txt_no_newsgroups), HIST_POST_NEWSGROUPS))
 						break;
-
-					if (group_find(buf) == NULL) {
-						error_message(_(txt_not_in_active_file), buf);
+					if (group_find(tinrc.default_post_newsgroups) == NULL) {
+						error_message(_(txt_not_in_active_file), tinrc.default_post_newsgroups);
 						break;
+					} else {
+						strcpy(buf, tinrc.default_post_newsgroups);
+#if 1 /* TODO: fix the rest of the code so we don't need this anymore */
+						/*
+						 * this is a gross hack to avoid a crash in the
+						 * CHARSET_CONVERSION conversion case in new_part()
+						 * which relies currently relies on CURR_GROUP
+						 */
+						selmenu.curr = my_group_add(buf);
+						/*
+						 * and the next hack to avoid a grabbled selection
+						 * screen after the posting
+						 */
+						toggle_my_groups(NULL);
+						toggle_my_groups(NULL);
+#endif /* 1 */
 					}
 				} else
 					strcpy(buf, CURR_GROUP.name);
+				if (!can_post && !CURR_GROUP.bogus && !CURR_GROUP.attribute->mailing_list) {
+					info_message(_(txt_cannot_post));
+					break;
+				}
 				if (post_article(buf))
 					show_selection_page();
 				break;
@@ -511,14 +521,13 @@ show_selection_page(
 	signal_context = cSelect;
 	currmenu = &selmenu;
 
+	if (read_news_via_nntp)
+		snprintf(buf, sizeof(buf), "%s (%s  %d%s)", _(txt_group_selection), nntp_server, selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
+	else
+		snprintf(buf, sizeof(buf), "%s (%d%s)", _(txt_group_selection), selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
+
 	MoveCursor(0, 0);		/* top left corner */
 	CleartoEOLN();
-
-	if (read_news_via_nntp)
-		sprintf(buf, "%s (%s  %d%s)", _(txt_group_selection), nntp_server, selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
-	else
-		sprintf(buf, "%s (%d%s)", _(txt_group_selection), selmenu.max, (tinrc.show_only_unread_groups ? _(" R") : ""));
-
 	show_title(buf);
 	MoveCursor(1, 0);
 	CleartoEOLN();
@@ -571,20 +580,20 @@ show_selection_page(
 			if (active[n].description) {
 				strncpy(group_descript, active[n].description, blank_len);
 				group_descript[blank_len] = '\0';
-				sprintf(sptr, "  %c %s %s  %-*.*s  %-*.*s" cCRLF,
+				sprintf(sptr, "  %c %s %s  %-*.*s  %-*.*s%s",
 				         subs, tin_ltoa(i + 1, 4), tmp,
 				         groupname_len, groupname_len, active_name,
-				         blank_len, blank_len, group_descript);
+				         blank_len, blank_len, group_descript, cCRLF);
 			} else
-				sprintf(sptr, "  %c %s %s  %-*.*s  " cCRLF,
+				sprintf(sptr, "  %c %s %s  %-*.*s  %s",
 				         subs, tin_ltoa(i + 1, 4), tmp,
 				         (groupname_len + blank_len),
-				         (groupname_len + blank_len), active[n].name);
+				         (groupname_len + blank_len), active[n].name, cCRLF);
 		} else {
 			if (tinrc.draw_arrow)
-				sprintf(sptr, "  %c %s %s  %-*.*s" cCRLF, subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name);
+				sprintf(sptr, "  %c %s %s  %-*.*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, cCRLF);
 			else
-				sprintf(sptr, "  %c %s %s  %-*.*s%*s" cCRLF, subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, blank_len, " ");
+				sprintf(sptr, "  %c %s %s  %-*.*s%*s%s", subs, tin_ltoa(i + 1, 4), tmp, groupname_len, groupname_len, active_name, blank_len, " ", cCRLF);
 		}
 		if (tinrc.strip_blanks)
 			strcat(strip_line(sptr), cCRLF);
@@ -636,7 +645,7 @@ yank_active_file(
 	void)
 {
 	if (yanked_out) {										/* Yank in */
-		if (selmenu.max == num_active)						/* All groups currently present ? */
+		if (selmenu.max == num_active)						/* All groups currently present? */
 			info_message(_(txt_yanked_none));
 		else {
 			int i;
@@ -977,7 +986,7 @@ read_groups(
 			case GRP_ENTER:		/* group_page() has already set selmenu.curr */
 				break;
 
-			case GRP_RETURN:
+			case GRP_RETSELECT:
 			case GRP_EXIT:
 			default:
 				done = TRUE;
@@ -1002,7 +1011,7 @@ set_groupname_len(
 	t_bool all_groups)
 {
 	int len;
-	register int i;
+	int i;
 
 	groupname_len = 0;
 
@@ -1107,7 +1116,7 @@ toggle_my_groups(
 		}
 	}
 	fclose(fp);
-#endif /* 1 */
+#endif /* 0 */
 	selmenu.curr = save_restore_curr_group(FALSE);			/* Restore saved group position */
 }
 
@@ -1124,7 +1133,7 @@ subscribe_pattern(
 	t_bool state)
 {
 	char buf[LEN];
-	int i, subscribe_num;
+	int i, subscribe_num = 0;
 
 	if (!num_active || no_write)
 		return;
@@ -1137,13 +1146,14 @@ subscribe_pattern(
 	wait_message(0, message);
 
 	/*
-	 * TODO why do we do a pass over my_group[] before another one over active[] ?
+	 * TODO: why do we do a pass over my_group[] before another one
+	 *       over active[]? AFAICS the first loop can go away.
 	 */
-	for (subscribe_num = 0, i = 0; i < selmenu.max; i++) {
+	for (i = 0; i < selmenu.max; i++) {
 		if (match_group_list(active[my_group[i]].name, buf)) {
 			if (active[my_group[i]].subscribed != (state != FALSE)) {
 				spin_cursor();
-				subscribe(&active[my_group[i]], SUB_CHAR(state));
+				subscribe(&active[my_group[i]], SUB_CHAR(state), TRUE);
 				subscribe_num++;
 			}
 		}
@@ -1155,10 +1165,13 @@ subscribe_pattern(
 				if (active[i].subscribed != (state != FALSE)) {
 					spin_cursor();
 					/* If found and group is not subscribed add it to end of my_group[]. */
-					subscribe(&active[i], SUB_CHAR(state));
+					subscribe(&active[i], SUB_CHAR(state), TRUE);
 					if (state) {
 						my_group_add(active[i].name);
-/* TODO grp_mark_unread() or something needs to do a GrpGetArtInfo to get initial count right */
+						/*
+						 * TODO: grp_mark_unread() or something needs to do a
+						 *       group_get_art_info() to get initial count right
+						 */
 						grp_mark_unread(&active[i]);
 					}
 					subscribe_num++;
@@ -1207,21 +1220,21 @@ static void
 select_read_group(
 	void)
 {
-	struct t_group currgrp;
+	struct t_group *currgrp;
 
 	if (!selmenu.max || selmenu.curr == -1) {
 		info_message(_(txt_no_groups));
 		return;
 	}
 
-	currgrp = CURR_GROUP;
+	currgrp = &CURR_GROUP;
 
-	if (currgrp.bogus) {
+	if (currgrp->bogus) {
 		info_message(_(txt_not_exist));
 		return;
 	}
 
-	if (currgrp.xmax > 0 && (currgrp.xmin <= currgrp.xmax))
+	if (currgrp->xmax > 0 && (currgrp->xmin <= currgrp->xmax))
 		read_groups();
 	else
 		info_message(_(txt_no_arts));
