@@ -1,12 +1,12 @@
 /*
  *  Project   : tin - a Usenet reader
  *  Module    : tags.c
- *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
+ *  Author    : Jason Faultless <jason@altarstone.com>
  *  Created   : 1999-12-06
- *  Updated   : 2002-11-11
+ *  Updated   : 2003-07-19
  *  Notes     : Split out from other modules
  *
- * Copyright (c) 1999-2003 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1999-2003 Jason Faultless <jason@altarstone.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Jason Faultless.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -216,11 +213,11 @@ tag_multipart(
 
 	/* check for failure... */
 	if (qty == 0) {
-		info_message(_("Not a multi-part message")); /* FIXME: -> lang.c */
+		info_message(_(txt_info_not_multipart_message));
 		return 0;
 	}
 	if (qty < 0) {
-		info_message(_("Missing part #%d"), -qty); /* FIXME: -> lang.c */
+		info_message(_(txt_info_missing_part), -qty);
 		return 0;
 	}
 
@@ -230,7 +227,7 @@ tag_multipart(
 	 */
 	for (i = 0; i < qty; ++i) {
 		if (arts[base[info[i].base_index]].tagged != 0)
-			remove_tag(base[info[i].base_index]);
+			untag_article(base[info[i].base_index]);
 	}
 
 	/*
@@ -242,7 +239,6 @@ tag_multipart(
 
 	free(info);
 
-	info_message(_("All parts tagged")); /* FIXME -> lang.c */
 	return qty;
 }
 
@@ -279,12 +275,12 @@ tag_article(
 	int art)
 {
 	if (arts[art].tagged != 0) {
-		remove_tag(art);
-		info_message(_(txt_untagged_art));
+		untag_article(art);
+		info_message(_(txt_prefix_untagged), txt_article_singular);
 		return FALSE;
 	} else {
 		arts[art].tagged = ++num_of_tagged_arts;
-		info_message(_(txt_tagged_art));
+		info_message(_(txt_prefix_tagged), txt_article_singular);
 		return TRUE;
 	}
 }
@@ -296,7 +292,7 @@ tag_article(
  * greater than 'tag', fixup counters
  */
 void
-remove_tag(
+untag_article(
 	long art)
 {
 	int i, j;
@@ -440,6 +436,7 @@ set_range(
 
 		default:
 			return FALSE;
+			/* NOTREACHED */
 			break;
 	}
 	return TRUE;
@@ -470,10 +467,10 @@ parse_range(
 	while (*ptr && state != DONE) {
 		if (isdigit(*ptr)) {
 			if (state == FINDMAX) {
-				*range_end = atoi (ptr);
+				*range_end = atoi(ptr);
 				state = DONE;
 			} else
-				*range_start = atoi (ptr);
+				*range_start = atoi(ptr);
 			while (isdigit(*ptr))
 				ptr++;
 		} else {
@@ -522,7 +519,7 @@ do_auto_select_arts(
 #	ifdef DEBUG_NEWSRC
 			debug_print_comment("group.c: X command");
 #	endif /* DEBUG_NEWSRC */
-			art_mark_read(&CURR_GROUP, &arts[i]);
+			art_mark(&CURR_GROUP, &arts[i], ART_READ);
 			arts[i].zombie = TRUE;
 		}
 	}
@@ -546,7 +543,7 @@ undo_auto_select_arts(
 #	ifdef DEBUG_NEWSRC
 			debug_print_comment("group.c: + command");
 #	endif /* DEBUG_NEWSRC */
-			art_mark_unread(&CURR_GROUP, &arts[i]);
+			art_mark(&CURR_GROUP, &arts[i], ART_UNREAD);
 			arts[i].zombie = FALSE;
 		}
 	}
@@ -569,3 +566,63 @@ undo_selections(
 		arts[i].zombie = FALSE;
 	}
 }
+
+
+/*
+ * Return TRUE if there are any selected arts
+ */
+t_bool
+arts_selected(
+	void)
+{
+	int i;
+
+	for_each_art(i) {
+		 if (arts[i].selected)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
+
+/*
+ * Mark all tagged articles as read.
+ * Return number of articles that have been marked.
+ */
+int
+mark_tagged_read(
+	struct t_group *group)
+{
+	int i, cnt = 0;
+
+	for_each_art(i) {
+		if (arts[i].tagged && (arts[i].status == ART_UNREAD || arts[i].status == ART_WILL_RETURN)) {
+			art_mark(group, &arts[i], ART_READ);
+			cnt++;
+		}
+	}
+	untag_all_articles();
+
+	return cnt;
+}
+
+
+/*
+ * Check if there is at least one tagged _and_ unread article.
+ */
+t_bool
+got_tagged_unread_arts(
+	void)
+{
+	int i;
+
+	if (num_of_tagged_arts > 0) {
+		for_each_art(i) {
+			if (arts[i].tagged && (arts[i].status == ART_UNREAD || arts[i].status == ART_WILL_RETURN))
+				return TRUE;
+		}
+	}
+
+	return FALSE;
+}
