@@ -3,7 +3,7 @@
  *  Module    : tcurses.c
  *  Author    : Thomas Dickey <dickey@herndon4.his.com>
  *  Created   : 1997-03-02
- *  Updated   : 2003-01-18
+ *  Updated   : 2003-08-03
  *  Notes     : This is a set of wrapper functions adapting the termcap
  *	             interface of tin to use SVr4 curses (e.g., ncurses).
  *
@@ -18,10 +18,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Thomas Dickey.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -57,6 +54,8 @@
 int cLINES;
 int cCOLS;
 
+static int my_innstr(char *str, int n);
+
 
 #	ifdef HAVE_XCURSES
 static int
@@ -69,7 +68,7 @@ vwprintw(
 	char *string = buffer;
 	int y, x, code;
 
-	vsprintf(buffer, fmt, ap);
+	vsnprintf(buffer, sizeof(buffer), fmt, ap);
 	getyx(w, y, x);
 	TRACE(("vwprintw[%d/%d,%d/%d]:%s", y, cLINES, x, cCOLS, buffer));
 	while (*string == '\b')
@@ -225,23 +224,26 @@ StartInverse(
 }
 
 
+#	if 0 /* not used */
 static int
 isInverse(
 	void)
 {
-#	ifdef HAVE_COLOR
+#		ifdef HAVE_COLOR
 	if (use_color) {
 		short pair = PAIR_NUMBER(getattrs(stdscr));
 		short fg, bg;
 		pair_content(pair, &fg, &bg);
 		return (fg == tinrc.col_invers_fg) && (bg == tinrc.col_invers_bg);
 	}
-#	endif /* HAVE_COLOR */
+#		endif /* HAVE_COLOR */
 
 	return (getattrs(stdscr) & A_REVERSE);
 }
+#	endif /* 0 */
 
 
+#	if 0 /* doesn't work correct with ncurses4.x */
 /*
  */
 void
@@ -253,6 +255,7 @@ ToggleInverse(
 	else
 		StartInverse();
 }
+#	endif /* 0 */
 
 
 /*
@@ -377,8 +380,10 @@ highlight_string(
 		MoveCursor(row, 0);
 		my_innstr(tmp, cCOLS);
 		tmp[col] = '\0';
-		if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1)
+		if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+			wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
 			col = wcswidth(wtmp, ARRAY_SIZE(wtmp));
+		}
 	}
 #	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
@@ -393,7 +398,6 @@ highlight_string(
 }
 
 
-#	ifdef HAVE_COLOR
 /*
  * Color 'size' chars at (row,col) with 'color' and handle marks
  */
@@ -404,6 +408,18 @@ word_highlight_string(
 	int size,
 	int color)
 {
+	/*
+	 * Mapping of the tinrc.mono_mark* values to the ncurses attributes
+	 */
+	int attributes[] = {
+		A_NORMAL,
+		A_STANDOUT,
+		A_UNDERLINE,
+		A_REVERSE,
+		A_BLINK,
+		A_DIM,
+		A_BOLD
+	};
 	char tmp[LEN];
 	int wsize = size;
 #		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
@@ -417,8 +433,10 @@ word_highlight_string(
 		MoveCursor(row, 0);
 		my_innstr(tmp, cCOLS);
 		tmp[col] = '\0';
-		if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1)
+		if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+			wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
 			col = wcswidth(wtmp, ARRAY_SIZE(wtmp));
+		}
 	}
 #		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
@@ -427,8 +445,10 @@ word_highlight_string(
 
 #		if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 	tmp[size] = '\0';
-	if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1)
+	if (mbstowcs(wtmp, tmp, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+		wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
 		wsize = wcswidth(wtmp, ARRAY_SIZE(wtmp));
+	}
 #		endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 	/* safegurad against bogus regexps */
@@ -447,20 +467,36 @@ word_highlight_string(
 				break;
 
 			case 2: /* print space */
+				MoveCursor(row, col + wsize - 1);
+				my_fputs(" ", stdout);
+				MoveCursor(row, col);
+				my_fputs(" ", stdout);
 				tmp[0] = tmp[size - 1] = ' ';
+				str_trim(tmp);
 				break;
 
 			default: /* print mark (case 1) */
 				break;
 		}
 	}
-	fcol(color);
+#	ifdef HAVE_COLOR
+	if (use_color)
+		fcol(color);
+	else
+#	endif /* HAVE_COLOR */
+		if (color > 0 && color <= MAX_ATTR)
+			attron(attributes[color]);
 	my_fputs(tmp, stdout);
 	my_flush();
-	fcol(tinrc.col_text);
+#	ifdef HAVE_COLOR
+	if (use_color)
+		fcol(tinrc.col_text);
+	else
+#	endif /* HAVE_COLOR */
+		if (color > 0 && color <= MAX_ATTR)
+			attroff(attributes[color]);
 	stow_cursor();
 }
-#	endif /* HAVE_COLOR */
 
 
 int
@@ -520,12 +556,14 @@ ReadWch(
 		if (wch < KEY_MIN) {
 			/* read in the multibyte sequence */
 			char *mbs = my_malloc(MB_CUR_MAX + 1);
-			int i;
+			int i, ch;
 
 			mbs[0] = (char) wch;
 			nodelay(stdscr, TRUE);
 			for (i = 1; i < (int) MB_CUR_MAX; i++) {
-				if ((mbs[i] = getch()) == ERR)
+				if ((ch = getch()) != ERR)
+					mbs[i] = (char) ch;
+				else
 					break;
 			}
 			nodelay(stdscr, FALSE);
@@ -689,10 +727,11 @@ my_erase(
 	if (!cmd_line) {
 		erase();
 
-		/* Curses doesn't actually do an erase() until refresh() is
-		 * called.  Ncurses 4.0 (and lower) reset the background color
-		 * when doing an erase().  So the only way to ensure we'll get
-		 * the same background colors is to reset them here.
+		/*
+		 * Curses doesn't actually do an erase() until refresh() is called.
+		 * Ncurses 4.0 (and lower) reset the background color when doing an
+		 * erase(). So the only way to ensure we'll get the same background
+		 * colors is to reset them here.
 		 */
 		refresh();
 #	ifdef HAVE_COLOR
@@ -733,7 +772,7 @@ my_retouch(
  * innstr can't read multibyte chars
  * we use innwstr (if avaible) and convert to multibyte chars
  */
-int
+static int
 my_innstr(
 	char *str,
 	int n)
