@@ -3,7 +3,7 @@
  *  Module    : thread.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2003-01-21
+ *  Updated   : 2003-08-10
  *  Notes     :
  *
  * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
@@ -54,14 +54,13 @@
 #define MAGIC		3
 
 int thread_basenote = 0;				/* Index in base[] of basenote */
-/* char i_key_search_last; */					/* for repeated search */
 static int thread_respnum = 0;			/* Index in arts[] of basenote ie base[thread_basenote] */
 t_bool show_subject;
 
 /*
  * Local prototypes
  */
-static int enter_pager(int art, t_bool ignore_unavail);
+static int enter_pager(int art, t_bool ignore_unavail, int level);
 static int thread_catchup(int ch);
 static int thread_left(void);
 static int thread_right(void);
@@ -73,6 +72,7 @@ static void draw_thread_arrow(void);
 static void make_prefix(struct t_msgid *art, char *prefix, int maxlen);
 static void show_thread_page(void);
 static void update_thread_page(void);
+static int mark_art_read(struct t_group *group);
 
 
 /*
@@ -131,8 +131,8 @@ build_tline(
 		} else if (art->killed && tinrc.kill_level != KILL_NOTHREAD) {
 			mark = tinrc.art_marked_killed;
 		} else {
-			if (/* tinrc.kill_level != KILL_READ && */ art->score >= tinrc.score_select)
-				mark = tinrc.art_marked_read_selected ; /* read hot chil^H^H^H^H article */
+			if (/* tinrc.kill_level != KILL_UNREAD && */ art->score >= tinrc.score_select)
+				mark = tinrc.art_marked_read_selected; /* read hot chil^H^H^H^H article */
 			else
 				mark = tinrc.art_marked_read;
 		}
@@ -146,18 +146,18 @@ build_tline(
 	 * Add the number of lines and/or the score if enabled
 	 * (inside "[,]", 1+4[+1+6]+1+2 chars total)
 	 */
-	if (tinrc.show_lines || tinrc.show_score) { /* add [ */
+	if (tinrc.show_info != SHOW_INFO_NOTHING) { /* add [ */
 		strcat(buff, "[");
 		rest_of_line--;
 	}
 
-	if (tinrc.show_lines) { /* add lines */
+	if (tinrc.show_info == SHOW_INFO_LINES || tinrc.show_info == SHOW_INFO_BOTH) { /* add lines */
 		strcat(buff, ((art->line_count != -1) ? tin_ltoa(art->line_count, 4): "   ?"));
 		rest_of_line -= 4;
 	}
 
-	if (tinrc.show_score) {
-		if (tinrc.show_lines) { /* insert a separator if show lines and score */
+	if (tinrc.show_info == SHOW_INFO_SCORE || tinrc.show_info == SHOW_INFO_BOTH) {
+		if (tinrc.show_info == SHOW_INFO_BOTH) { /* insert a separator if show lines and score */
 			strcat(buff, ",");
 			rest_of_line--;
 		}
@@ -165,7 +165,7 @@ build_tline(
 		rest_of_line -= 6;
 	}
 
-	if (tinrc.show_lines || tinrc.show_score) { /* add closing ] and two spaces */
+	if (tinrc.show_info != SHOW_INFO_NOTHING) { /* add closing ] and two spaces */
 		strcat(buff, "]  ");
 		rest_of_line -= 3;
 	}
@@ -217,10 +217,11 @@ build_tline(
 			size_t len = strlen(buff);
 			for (ptr = art->refptr->parent; ptr && EXPIRED(ptr); ptr = ptr->parent)
 				;
-			if (!(ptr && arts[ptr->article].subject == art->subject))
+			if (!(ptr && arts[ptr->article].subject == art->subject)) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 				if (mbstowcs(wtmp2, art->subject, ARRAY_SIZE(wtmp2) - 1) != (size_t) -1) {
-					wcspart(wtmp, wtmp2, gap, ARRAY_SIZE(wtmp));
+					wtmp2[ARRAY_SIZE(wtmp2) - 1] = (wchar_t) '\0';
+					wcspart(wtmp, wtmp2, gap, ARRAY_SIZE(wtmp), TRUE);
 					if (wcstombs(tmp, wtmp, sizeof(tmp) - 1) != (size_t) -1)
 #	ifdef USE_CURSES
 						strncat(buff, tmp, sizeof(buff) - len - 1);
@@ -228,9 +229,10 @@ build_tline(
 						strncat(buff, tmp, cCOLS * MB_CUR_MAX - len - 1);
 #	endif /* USE_CURSES */
 				}
+			}
 #else
 				strncat(buff, art->subject, gap);
-
+			}
 			buff[len + gap] = '\0';	/* Just in case */
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 		}
@@ -240,9 +242,10 @@ build_tline(
 		 */
 		if (len_from) {
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-			if (mbstowcs(wtmp, buff, ARRAY_SIZE(wtmp) - 1) != (size_t) -1)
+			if (mbstowcs(wtmp, buff, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+				wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
 				fill = cCOLS - len_from - wcswidth(wtmp, ARRAY_SIZE(wtmp) - 1);
-			else
+			} else
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 				fill = cCOLS - len_from - strlen(buff);
 
@@ -257,8 +260,9 @@ build_tline(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 			get_author(TRUE, art, tmp, sizeof(tmp) - 1);
 
-			if (mbstowcs(wtmp2, tmp, ARRAY_SIZE(wtmp2) -1) != (size_t) -1) {
-				wcspart(wtmp, wtmp2, len_from, ARRAY_SIZE(wtmp));
+			if (mbstowcs(wtmp2, tmp, ARRAY_SIZE(wtmp2) - 1) != (size_t) -1) {
+				wtmp2[ARRAY_SIZE(wtmp2) - 1] = (wchar_t) '\0';
+				wcspart(wtmp, wtmp2, len_from, ARRAY_SIZE(wtmp), TRUE);
 				if (wcstombs(tmp, wtmp, sizeof(tmp) - 1) != (size_t) -1)
 #	ifdef USE_CURSES
 					strncat(buff, tmp, sizeof(buff) - strlen(buff) - 1);
@@ -275,8 +279,9 @@ build_tline(
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
 		get_author(TRUE, art, tmp, sizeof(tmp) - 1);
 
-		if (mbstowcs(wtmp2, tmp, ARRAY_SIZE(wtmp2) -1) != (size_t) -1) {
-			wcspart(wtmp, wtmp2, cCOLS - strlen(buff), ARRAY_SIZE(wtmp));
+		if (mbstowcs(wtmp2, tmp, ARRAY_SIZE(wtmp2) - 1) != (size_t) -1) {
+			wtmp2[ARRAY_SIZE(wtmp2) - 1] = (wchar_t) '\0';
+			wcspart(wtmp, wtmp2, cCOLS - strlen(buff), ARRAY_SIZE(wtmp), TRUE);
 			if (wcstombs(tmp, wtmp, sizeof(tmp) - 1) != (size_t) -1)
 #	ifdef USE_CURSES
 				strncat(buff, tmp, sizeof(buff) - strlen(buff) - 1);
@@ -297,9 +302,10 @@ build_tline(
 		 * Pad to end of line so that inverse bar looks 'good'
 		 */
 #if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
-		if (mbstowcs(wtmp, buff, ARRAY_SIZE(wtmp) - 1) != (size_t) -1)
+		if (mbstowcs(wtmp, buff, ARRAY_SIZE(wtmp) - 1) != (size_t) -1) {
+			wtmp[ARRAY_SIZE(wtmp) - 1] = (wchar_t) '\0';
 			fill = cCOLS - wcswidth(wtmp, ARRAY_SIZE(wtmp) - 1);
-		else
+		} else
 #endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			fill = cCOLS - strlen(buff);
 
@@ -336,12 +342,11 @@ draw_line(
 	char *s = &(screen[INDEX2TNUM(i)].col[startpos]);
 #endif /* USE_CURSES */
 
+	if (tinrc.strip_blanks)
+		 strip_line(s);
 	if (!magic) {
-		if (tinrc.strip_blanks) {
-			strip_line(s);
-			CleartoEOLN();
-		}
 		tlen = strlen(s);	/* note new line length */
+		CleartoEOLN();
 	} else
 		tlen = magic;
 
@@ -393,11 +398,11 @@ thread_right(
  * If threaded on References: or Archive-name: show
  *   <respnum> <subject> <name>
  * Return values:
- *		GRP_RETURN		Return to selection screen
+ *		GRP_RETSELECT	Return to selection screen
  *		GRP_QUIT		'Q'uit all the way out
  *		GRP_NEXT		Catchup goto next group
  *		GRP_NEXTUNREAD	Catchup enter next unread thread
- *		GRP_KILLED		Thread was killed at art level ?????
+ *		GRP_KILLED		Thread was killed at art level?
  *		GRP_EXIT		Return to group menu
  */
 int
@@ -422,11 +427,6 @@ thread_page(
 		return GRP_EXIT;
 	}
 
-	/*
-	 * If threading by Refs, it helps to see the subject line
-	 */
-	show_subject = ((arts[thread_respnum].archive != NULL) || (group->attribute->thread_arts == THREAD_REFS) || (group->attribute->thread_arts == THREAD_BOTH));
-
 	/*
 	 * Set the cursor to the last response unless pos_first_unread is on
 	 * or an explicit thread_depth has been specified
@@ -439,9 +439,9 @@ thread_page(
 		if (tinrc.pos_first_unread) {
 			if ((i = new_responses(thread_basenote))) {
 				for (n = 0, i = (int) base[thread_basenote]; i >= 0; i = arts[i].thread, n++) {
-					if (arts[i].status == ART_UNREAD) {
+					if (arts[i].status == ART_UNREAD || arts[i].status == ART_WILL_RETURN) {
 						if (arts[i].thread == ART_EXPIRED)
-							art_mark_read(group, &arts[i]);
+							art_mark(group, &arts[i], ART_READ);
 						else
 							thdmenu.curr = n;
 						break;
@@ -458,7 +458,7 @@ thread_page(
 	 * See if we're on a direct call from the group menu to the pager
 	 */
 	if (page) {
-		if ((ret_code = enter_pager(page->art, page->ignore_unavail)) != 0)
+		if ((ret_code = enter_pager(page->art, page->ignore_unavail, GROUP_LEVEL)) != 0)
 			return ret_code;
 		/* else fall through to stay in thread level */
 	}
@@ -506,7 +506,7 @@ thread_page(
 					info_message(_(txt_no_last_message));
 					break;
 				}
-				ret_code = enter_pager(this_resp, FALSE);
+				ret_code = enter_pager(this_resp, FALSE, THREAD_LEVEL);
 				break;
 
 			case iKeySetRange:	/* set range */
@@ -516,31 +516,44 @@ thread_page(
 
 			case iKeyPipe:			/* pipe article to command */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_PIPE, THREAD_LEVEL, &CURR_GROUP, find_response(thread_basenote, thdmenu.curr));
+					feed_articles(FEED_PIPE, THREAD_LEVEL, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 
 			case iKeyThreadMail:	/* mail article to somebody */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_MAIL, THREAD_LEVEL, &CURR_GROUP, find_response(thread_basenote, thdmenu.curr));
+					feed_articles(FEED_MAIL, THREAD_LEVEL, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 
 			case iKeyThreadSave:	/* save articles with prompting */
 				if (thread_basenote >= 0)
-					feed_articles(FEED_SAVE, THREAD_LEVEL, &CURR_GROUP, find_response(thread_basenote, thdmenu.curr));
+					feed_articles(FEED_SAVE, THREAD_LEVEL, group, find_response(thread_basenote, thdmenu.curr));
 				break;
 
-			case iKeyThreadAutoSaveTagged:	/* Auto-save tagged articles without prompting */
-				if (thread_basenote >= 0) {
-					if (num_of_tagged_arts)
-						feed_articles(FEED_AUTOSAVE_TAGGED, THREAD_LEVEL, &CURR_GROUP, (int) base[grpmenu.curr]);
-					else
-						info_message(_(txt_no_tagged_arts_to_save));
-				}
+			case iKeyThreadAutoSave:	/* Auto-save articles without prompting */
+				if (thread_basenote >= 0)
+					feed_articles(FEED_AUTOSAVE, THREAD_LEVEL, group, (int) base[grpmenu.curr]);
 				break;
 
+#if 0 /* FIXME: crsr-position after kill */
+			case iKeyThreadAutoSel:
+			case iKeyThreadAutoKill:
+				n = find_response(thread_basenote, thdmenu.curr);
+				filter_menu((ch == iKeyThreadAutoKill) ? FILTER_KILL : FILTER_SELECT, group, &arts[n]);
+				if (filter_articles(group)) {
+					make_threads(group, FALSE);
+					if ((n = next_unread(n)) == -1) {
+						ret_code = GRP_EXIT;
+						break;
+					}
+					fixup_thread(n, TRUE);
+				}
+				show_thread_page();
+				break;
+#endif /* 0 */
+
 			case iKeyThreadReadArt:
 			case iKeyThreadReadArt2:	/* read current article within thread */
-				ret_code = enter_pager(find_response(thread_basenote, thdmenu.curr), FALSE);
+				ret_code = enter_pager(find_response(thread_basenote, thdmenu.curr), FALSE, THREAD_LEVEL);
 				break;
 
 			case iKeyThreadReadNextArtOrThread:
@@ -587,18 +600,7 @@ thread_page(
 				break;
 
 			case iKeyThreadMarkArtRead: /* mark article as read */
-				n = find_response(thread_basenote, thdmenu.curr);
-				if ((arts[n].status == ART_UNREAD) || (arts[n].status == ART_WILL_RETURN)) {
-					art_mark_read(group, &arts[n]);
-					build_tline(thdmenu.curr, &arts[n]);
-					draw_line(thdmenu.curr, MAGIC);
-				}
-				if ((n = next_unread(n)) == -1) {	/* no more unread articles */
-					ret_code = GRP_EXIT;
-					break;
-				}
-				fixup_thread(n, TRUE);			/* We may be in the next thread now */
-				move_to_item(which_response(n));
+				ret_code = mark_art_read(group);
 				break;
 
 			case iKeyThreadToggleSubjDisplay:	/* toggle display of subject & subj/author */
@@ -609,8 +611,15 @@ thread_page(
 				break;
 
 			case iKeyOptionMenu:
-				(void) change_config_file(group);
-				show_thread_page();
+				change_config_file(group);
+				n = find_response(thread_basenote, thdmenu.curr);
+				if (which_thread(n) == -1) /* We have lost the thread */
+					ret_code = GRP_EXIT;
+				else {
+					fixup_thread(n, FALSE);
+					thdmenu.curr = which_response(n);
+					show_thread_page();
+				}
 				break;
 
 			case iKeyHelp:					/* help */
@@ -618,15 +627,22 @@ thread_page(
 				show_thread_page();
 				break;
 
+			/* TODO: assign a new key here */
+			case iKeyGroupKill:
+				n = find_response(thread_basenote, thdmenu.curr);
+				filter_menu(FILTER_KILL, group, &arts[n]);
+				show_thread_page();
+				break;
+
 			case iKeyLookupMessage:
 				if ((n = prompt_msgid()) != ART_UNAVAILABLE)
-					ret_code = enter_pager(n, FALSE);
+					ret_code = enter_pager(n, FALSE, THREAD_LEVEL);
 				break;
 
 			case iKeySearchBody:			/* search article body */
-				if ((n = search_body(find_response(thread_basenote, thdmenu.curr), repeat_search)) != -1) {
-					fixup_thread(n, TRUE);
-					ret_code = enter_pager(n, FALSE);
+				if ((n = search_body(group, find_response(thread_basenote, thdmenu.curr), repeat_search)) != -1) {
+					fixup_thread(n, FALSE);
+					ret_code = enter_pager(n, FALSE, THREAD_LEVEL);
 				}
 				break;
 
@@ -674,19 +690,23 @@ thread_page(
 				/* Find index of current article */
 				if ((n = find_response(thread_basenote, thdmenu.curr)) < 0)
 					break;
+				else {
+					t_bool tagged;
 
-				if (tag_article(n)) {
-					build_tline(thdmenu.curr, &arts[n]);	/* Update just this line */
-					draw_line(thdmenu.curr, MAGIC);
-				} else
-					update_thread_page();						/* Must update whole page */
+					if ((tagged = tag_article(n))) {
+						build_tline(thdmenu.curr, &arts[n]);	/* Update just this line */
+						draw_line(thdmenu.curr, MAGIC);
+					} else
+						update_thread_page();						/* Must update whole page */
 
-				/* Automatically advance to next art if not at end of thread */
-				if (thdmenu.curr + 1 < thdmenu.max) {
-					move_down();
-					break;
+					/* Automatically advance to next art if not at end of thread */
+					if (thdmenu.curr + 1 < thdmenu.max)
+						move_down();
+					else
+						draw_thread_arrow();
+
+					info_message(tagged ? _(txt_prefix_tagged) : _(txt_prefix_untagged), txt_article_singular);
 				}
-				draw_thread_arrow();
 				break;
 
 			case iKeyThreadBugReport:
@@ -704,32 +724,31 @@ thread_page(
 
 			case iKeyThreadMarkArtUnread:		/* mark article as unread */
 				n = find_response(thread_basenote, thdmenu.curr);
-				art_mark_will_return(group, &arts[n]); /* art_mark_unread(group, &arts[n]); */
+				art_mark(group, &arts[n], ART_WILL_RETURN);
 				build_tline(thdmenu.curr, &arts[n]);
 				draw_line(thdmenu.curr, MAGIC);
-				info_message(_(txt_marked_as_unread), _("Article"));
+				info_message(_(txt_marked_as_unread), _(txt_article_upper));
 				draw_thread_arrow();
 				break;
 
 			case iKeyThreadMarkThdUnread:		/* mark thread as unread */
 				thd_mark_unread(group, base[thread_basenote]);
 				update_thread_page();
-				info_message(_(txt_marked_as_unread), _("Thread"));
+				info_message(_(txt_marked_as_unread), _(txt_thread_upper));
 				break;
 
 			case iKeyThreadSelArt:		/* mark article as selected */
 			case iKeyThreadToggleArtSel:		/* toggle article as selected */
 				if ((n = find_response(thread_basenote, thdmenu.curr)) < 0)
 					break;
-				arts[n].selected = (!(ch == iKeyThreadToggleArtSel && arts[n].selected));	/* TODO optimise? */
+				arts[n].selected = (!(ch == iKeyThreadToggleArtSel && arts[n].selected));	/* TODO: optimise? */
 /*				update_thread_page(); */
 				build_tline(thdmenu.curr, &arts[n]);
 				draw_line(thdmenu.curr, MAGIC);
-				if (thdmenu.curr + 1 < thdmenu.max) {
+				if (thdmenu.curr + 1 < thdmenu.max)
 					move_down();
-					break;
-				}
-				draw_thread_arrow();
+				else
+					draw_thread_arrow();
 				break;
 
 			case iKeyThreadReverseSel:		/* reverse selections */
@@ -779,8 +798,7 @@ static void
 show_thread_page(
 	void)
 {
-	int i;
-	static int the_index = 0;
+	int i, art;
 
 	signal_context = cThread;
 	currmenu = &thdmenu;
@@ -789,14 +807,17 @@ show_thread_page(
 
 	set_first_screen_item();
 
-	the_index = find_response(thread_basenote, thdmenu.first);
+	art = find_response(thread_basenote, thdmenu.first);
 
-	assert(thdmenu.first != 0 || the_index == thread_respnum);
+	/*
+	 * If threading by Refs, it helps to see the subject line
+	 */
+	show_subject = ((arts[thread_respnum].archive != NULL) || (CURR_GROUP.attribute->thread_arts == THREAD_REFS) || (CURR_GROUP.attribute->thread_arts == THREAD_BOTH));
 
-	if (show_subject) /* TODO: -> lang.c */
-		snprintf(mesg, sizeof(mesg) - 1, _("List Thread (%d of %d)"), grpmenu.curr + 1, grpmenu.max);
+	if (show_subject)
+		snprintf(mesg, sizeof(mesg), _(txt_stp_list_thread), grpmenu.curr + 1, grpmenu.max);
 	else
-		snprintf(mesg, sizeof(mesg) - 1, _("Thread (%.*s)"), cCOLS - 23, arts[thread_respnum].subject);
+		snprintf(mesg, sizeof(mesg), _(txt_stp_thread), cCOLS - 23, arts[thread_respnum].subject);
 
 	/*
 	 * Slight misuse of the 'mesg' buffer here. We need to clear it so that progress messages
@@ -808,11 +829,9 @@ show_thread_page(
 	MoveCursor(INDEX_TOP, 0);
 
 	for (i = thdmenu.first; i < thdmenu.last; ++i) {
-		if (the_index < 0 || the_index >= max_art)
-			break;
-		build_tline(i, &arts[the_index]);
+		build_tline(i, &arts[art]);
 		draw_line(i, 0);
-		the_index = next_response(the_index);
+		art = next_response(art);
 	}
 
 	CleartoEOS();
@@ -829,7 +848,7 @@ static void
 update_thread_page(
 	void)
 {
-	register int i, j, the_index;
+	int i, j, the_index;
 
 	the_index = find_response(thread_basenote, thdmenu.first);
 	assert(thdmenu.first != 0 || the_index == thread_respnum);
@@ -866,13 +885,14 @@ fixup_thread(
 	t_bool redraw)
 {
 	int basenote = which_thread(respnum);
+	int old_thread_basenote = thread_basenote;
 
-	if (basenote != thread_basenote && basenote >= 0) {
+	if (basenote >= 0) {
 		thread_basenote = basenote;
 		thdmenu.max = num_of_responses(thread_basenote) + 1;
 		thread_respnum = base[thread_basenote];
 		grpmenu.curr = basenote;
-		if (redraw)
+		if (redraw && basenote != old_thread_basenote)
 			show_thread_page();
 	}
 
@@ -918,35 +938,15 @@ which_thread(
 {
 	int i, j;
 
-	for (i = 0; i < grpmenu.max; i++) {
-		for_each_art_in_thread(j, i) {
-			if (j == n)
-#ifdef JUST_TESTING
-			{
-				/*
-				 * We can much more rapidly locate the topmost available parent
-				 * than the brute force method above. This is definitely the
-				 * Right Way to go in 1.5
-				 */
-				int val = n;
-				struct t_msgid *ptr;
-
-				for (ptr = arts[n].refptr; ptr->parent; ptr = ptr->parent) {
-					if (ptr->parent->article != ART_UNAVAILABLE)
-						val = ptr->parent->article;
-				}
-				fprintf(stderr, "TEST: which_thread found %d, actual is %d\n", val, base[i]);
-
-				return i;
-			}
-#else
-				return i;
-#endif /* JUST_TESTING */
-		}
-	}
+	/* Move to top of thread */
+	for (i = n; arts[i].prev >= 0; i = arts[i].prev)
+		;
+	/* Find in base[] */
+	for (j = 0; j < grpmenu.max; j++)
+		if (base[j] == i)
+			return j;
 
 	error_message(_(txt_cannot_find_base_art), n);
-/*	assert(0 != 0); */
 	return -1;
 }
 
@@ -961,8 +961,8 @@ which_response(
 	int i, j;
 	int num = 0;
 
-	i = which_thread(n);
-	assert(i >= 0);
+	if ((i = which_thread(n)) == -1)
+		return 0;
 
 	for_each_art_in_thread(j, i) {
 		if (j == n)
@@ -1017,7 +1017,7 @@ get_score_of_thread(
 	int score = 0;
 
 	for (i = n; i >= 0; i = arts[i].thread) {
-		if (arts[i].status == ART_UNREAD) {
+		if (arts[i].status != ART_READ || arts[i].killed == ART_KILLED_UNREAD) {
 			if (tinrc.thread_score == THREAD_SCORE_MAX) {
 				/* we use the maximum article score for the complete thread */
 				if ((arts[i].score > score) && (arts[i].score > 0))
@@ -1057,8 +1057,9 @@ stat_thread(
 	sbuf->deleted = 0;
 	sbuf->inrange = 0;
 	sbuf->selected_total = 0;
-	sbuf->selected_unread= 0;
+	sbuf->selected_unread = 0;
 	sbuf->selected_seen = 0;
+	sbuf->killed = 0;
 	sbuf->art_mark = tinrc.art_marked_read;
 	sbuf->score = 0 /* -(SCORE_MAX) */;
 	sbuf->time = 0;
@@ -1090,12 +1091,10 @@ stat_thread(
 				++sbuf->selected_seen;
 		}
 
-#if 0
 		if (arts[i].killed)
 			++sbuf->killed;
-#endif /* 0 */
 
-		if ((CURR_GROUP.attribute && CURR_GROUP.attribute->thread_arts == THREAD_MULTI) && global_get_multipart_info(i, &minfo) && (minfo.total >= 1)) {
+		if ((CURR_GROUP.attribute->thread_arts == THREAD_MULTI) && global_get_multipart_info(i, &minfo) && (minfo.total >= 1)) {
 			sbuf->multipart_compare_len = minfo.subject_compare_len;
 			sbuf->multipart_total = minfo.total;
 			sbuf->multipart_have++;
@@ -1103,7 +1102,27 @@ stat_thread(
 	}
 
 	sbuf->score = get_score_of_thread((int) base[n]);
-	sbuf->art_mark = (sbuf->inrange ? tinrc.art_marked_inrange : (sbuf->deleted ? tinrc.art_marked_deleted : (sbuf->selected_unread ? tinrc.art_marked_selected : (sbuf->unread ? (tinrc.recent_time && (time((time_t) 0) - sbuf->time) < (tinrc.recent_time * DAY)) ? tinrc.art_marked_recent : tinrc.art_marked_unread : (sbuf->seen ? tinrc.art_marked_return : tinrc.art_marked_read)))));
+
+	if (sbuf->inrange)
+		sbuf->art_mark = tinrc.art_marked_inrange;
+	else if (sbuf->deleted)
+		sbuf->art_mark = tinrc.art_marked_deleted;
+	else if (sbuf->selected_unread)
+		sbuf->art_mark = tinrc.art_marked_selected;
+	else if (sbuf->unread) {
+		if (tinrc.recent_time && (time((time_t) 0) - sbuf->time) < (tinrc.recent_time * DAY))
+			sbuf->art_mark = tinrc.art_marked_recent;
+		else
+			sbuf->art_mark = tinrc.art_marked_unread;
+	}
+	else if (sbuf->seen)
+		sbuf->art_mark = tinrc.art_marked_return;
+	else if (sbuf->selected_total)
+		sbuf->art_mark = tinrc.art_marked_read_selected;
+	else if (sbuf->killed == sbuf->total)
+		sbuf->art_mark = tinrc.art_marked_killed;
+	else
+		sbuf->art_mark = tinrc.art_marked_read;
 	return sbuf->total;
 }
 
@@ -1157,13 +1176,10 @@ int
 prev_response(
 	int n)
 {
-	int resp;
 	int i;
 
-	resp = which_response(n);
-
-	if (resp > 0)
-		return find_response(which_thread(n), resp - 1);
+	if (arts[n].prev >= 0)
+		return arts[n].prev;
 
 	i = which_thread(n) - 1;
 
@@ -1211,12 +1227,14 @@ next_unread(
 		n = next_response(n);
 	}
 
-	n = base[0];
-	while (n != cur_base_art) {
-		if (((arts[n].status == ART_UNREAD) || (arts[n].status == ART_WILL_RETURN)) && arts[n].thread != ART_EXPIRED)
-			return n;
+	if (tinrc.wrap_on_next_unread) {
+		n = base[0];
+		while (n != cur_base_art) {
+			if (((arts[n].status == ART_UNREAD) || (arts[n].status == ART_WILL_RETURN)) && arts[n].thread != ART_EXPIRED)
+				return n;
 
-		n = next_response(n);
+			n = next_response(n);
+		}
 	}
 
 	return -1;
@@ -1225,14 +1243,13 @@ next_unread(
 
 /*
  * Find the previous unread response in this thread
- * TODO: why doesn't this handle ART_WILL_RETURN like next_unread() does?
  */
 int
 prev_unread(
 	int n)
 {
 	while (n >= 0) {
-		if (arts[n].status == ART_UNREAD && arts[n].thread != ART_EXPIRED)
+		if (arts[n].status != ART_READ && arts[n].thread != ART_EXPIRED)
 			return n;
 
 		n = prev_response(n);
@@ -1348,9 +1365,8 @@ thread_catchup(
 			break;
 	}
 
-/* FIXME do some NLS/snprintf work here - see equivalent code in group_catchup() */
 	if (i != -1) {				/* still unread arts in this thread */
-		sprintf(buf, _(txt_mark_thread_read), (ch == iKeyThreadCatchupNextUnread) ? _(txt_enter_next_thread) : "");
+		snprintf(buf, sizeof(buf), _(txt_mark_thread_read), (ch == iKeyThreadCatchupNextUnread) ? _(txt_enter_next_thread) : "");
 		if ((!TINRC_CONFIRM_ACTION) || (pyn = prompt_yn(cLINES, buf, TRUE)) == 1)
 			thd_mark_read(&CURR_GROUP, base[thread_basenote]);
 	}
@@ -1387,36 +1403,53 @@ thread_catchup(
 
 /*
  * This is the single entry point into the article pager
- * 'art' is the arts[art] we wish to read
- * ignore_unavail should be set if we wish to keep going after 'article
- * unavailable'.
+ * art
+ *		is the arts[art] we wish to read
+ * ignore_unavail
+ *		should be set if we wish to keep going after article unavailable
+ * level
+ *		is the menu from which we came. This should be only be GROUP or THREAD
+ *		it is used to set the return code to go back to the calling menu when
+ *		not explicitly set
  * Return:
  *	<0 to quit to group menu
  *	 0 to stay in thread menu
- * >0 after normal exit from pager to return to previous menu level
+ *  >0 after normal exit from pager to return to previous menu level
  */
 static int
 enter_pager(
 	int art,
-	t_bool ignore_unavail)
+	t_bool ignore_unavail,
+	int level)
 {
 	int i;
 
 again:
-	i = show_page(&CURR_GROUP, art, &thdmenu.curr);
-
-	switch (i) {
+	switch ((i = show_page(&CURR_GROUP, art, &thdmenu.curr))) {
 		/* These exit to previous menu level */
 		case GRP_QUIT:				/* 'Q' all the way out */
-		case GRP_RETURN:			/* 'T' back to select menu */
+		case GRP_RETSELECT:			/* 'T' back to select menu */
 		case GRP_NEXT:				/* 'c' Move to next thread on group menu */
 		case GRP_NEXTUNREAD:		/* 'C' */
 			break;
 
+		case GRP_ARTABORT:			/* user 'q'uit load of article */
+			/* break forces return to group menu */
+			if (level == GROUP_LEVEL)
+				break;
+			/* else stay on thread menu */
+			show_thread_page();
+			return 0;
+
 		/* Keeps us in thread menu */
-		case GRP_ARTFAIL:
+		case GRP_ARTUNAVAIL:
 			if (ignore_unavail && (art = next_unread(art)) != -1)
 				goto again;
+			else if (level == GROUP_LEVEL)
+				return GRP_ARTABORT;
+			/* back to thread menu */
+			show_thread_page();
+			return 0;
 
 		case GRP_GOTOTHREAD:		/* 'l' from pager */
 			show_thread_page();
@@ -1424,7 +1457,7 @@ again:
 
 		default:					/* >=0 normal exit, new basenote */
 			if (filtered_articles)
-				return GRP_KILLED; /* ?? set group cursor back to 0 and do nothing */
+				return GRP_KILLED; /* ? set group cursor back to 0 and do nothing */
 			fixup_thread(this_resp, FALSE);
 
 			if (currmenu != &grpmenu)	/* group menu will redraw itself */
@@ -1457,7 +1490,7 @@ thread_tab_pressed(
 	 */
 	for (i = n; i != -1; i = arts[i].thread) {
 		if ((arts[i].status == ART_UNREAD) || (arts[i].status == ART_WILL_RETURN))
-			return (enter_pager(i, TRUE));
+			return (enter_pager(i, TRUE, THREAD_LEVEL));
 	}
 
 	/*
@@ -1465,3 +1498,64 @@ thread_tab_pressed(
 	 */
 	return GRP_NEXTUNREAD;
 }
+
+
+/*
+ * If there are any tagged and unread articles, prompt user to mark either
+ * all tagged arts as read, or only current article, or cancel operation.
+ * Otherwise, use current article.
+ * Finally move to next unread article.
+ *
+ * Return GRP_EXIT if there are no more unread articles in this group,
+ * else return 0.
+ */
+static int
+mark_art_read(
+	struct t_group *group)
+{
+	char keytagged[MAXKEYLEN], keycurrent[MAXKEYLEN], keyquit[MAXKEYLEN];
+	int ch = iKeyMarkReadCur;
+	int n = -1, cnt = 0;
+	int tmp_num_of_tagged_arts = num_of_tagged_arts;
+
+	if (!tinrc.mark_ignore_tags && got_tagged_unread_arts()) {
+		ch = prompt_slk_response(iKeyMarkReadTag,
+				&menukeymap.mark_read_tagged_current,
+				_(txt_mark_art_read_tagged_current),
+				printascii(keytagged, map_to_local(iKeyMarkReadTag, &menukeymap.mark_read_tagged_current)),
+				printascii(keycurrent, map_to_local(iKeyMarkReadCur, &menukeymap.mark_read_tagged_current)),
+				printascii(keyquit, map_to_local(iKeyQuit, &menukeymap.mark_read_tagged_current)));
+	}
+
+	switch (ch) {
+		case iKeyMarkReadTag: /* mark tagged unread articles as read */
+			cnt = mark_tagged_read(group);
+			show_thread_page();
+			n = find_response(thread_basenote, thdmenu.curr);
+			break;
+
+		case iKeyMarkReadCur: /* mark current article as read */
+			n = find_response(thread_basenote, thdmenu.curr);
+			if ((arts[n].status == ART_UNREAD) || (arts[n].status == ART_WILL_RETURN)) {
+				art_mark(group, &arts[n], ART_READ);
+				build_tline(thdmenu.curr, &arts[n]);
+				draw_line(thdmenu.curr, MAGIC);
+			}
+			break;
+
+		case iKeyQuit: /* cancel operation */
+		case iKeyAbort:
+			return 0;
+			break;
+	}
+
+	if ((n = next_unread(n)) == -1)	/* no more unread articles */
+		return GRP_EXIT;
+	else
+		fixup_thread(n, TRUE);	/* We may be in the next thread now */
+
+	if (ch == iKeyMarkReadTag)
+		info_message(_(txt_marked_tagged_arts_as_read), cnt, tmp_num_of_tagged_arts, PLURAL(tmp_num_of_tagged_arts, txt_article));
+
+	return 0;
+}
