@@ -3,8 +3,34 @@
 # reads an article on STDIN, mails any copies if requied,
 # signs the article and posts it.
 #
-# 2002-04-05 <urs@tin.org>
-#            <marc@marcbrockschmidt.de>
+#
+# Copyright (c) 2002-2003 Urs Janssen <urs@tin.org>,
+#                         Marc Brockschmidt <marc@marcbrockschmidt.de>
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+# 1. Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+# 2. Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+# 3. The name of the author may not be used to endorse or promote
+#    products derived from this software without specific prior written
+#    permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS OR
+# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+# DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
+# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
+# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
+# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
+# IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+# POSSIBILITY OF SUCH DAMAGE.
+#
 #
 # TODO: - add pgp6 support
 #       - add debug mode which doesn't dele tmp-files and is verbose
@@ -13,7 +39,7 @@
 #       - check for ~/.newsauth and use username/password if found
 #
 # version Number
-my $version = "1.0.4";
+my $version = "1.1.0";
 
 # TODO: put into a "my %config('NNTPServer' => 'news', ... );" array
 my $NNTPServer	= 'news';		# your NNTP servers name
@@ -28,12 +54,12 @@ my $PGPVersion	= '2';			# Use 2 for 2.X, 5 for PGP > 2.X and GPG for GPG
 
 my $Interactive = 1;			# allow interactive usage
 
-my $sendmail	= '| /usr/sbin/sendmail -t'; # set to '' to disable mail-actions
+my $sendmail	= '| /usr/sbin/sendmail -i -t'; # set to '' to disable mail-actions
 
 my @PGPSignHeaders = ('From', 'Newsgroups', 'Subject', 'Control',
 	'Supersedes', 'Followup-To', 'Date', 'Sender', 'Approved',
-	'Message-ID', 'Reply-To', 'Cancel-Lock', 'Cancel-Key',
-	'Also-Control', 'Distribution');
+	'Message-ID', 'Reply-To', 'Cancel-Key', 'Also-Control',
+	'Distribution');
 my @PGPorderheaders = ('from', 'newsgroups', 'subject', 'control',
 	'supersedes', 'followup-To', 'date', 'organization', 'lines',
 	'sender', 'approved', 'distribution', 'message-id',
@@ -182,6 +208,9 @@ sub readarticle {
 				$$HeaderR{$currentheader} = "$1: $2";
 			} elsif (m/^[ \t]/o) {
 				$$HeaderR{$currentheader} .= $_;
+			} else {
+				chomp($_);
+				die ("'$_' is not a correct header-line");
 			}
 		} else {
 			push @$BodyR, $_;
@@ -307,7 +336,7 @@ sub getpgpcommand {
 		}
 	} elsif ($PGPVersion eq '5') {
 		if ($PathtoPGPPass) {
-			$PGPCommand = "PGPPASSFD=2 ".$pgp."s -u \"".$PGPSigner."\" -t --armor -o ".$pgptmpf.".txt.asc -z -f < ".$pgptmpf.".txt 2<".$PathtoPGPPass;
+			$PGPCommand = "PGPPASSFD=42 ".$pgp."s -u \"".$PGPSigner."\" -t --armor -o ".$pgptmpf.".txt.asc -z -f < ".$pgptmpf.".txt 42<".$PathtoPGPPass;
 		} elsif ($Interactive) {
 			$PGPCommand = $pgp."s -u \"".$PGPSigner."\" -t --armor -o ".$pgptmpf.".txt.asc -z -f < ".$pgptmpf.".txt";
 		} else {
@@ -315,7 +344,7 @@ sub getpgpcommand {
 		}
 	} elsif ($PGPVersion =~ m/GPG/io) {
 		if ($PathtoPGPPass) {
-			$PGPCommand = $pgp." --digest-algo MD5 -a -u \"".$PGPSigner."\" -o ".$pgptmpf.".txt.asc --no-tty --batch --passphrase-fd 2 2<".$PathtoPGPPass." --clearsign ".$pgptmpf.".txt";
+			$PGPCommand = $pgp." --digest-algo MD5 -a -u \"".$PGPSigner."\" -o ".$pgptmpf.".txt.asc --no-tty --batch --passphrase-fd 42 42<".$PathtoPGPPass." --clearsign ".$pgptmpf.".txt";
 		} elsif ($Interactive) {
 			$PGPCommand = $pgp." --digest-algo MD5 -a -u \"".$PGPSigner."\" -o ".$pgptmpf.".txt.asc --no-secmem-warning --no-batch --clearsign ".$pgptmpf.".txt";
 		} else {
@@ -471,7 +500,7 @@ tinews.pl - Post and sign an article via NNTP
 
 =head1 SYNOPSIS
 
-B<tinews.pl> < I<input>
+B<tinews.pl> E<lt> I<input>
 
 =head1 DESCRIPTION
 
@@ -492,11 +521,11 @@ The following exit values are returned:
 
 =over 4
 
-=item 0
+=item S< 0> 
 
 Successful completion.
 
-=item >0
+=item S<!=0>
 
 An error occurred.
 
@@ -528,12 +557,12 @@ From:-field will be used.
 =item B<$REPLYTO>
 
 Set the article header field Reply-To: to the return address specified by
-the variable if there isn't allready a Reply-To: header in the article.
+the variable if there isn't already a Reply-To: header in the article.
 
 =item B<$ORGANIZATION>
 
 Set the article header field Organization: to the contents of the variable
-if there isn't allready a Organization: header in the article.
+if there isn't already a Organization: header in the article.
 
 =back
 
@@ -567,8 +596,8 @@ security is an issue, don't use this script.
 
 =head1 NOTES
 
-B<tinews.pl> is designed to be used with B<pgp>(1)-2.6.3.
-B<pgp>(1)-5 and B<gpg>(1) may also work.
+B<tinews.pl> is designed to be used with B<pgp>(1)-2.6.3,
+B<pgp>(1)-5 and B<gpg>(1).
 
 B<tinews.pl> requires the following standard modules to be installed:
 B<Net::NNTP>(3pm), B<Time::Local>(3pm) and B<Term::Readline>(3pm).
